#!/bin/sh

read_ini_key()
{
    ini_file=$1
    ini_section=$2
    ini_key=$3

    ini_value=`sed -e 's/[[:space:]]*\=[[:space:]]*/=/g' \
        -e 's/[[:space:]]*$//' -e 's/^[[:space:]]*//' < $ini_file \
        | sed -n "/^\[$ini_section\]/,/^\s*\[/{/^[^;].*\=.*/p;}" | grep "^$ini_key" | cut -d'=' -f2`
}

relative_path=`dirname ${0}`
absolute_path=`cd $relative_path >/dev/null; pwd ; cd - >/dev/null`
COMMON_LDAP_PATH=$absolute_path

export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$COMMON_LDAP_PATH:$COMMON_LDAP_PATH/libs/:$COMMON_LDAP_PATH/libprotobuf/lib/

case "$1" in
start)
    if ! [ -e "${COMMON_LDAP_PATH}/log/" ]; then
        mkdir -p ${COMMON_LDAP_PATH}/log/
    fi
    chmod +w $COMMON_LDAP_PATH/krb5.conf

    # check the daemon boot mode
    for arg in $@; do
        [ "$arg" = "verbose" ] && verbose_from_cmd="--verbose"
        [ "$arg" = "--verbose" ] && verbose_from_cmd="--verbose"
    done

    if [ -z "$verbose_from_cmd" ]; then
        read_ini_key "${COMMON_LDAP_PATH}/LdapSetting.ini" "LDAP_Setting" "verbose"
        [ "${ini_value}" = "yes" ] && verbose="--verbose"
    fi

    if [ -z "${verbose}" ]; then
        echo "Start common ldap daemon..."
    else
        echo "Start common ldap daemon in verbose mode..."
    fi

    ${COMMON_LDAP_PATH}/create_krb5.sh --workdir ${COMMON_LDAP_PATH}

    export KRB5_CONFIG="${COMMON_LDAP_PATH}/krb5.conf"
    ${COMMON_LDAP_PATH}/AuthDaemon --workdir ${COMMON_LDAP_PATH} ${verbose} $@
    ;;

stop)
    echo "Stop common ldap daemon..."

    count=`ps -ef | grep "AuthDaemon --workdir" | grep -v "grep" | wc -l`
	[ "$count" -gt 0 ] && killall -SIGTERM AuthDaemon

    for ((i = 0; i < 10; i ++))
    do
		sleep 1

    	count=`ps -ef | grep "AuthDaemon --workdir" | grep -v "grep" | wc -l`
    	if [ "$count" -eq 0 ]; then
    		echo "all of authdaemon exit."
    		break
    	fi
	done

    [ "$count" -gt 0 ] && killall -9 AuthDaemon
    ;;

*)
    echo "Usage:"
    echo "  only support start/stop action"
    echo "  additional boot parameter chould be: verbose/--verbose/--process-num 2/--thread-num 10"
    ;;

esac

