#!/bin/sh

function is_cert_expire()
{
	certpath=$1
	
	if [ "##" != "##${certpath}" -a -f "${certpath}" ]; then
		expire_date=$( openssl x509 -noout -enddate -in $certpath | awk -F= '{ print $2 }' )
		expire_seconds=$( date --date="${expire_date}" +%s )
		
		now_seconds=$( date +%s )
		
		if [ $now_seconds -lt $expire_seconds ]; then
			return 0
		fi
	fi
	
	return 1
}

jscan_cert_dir="/etc/iscan/jscan/certificates/dynamic"
jscan_cert_status="/etc/iscan/jscan/certificates/certstatus.props"

UN="/usr/iwss/bin/uniqueName"
PEM_tmp_dir="/tmp/PEM_tmp_dir"

mkdir $PEM_tmp_dir

python /usr/iwss/bin/ca_converter.py &> /dev/null

for file in $PEM_tmp_dir/*
do
	is_cert_expire $file
	if [ $? -ne 0 ]; then
		continue
	fi

	MD5=`$UN $file`
	ls $jscan_cert_dir | grep $MD5 > /dev/null
	if [ $? -ne 0 ]; then
		dst=$jscan_cert_dir"/"$MD5".cer"
		openssl x509 -inform PEM -in $file -outform DER -out $dst
		chown iscan:iscan $dst
		echo $MD5".act = active" >> $jscan_cert_status
	fi
done

rm -rf $PEM_tmp_dir

## sync certificates to https scan dir
rm -f /etc/iscan/https/certificates/pem/*.*
rm -f /etc/iscan/https/certificates/active/*.*
rm -f /etc/iscan/https/certificates/inactive/*.*
su iscan /usr/iwss/certsynctool/certsync_tohttpsdir.sh > /dev/null 2>&1
su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/active /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/active
su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/inactive /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/inactive

# restart daemons
/etc/iscan/S99IScanHttpd restart
/etc/iscan/S99ISproxy reload
