#!/usr/bin/env python
# -*- coding: utf-8 -*-

logformat = [
    'CLF_CompanyID',
    'CLF_MsgType',
    'CLF_ProductID',
    'CLF_DeviceName',
    'CLF_UTCTimeStamp',
    'CLF_TimeZone',
    'CLF_UserName',
    'CLF_UserID',
    'CLF_GroupName',
    'CLF_Department',
    'CLF_Location',
    'CLF_URL',
    'CLF_Size',
    'CLF_AppID',
    'CLF_Channel',
    'CLF_TransportProtocol',
    'CLF_MimeType',
    'CLF_ClientIP',
    'CLF_ServerIP',
    'CLF_Domain',
    'CLF_FileName',
    'CLF_ProtocolAction',
    'CLF_Connection',
    'CLF_URLCategory',
    'CLF_CustomURLCategory',
    'CLF_AdvancedCategory',
    'CLF_OnlineTime',
    'CLF_MalwareName',
    'CLF_RuleName',
    'CLF_MatchedContent',
    'CLF_Action',
    'CLF_PolicyName',
    'CLF_Phase',
    'CLF_SessionPackets',
    'CLF_SessionBytes',
    'CLF_InByte',
    'CLF_OutByte',
    'CLF_WRSScore',
    'CLF_SessionStartTime',
    'CLF_SessionEndTime',
    'CLF_ActionTime',
    'CLF_Direction',
    'CLF_SourceInterface',
    'CLF_DestinationInterface',
    'CLF_SourcePort',
    'CLF_DestinationPort',
    'CLF_SourceZone',
    'CLF_DestinationZone',
    'CLF_IPSRule',
    'CLF_ErsCategory',
    'CLF_MailSender',
    'CLF_MailReceiver',
    'CLF_MailSubject',
    'CLF_RiskLevel',
    'CLF_ModuleID',
    'CLF_ErrorCode',
    'CLF_ErrorParameters',
    'CLF_TimeStamp',
    'CLF_ScanResult'
]

log_datatype = [
    'string',
    'int',
    'string',
    'string',
    'int',
    'int',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'int',
    'int',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'int',
    'int',
    'string',
    'int',
    'int',
    'string',
    'string',
    'string',
    'int',
    'string',
    'int',
    'bigint',
    'bigint',
    'bigint',
    'bigint',
    'int',
    'int',
    'int',
    'int',
    'int',
    'string',
    'string',
    'int',
    'int',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'string',
    'int',
    'int',
    'int',
    'string',
    'string',
    'int'

]

summary_map = {
          'facet_bandwidth_in':{
                 'index_name':'facet_bandwith',
                 'value':['CLF_DeviceName', 'CLF_UserName', 'CLF_AppID', 'CLF_ClientIP', 'CLF_PolicyName']
          },
          'facet_bandwidth_out':{
                  'index_name':'facet_bandwith',
                 'value':['CLF_DeviceName', 'CLF_UserName', 'CLF_AppID', 'CLF_ClientIP', 'CLF_PolicyName']

          },

          'facet_bandwidth_total':{
                  'index_name':'facet_bandwith',
                 'value':['CLF_DeviceName', 'CLF_UserName', 'CLF_AppID', 'CLF_ClientIP', 'CLF_PolicyName']
          },

          'facet_policy_enforce':{
                  'index_name':'facet_policy_enforce',
                  'value':['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_AppID', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName', 'CLF_URLCategory', 'CLF_CustomURLCategory', 'CLF_RuleName']
          },


          'facet_internet_access':{
               'index_name':'facet_internet_access',
               'value':['CLF_DeviceName', 'CLF_UserName',  'CLF_ClientIP',  'CLF_Domain', 'CLF_URLCategory', 'CLF_CustomURLCategory']
          },
          'facet_internet_security':{
              'index_name':'facet_internet_security',
              'value':['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_MalwareName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName']
           },
           'facet_access_control':{
               'index_name':'facet_access_control',
               'value': ['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName']
           },
           'facet_data_security':{
              'index_name':'facet_data_security',
              'value':['CLF_DeviceName', 'CLF_UserName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName', 'CLF_RuleName']
           },
           'module_id':{
               'index_name':'module_id',
               'value':['CLF_ModuleID','CLF_UserName','CLF_Domain']
           },
           'error_code':{
               'index_name':'error_code',
               'value':['CLF_ModuleID','CLF_ErrorCode']
           },

}


# creating summary index for log query, so summary index should
# reference facet stats definate schema
summary_indx = {
                'facet_bandwith':{
                     'compound_index':['CLF_DeviceName', 'CLF_UserName', 'CLF_AppID', 'CLF_ClientIP', 'CLF_PolicyName'],

                     'CLF_MsgType': ['65537']
                 },

                'facet_policy_enforce':{
                      'compound_index': ['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_AppID', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName', 'CLF_URLCategory', 'CLF_CustomURLCategory', 'CLF_RuleName'],
                       'CLF_MsgType': ['65538', '524289', '720897', '327681']
                },

                'facet_internet_access': {
                       'compound_index':['CLF_DeviceName', 'CLF_UserName',  'CLF_ClientIP',  'CLF_Domain', 'CLF_URLCategory', 'CLF_CustomURLCategory'],
                       'CLF_MsgType': ['131073']
                 },

                'facet_internet_security':{
                     'compound_index':['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_MalwareName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName'],
                     'CLF_MsgType': ['196609', '196610', '196616', '589825', '196612', '458753', '720898', '262145', '196624', '851969']
                },

                'facet_data_security':{
                    'compound_index':['CLF_DeviceName', 'CLF_UserName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName', 'CLF_RuleName'],
                    'CLF_MsgType': ['393217']
                 },
                'facet_access_control':{
                    'compound_index':['CLF_DeviceName', 'CLF_MsgType', 'CLF_UserName', 'CLF_Channel', 'CLF_ClientIP', 'CLF_Action', 'CLF_PolicyName'],
                    'CLF_MsgType': ['655361', '786433', '786434']
                 },
                 'module_id':{
                     'compound_index':['CLF_ModuleID','CLF_UserName','CLF_Domain'],
                     'CLF_MsgType':['131073']
                 },
                 'error_code':{
                     'compound_index':['CLF_ModuleID','CLF_ErrorCode'],
                     'CLF_MsgType':['131073']
                 },

}


search_indx = {
            'CLF_ClientIP': 'CLF_ClientIP',
            'CLF_DeviceName': 'CLF_DeviceName',
            'CLF_Domain':'CLF_Domain',
            'CLF_MalwareName': 'CLF_MalwareName',
            'CLF_PolicyName':'CLF_PolicyName',
            'CLF_RuleName': 'CLF_RuleName',
            'CLF_UserName':'CLF_UserName',
            'CLF_CustomURLCategory': 'CLF_CustomURLCategory'
}

#summary_indx = {
#            'CLF_Action':'CLF_Action',
#            'CLF_AppID':'CLF_AppID',
#            'CLF_Channel': 'CLF_Channel',
#            'CLF_ClientIP': 'CLF_ClientIP',
#            'CLF_DeviceName': 'CLF_DeviceName',
#            'CLF_Domain':'CLF_Domain',
#            'CLF_GroupName':'CLF_GroupName',
#            'CLF_MalwareName': 'CLF_MalwareName',
#            'CLF_MimeType': 'CLF_MimeType',
#            'CLF_MsgType': 'CLF_MsgType',
#            'CLF_PolicyName':'CLF_PolicyName',
#            'CLF_RuleName': 'CLF_RuleName',
#            'CLF_ServerIP': 'CLF_ServerIP',
#            'CLF_UserName':'CLF_UserName',
#            'CLF_URLCategory': 'CLF_URLCategory',
#            'CLF_CustomURLCategory': 'CLF_CustomURLCategory'
#}

if __name__ == '__main__':
    pass

