#!/usr/bin/python
#coding:UTF-8

import os, sys
import shutil
import time
import codecs
import re
from mig_python_logger import logger

separator = r'(?<!\\),'


#get  Policy id of "FTP DLP Policy" from the table tb_policy
#IWSVA 5.6 hotfix 1418 introduce DLP module supports FTP traffic.
def get_FTP_DLP_policy_id(src_policy_file, policy_name):
    logger.debug("in [get_FTP_DLP_policy_id] get " + policy_name +  " id ")
    FTP_DLP_policy_id = []
    policy_fd,coding = open_file(src_policy_file, "r")

    line = policy_fd.readline()
    while line:
        elements = re.split(separator, line)
        if  re.search(str(policy_name), elements[1]) != None:
            FTP_DLP_policy_id.append(elements[0])
            logger.debug("in [FTP_DLP_policy_id] " + policy_name + " id is: " + elements[0])
            break
        line = policy_fd.readline()
    policy_fd.close()
    return FTP_DLP_policy_id

def open_file(file_name, flag):

    if not os.path.exists(file_name):
        logger.error("file not exit: " + file_name)
        return None
    real_encoding = None
    encodings = ('utf-16', 'utf-8', 'gb2312')
    for encoding in encodings:
        fd = codecs.open(file_name, flag, encoding)
        try:
            fd.readline()
        except UnicodeError:
            fd.close()
        else:
            real_encoding = encoding
            fd.close()
            break
    if not real_encoding:
        logger.error("what's the file encoding? i cannot figure out")
        return None
    fd = codecs.open(file_name, flag, real_encoding)
    return fd, real_encoding


def handle_rule_file(src_rule_file):

    logger.debug("in [handle_rule_file] handle rule file: " + src_rule_file)

    rule_id = []
    dst_rule_file = src_rule_file + "_temp"

    src_rule_fd,coding = open_file(src_rule_file, "r")
    new_fd = codecs.open(dst_rule_file, "w", coding)

    line = src_rule_fd.readline()
    while line:
        #add "apt and ccca"
        if re.search('''phishing_action''',line) != None:
            new_element="anti_apt=no|apt_action=2|anti_cnc=yes|\n"
            line_new=line.rstrip('\n') + new_element
            new_fd.write(line_new)
        else:
            new_fd.write(line)
        elements = re.split(separator, line)
        if len(elements) > 1:
            rule_id.append(elements[0])

        line = src_rule_fd.readline()

    src_rule_fd.close()
    
     #new rule for dlp
    app_policy_rule = ['RULE_ID,POLICY_ID,247,dlp_templates=|filesize_limit=30']

    dlp_rule = app_policy_rule 
    
    unused_rule_id = []
    
    for i in range(1, len(rule_id) + len(dlp_rule) + 2):
        if str(i) in rule_id:
            continue
        unused_rule_id.append(str(i))
        
    if len(dlp_rule) > len(unused_rule_id):
        print "Error: cannot get enough unused rule id... ..."
        return -1
    
    policy_id = alloc_dlp_id 
    index = 0
    #replace the policy with unique id
    while index < len(dlp_rule):
        temp_rule = dlp_rule[index]
        temp_rule = temp_rule.replace('RULE_ID', unused_rule_id[0])
        unused_rule_id.remove(unused_rule_id[0])
        temp_rule = temp_rule.replace('POLICY_ID', policy_id[0])
        logger.debug("In [handle_rule_file] append policy:" + temp_rule)
        new_fd.write(temp_rule + '\n')
        
        index += 1
        policy_id.remove(policy_id[0])
    
    new_fd.close()

    logger.debug("In [handle_rule_file] copy file: " + dst_rule_file + "to file: " + src_rule_file)
    shutil.copy2(dst_rule_file, src_rule_file)
    return 0

#find all the policy id in tb_policy.csv
#generate 1 new id that not appears in the tb_policy.csv
#apend 1 new policy to tb_policy.csv using the 1 new id
#return the new id
def handle_policy_file(src_policy_file):
    logger.debug("in [handle_policy_file] handle file: " + src_policy_file)
    # get all policy id
    policy_id = []    
    dst_policy_file = src_policy_file + '_temp'
    policy_fd,coding = open_file(src_policy_file, "r")
    new_fd = codecs.open(dst_policy_file, "w", coding)
    
    line = policy_fd.readline()
    while line:
        new_fd.write(line)
        elements = re.split(separator, line)
        if len(elements) > 1:
            policy_id.append(elements[0])
        line = policy_fd.readline()        
    #add the content of ftp dlp policy
    policy_fd.close()
    dlp_policy = ['POLICY_ID,FTP DLP Scan Global Policy,11,1,1,2013-05-07 13:53:41.196835+08,2013-05-07 13:53:41.196835+08,FTP DLP Scan Default Policy']

    unused_policy_id = []
    for i in range(1, len(policy_id) + len(dlp_policy) + 2):
        if str(i) in policy_id:
            continue
        unused_policy_id.append(str(i))

    if len(dlp_policy) > len(unused_policy_id):
        print "Error: cannot get enought unused policy id... ..."
        return None

    alloc_policy_id = []
    alloc_dlp_id = []
    
    for temp_policy in dlp_policy:
            temp_policy = temp_policy.replace('POLICY_ID', unused_policy_id[0])
            logger.debug("in [handle_policy_file] append policy: " + temp_policy)
            new_fd.write(temp_policy + '\n')

            alloc_policy_id.append(unused_policy_id[0])
            
            if temp_policy.find("DLP") != -1:
                alloc_dlp_id.append(unused_policy_id[0])
                
            unused_policy_id.remove(unused_policy_id[0])
            
    new_fd.close()
    shutil.copy2(dst_policy_file, src_policy_file)
    return alloc_dlp_id

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print "Usage: " + sys.argv[0]+ " src_tb_rule.csv dst_tb_rule.csv"
        logger.error(sys.argv[0] + ": Missing the parameter")

    else:

        src_rule_file = sys.argv[1]
        dst_rule_file = sys.argv[2]

        src_dirname = os.path.dirname(sys.argv[1])
        dst_dirname = os.path.dirname(sys.argv[2])

        src_policy_file = os.path.join(src_dirname, "tb_policy.csv")
        src_policytype_file = os.path.join(src_dirname, "tb_policytype.csv")
        
        if not os.path.exists(src_rule_file):
            logger.error('Execute [db_table_convert] not find the file:' + src_rule_file)
        
        if not os.path.exists(src_policy_file):
            logger.error('Execute [db_table_convert] not find the file:' + src_policy_file)
        
        if not os.path.exists(src_policytype_file):
            logger.error('Execute [db_table_convert] not find the file:' + src_policytype_file)

        logger.debug('entry into [db_table_convert_5.6_to_6.0.py] ... ...')

        #backup 3 origin policy files that related with policy setting
        shutil.copy2( src_rule_file, src_rule_file + ".backup")
        shutil.copy2( src_policy_file, src_policy_file + ".backup")
        shutil.copy2( src_policytype_file, src_policytype_file + ".backup")

        policy_type_file = os.path.join(src_dirname, "tb_policytype.csv")
        ftp_dlp_policy_name = 'FTP DLP Scan Global Policy'
        FTP_DLP_policy_id=get_FTP_DLP_policy_id(src_policy_file, ftp_dlp_policy_name)

        if FTP_DLP_policy_id:
             logger.debug("Find FTP DLP Scan Default Policy id")

        else:
            logger.debug('in [main] handle_policy_file... ...')
            logger.debug('Can NOT find FTP DLP Scan Default Policy id ')
            alloc_dlp_id = handle_policy_file(src_policy_file)

            if alloc_dlp_id == None :
                logger.error("Error: the policy id is illegal... ...")
            else:
                logger.debug('in [main] handle_rule_file... ...')
                handle_rule_file(src_rule_file)


                # update the tb_policytype.csv, append the ftp dlp policy type
                logger.debug('in [main] append FTP DLP Policy type... ...')
                new_fd = open(policy_type_file, "a");
                new_fd.write('11,FTP DLP Policy,30,0,This policy_type has to be exactly the same as [policy].ftp_policy_type_id in IWSSPIDlpFilter.dsc file.\n')
                new_fd.close()

        logger.debug("in [main] copy file: " + src_rule_file + " to file: " + sys.argv[2])
        shutil.copy2(src_rule_file, dst_rule_file)
        logger.debug("in [main] copy file: " + src_policy_file + " to file: " + os.path.join(dst_dirname, "tb_policy.csv"))
        shutil.copy2(src_policy_file, os.path.join(dst_dirname, "tb_policy.csv"))
        logger.debug("in [main] copy file: " + policy_type_file + " to file: " + os.path.join(dst_dirname, "tb_policytype.csv"))
        shutil.copy2(policy_type_file, os.path.join(dst_dirname, "tb_policytype.csv"))


        logger.debug('exit[db_table_convert_5.6_to_6.0.py] ... ...')

