#!/bin/sh
# Author: Jeremy Yang
# This script is used to add, delete, activate, inactivate certificates manipulated thru UI by customers
# $1: original DER file
# $2: dst PEM dir
# $3: dst active hash index dir
# $4: dst inactive hash index dir
# $5: cert operation
# $6: cert status for add or del

log()
{
	echo "`date`" $@ >>/etc/iscan/tmp/certsync.log
}

if [ $# -ne 6 ]
then
	log "There is too many or too few arguments"
	exit
fi

if [ ! -d $2 ] || [ ! -d $3 ] || [ ! -d $4 ]
then
	log "$2, $3, $4 must be directories"
	exit
fi

log $@

if [ $5 != "add" ] && [ $5 != "del" ] && [ $5 != "activate" ] && [ $5 != "inactivate" ]
then
	log "$5 must be valid operation"
	exit
fi

if [ $6 != "active" ] && [ $6 != "inactive" ]
then
	log "$6 must be active or inactive"
	exit
fi

pemfile=${1##*/}
pemfile=${pemfile%.*}.pem

log "pemfile" $pemfile

case $5 in
	add)
		if [ ! -f $1 ]
		then
			log "$1 must be exist file"
			exit
		fi

		is_CA=`openssl x509 -inform DER -in $1 -purpose -noout | grep "Any Purpose CA" | awk -F" : " '{ print $2 }'`
		if [ $is_CA != "Yes" ]
		then
			log "$1 is not a CA certificate"
			exit
		fi

		if [ -f $2/$pemfile ]
		then
			log "the cert to be added already exists"
		else
			log create $2/$pemfile
			openssl x509 -inform DER -in $1 -outform PEM -out $2/$pemfile
		fi

		hash=`openssl x509 -hash -noout -in $2/$pemfile`
		log hash=$hash

		case $6 in
			active)
				cd $3

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`

						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $correct_link_file_name
					rm -f $correct_link_file_name

					log ln -sf $2/$pemfile $3/$correct_link_file_name
					ln -sf $2/$pemfile $3/$correct_link_file_name
				else
					log ln -sf $2/$pemfile $3/${hash}.$nextindexno
					ln -sf $2/$pemfile $3/${hash}.$nextindexno
				fi

				cd $4

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`

						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`

						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $correct_link_file_name
					rm -f $correct_link_file_name
				fi
				;;
			inactive)
				cd $4

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`

						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $correct_link_file_name
					rm -f $correct_link_file_name

					log ln -sf $2/$pemfile $4/$correct_link_file_name
					ln -sf $2/$pemfile $4/$correct_link_file_name
				else
					log ln -sf $2/$pemfile $4/${hash}.$nextindexno
					ln -sf $2/$pemfile $4/${hash}.$nextindexno
				fi

				cd $3

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`

						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $correct_link_file_name
					rm -f $correct_link_file_name
				fi
				;;
		esac
		;;
	del)
		if [ ! -f $2/$pemfile ]
		then
			log "the cert to be deleted is not exist"
			exit
		else
			hash=`openssl x509 -hash -noout -in $2/$pemfile`
			log hash=$hash

			cd $3

			same_hash_link_files=`ls ${hash}.*`
			log same_hash_link_files=$same_hash_link_files

			nextindexno=0
			correct_link_file_name="###"
			if [ ! -z "$same_hash_link_files" ]
			then
				for link_file in $same_hash_link_files
				do
					log link_file=$link_file

					link_file_full_info=`ls -l $link_file`
					log link_file_full_info=$link_file_full_info

					indexno=`echo $link_file | awk -F"." '{ print $2 }'`

					if [ $indexno -gt $nextindexno ]
					then
						nextindexno=$indexno
					fi

					dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
					if [ $dstfile = $2/$pemfile ]
					then
						correct_link_file_name=$link_file
						break
					fi
				done

				(( nextindexno = nextindexno + 1 ))
			fi

			log "correct_link_file_name" $correct_link_file_name

			if [ "###" != $correct_link_file_name ]
			then
				log rm -f $correct_link_file_name
				rm -f $correct_link_file_name

				remain_hash_link_files=`ls ${hash}.*`
				remain_hash_link_len=`ls ${hash}.* | wc -l`

				log $remain_hash_link_files

				new_hash_index=0
				if [ -n "$remain_hash_link_files" ]; then
					for remain_hash in $remain_hash_link_files; do
						mv $remain_hash ${hash}.${new_hash_index}
						(( new_hash_index = new_hash_index + 1 ))
					done
				fi
			fi

			cd $4

			same_hash_link_files=`ls ${hash}.*`
			log same_hash_link_files=$same_hash_link_files

			nextindexno=0
			correct_link_file_name="###"
			if [ ! -z "$same_hash_link_files" ]
			then
				for link_file in $same_hash_link_files
				do
					log link_file=$link_file

					link_file_full_info=`ls -l $link_file`
					log link_file_full_info=$link_file_full_info

					indexno=`echo $link_file | awk -F"." '{ print $2 }'`

					if [ $indexno -gt $nextindexno ]
					then
						nextindexno=$indexno
					fi

					dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
					if [ $dstfile = $2/$pemfile ]
					then
						correct_link_file_name=$link_file
						break
					fi
				done

				(( nextindexno = nextindexno + 1 ))
			fi

			log "correct_link_file_name" $correct_link_file_name

			if [ "###" != $correct_link_file_name ]
			then
				log rm -f $correct_link_file_name
				rm -f $correct_link_file_name

				remain_hash_link_files=`ls ${hash}.*`
				remain_hash_link_len=`ls ${hash}.* | wc -l`

				log $remain_hash_link_files

				new_hash_index=0
				if [ -n "$remain_hash_link_files" ]; then
					for remain_hash in $remain_hash_link_files; do
						log $remain_hash ${hash}.${new_hash_index}
						mv $remain_hash ${hash}.${new_hash_index}

						(( new_hash_index = new_hash_index + 1 ))
					done
				fi
			fi

			log rm -f $2/$pemfile
			rm -f $2/$pemfile
		fi
		;;
	activate)
		if [ $6 != "inactive" ]
		then
			log "the cert to be activated should be inactive"
			exit
		else
			if [ ! -f $2/$pemfile ]
			then
				log "the cert $2/$pemfile to be activated is not exist"
				exit
			else
				hash=`openssl x509 -hash -noout -in $2/$pemfile`
				log hash=$hash

				cd $4

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file_full_info | awk -F"." '{ print $2 }'`
						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $4/$correct_link_file_name
					rm -f $4/$correct_link_file_name
				else
					log "the cert to be activated is not inactive"
				fi

				cd $3

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file_full_info | awk -F"." '{ print $2 }'`
						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=$link_file
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $3/$correct_link_file_name
					rm -f $3/$correct_link_file_name

					log ln -sf $2/$pemfile $3/$correct_link_file_name
					ln -sf $2/$pemfile $3/$correct_link_file_name
				else
					log ln -sf $2/$pemfile $3/${hash}.$nextindexno
					ln -sf $2/$pemfile $3/${hash}.$nextindexno
				fi
			fi
		fi
		;;
	inactivate)
		if [ $6 != "active" ]
		then
			log "the cert to be inactivated should be active"
			exit
		else
			if [ ! -f $2/$pemfile ]
			then
				log "the cert $2/$pemfile to be inactivated is not exist"
				exit
			else
				hash=`openssl x509 -hash -noout -in $2/$pemfile`
				log hash=$hash

				cd $3

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`
						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=`echo $link_file_full_info | awk -F" " '{ print $9 }'`
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $3/$correct_link_file_name
					rm -f $3/$correct_link_file_name
				else
					log "the cert to be inactivated is not active"
				fi

				cd $4

				same_hash_link_files=`ls ${hash}.*`
				log same_hash_link_files=$same_hash_link_files

				if [ -z $same_hash_link_files ]
				then
					nextindexno=0
					correct_link_file_name="###"
				else
					nextindexno=0
					correct_link_file_name="###"

					for link_file in $same_hash_link_files
					do
						log link_file=$link_file

						link_file_full_info=`ls -l $link_file`
						log link_file_full_info=$link_file_full_info

						indexno=`echo $link_file | awk -F"." '{ print $2 }'`
						if [ $indexno -gt $nextindexno ]
						then
							nextindexno=$indexno
						fi

						dstfile=`echo $link_file_full_info | awk -F" " '{ print $(NF) }'`
						if [ $dstfile = $2/$pemfile ]
						then
							correct_link_file_name=`echo $link_file_full_info | awk -F" " '{ print $9 }'`
						fi
					done

					(( nextindexno = nextindexno + 1 ))
				fi

				log "correct_link_file_name" $correct_link_file_name

				if [ "###" != $correct_link_file_name ]
				then
					log rm -f $4/$correct_link_file_name
					rm -f $4/$correct_link_file_name

					log ln -sf $2/$pemfile $4/$correct_link_file_name
					ln -sf $2/$pemfile $4/$correct_link_file_name
				else
					log ln -sf $2/$pemfile $4/${hash}.$nextindexno
					ln -sf $2/$pemfile $4/${hash}.$nextindexno
				fi
			fi
		fi
		;;
	*)
		log "script should never run here"
		;;
esac
