#!/bin/sh
jscan_cert_dir="/etc/iscan/jscan/certificates/dynamic"
jscan_cert_status="/etc/iscan/jscan/certificates/certstatus.props"

PEM_certs="import_tmp"
UN="/usr/iwss/bin/uniqueName"
PEM_tmp_dir="PEM_tmp_dir"
DER_tmp_dir="DER_tmp_dir"

begin="-----BEGIN CERTIFICATE-----"
end="-----END CERTIFICATE-----"

if [ $# -ne 1 ] || [ ! -f $1 ]
then
    echo "Usage: $0 cert_file"
    echo "Only support PKCS7 DER format"
    exit 1
fi

mkdir $PEM_tmp_dir

# convert PKCS7 DER format to X509 PEM
openssl pkcs7 -print_certs -inform DER -in $1 -out $PEM_certs > /dev/null 2>&1

if [ $? -ne 0 ]
then
    echo "Error: Can't load certificate, please check."
    echo "Only support PKCS7 DER format"
    exit 2
fi

# divise certificates and sync them to jscan dir
while :
do
    ((++i))

    begin_line=`sed -n "/$begin/=" $PEM_certs | head -1`
    end_line=`sed -n "/$end/=" $PEM_certs | head -1`

    if [ -z $begin_line ] || [ -z $end_line ]
    then
        break
    fi

    sed -n "${begin_line}, ${end_line}p" $PEM_certs > $PEM_tmp_dir"/"$i
    sed -i "${begin_line}, ${end_line}d" $PEM_certs

    MD5=`$UN $PEM_tmp_dir"/"$i`

    ls $jscan_cert_dir | grep $MD5 > /dev/null
    if [ $? -ne 0 ]
    then
        dst=$jscan_cert_dir"/"$MD5".cer"
        openssl x509 -inform PEM -in $PEM_tmp_dir"/"$i -outform DER -out $dst
        chown iscan:iscan $dst
        echo $MD5".act = active" >> $jscan_cert_status
    fi

done

rm -rf $PEM_tmp_dir
rm -f $PEM_certs

# sync certificates to https scan dir
rm -f /etc/iscan/https/certificates/pem/*.*
rm -f /etc/iscan/https/certificates/active/*.*
rm -f /etc/iscan/https/certificates/inactive/*.*
su iscan /usr/iwss/certsynctool/certsync_tohttpsdir.sh > /dev/null 2>&1
su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/active /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/active
su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/inactive /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/inactive

# restart daemons
# /etc/iscan/S99IScanHttpd restart
/etc/iscan/S99ISproxy reload

sleep 5

echo "Import certificates done"
exit 0
