#!/bin/sh
# Script for IWSVA 3.1 network
export PATH="/bin:/sbin:/usr/bin:/usr/sbin"
debug=0
debug_dev=/dev/stderr
NETINI=/etc/iscan/network.ini
NETINIDEF="/etc/iscan/network.ini.default"
ISCANINI=/etc/iscan/intscan.ini
HTTPSDECRYPTINI=/etc/iscan/IWSSPIUrlFilter.dsc
LIB_DIR=/opt/TrendMicro/Common/lib
CFGNETWORK=/etc/sysconfig/network
bridgemode=0
INT=eth0
EXT=eth1
OTH=""
port_group=""
BRI=br0
BRII=br1
DUELINT=eth2
DUELEXT=eth3
MGMT=eth2
separate_mgmt="no"
enable_stp="no"

PROXY=eth0
PROXY_ALL=eth0
PROXY_IN=
PROXY_EX=

VID=0
verbose=0
IWSS_DIR=/usr/iwss
DEVCONF=$LIB_DIR/devconf
MODCONF=/etc/modprobe.conf
HTTP_PID_FILE=/var/iwss/pids/iwssd.pid
FTP_PID_FILE=/var/iwss/pids/IWSSFtpMain.pid

# for I18N
export TEXTDOMAIN="tm_utils"

if [ -f /etc/iscan/iwsvanetwork.ini ]; then
	. /etc/iscan/iwsvanetwork.ini
fi

. /usr/iwss/iwsvanetkmod.sh

if [ "$debug" -eq 1 ]; then
	# for testing
	cfgdir="."
else
	cfgdir="/etc/sysconfig/network-scripts"
fi

#debug log
if [ -n "$file" -a "x$file" = "x1" ]; then
	debug_dev="/tmp/debug.log"
fi

# global variable definitions
if [ $debug -eq 0 ]; then
	INITD_DIR=/etc/init.d
else
	if [ -z "$INITD_DIR" ]; then
		INITD_DIR=`pwd`
	fi
fi

NETSH_ORG=$INITD_DIR/network
NETSH_SYS=$INITD_DIR/network_sys
IWSVANETSH=$IWSS_DIR/iwsvanetwork.sh
MAIL_CONF=/opt/trend/imss/config/imss.ini

# default value of conntrack module
CT_MAX=2097152
CT_HTABLE_SIZE=209715
CT_REDIR_MAX=0

# load iwsva kernel mode just once
load_conntrack_kmod ()
{
	ret=0

	lsmod | grep -q "netbase"
	if [ $? -eq 1 ]; then

		get_param $NETINI ct_htable_size
		if [ "$((gvalue))" -gt 0 ]; then
			CT_HTABLE_SIZE=$gvalue
		fi
		
		modprobe ipv6 2> /dev/null > /dev/null
		
		disable_ipv6=$(/usr/bin/iniconfig -r -f $NETINI -s network -o disable_ipv6)
		
		if [ "x$disable_ipv6" == "x0" ];then
			sysctl -q -w net.ipv6.conf.all.disable_ipv6=0
			sysctl -q -w net.ipv6.conf.default.disable_ipv6=0
        else
			sysctl -q -w net.ipv6.conf.all.disable_ipv6=1
			sysctl -q -w net.ipv6.conf.default.disable_ipv6=1
		fi
		
		# kernel modules for traffic redirection
		insmod $LIB_DIR/netbase.ko 2>/dev/null >/dev/null
		insmod $LIB_DIR/conntrack.ko ct_htable_size=$CT_HTABLE_SIZE 2>/dev/null >/dev/null
		
		ret=$?
		dbg 2 "Load IWSVA kernel modules (ret=$ret)"
		# alway insert ip_conntrack_ftp.ko for supporting passive mode ftp
	fi
	
	load_ip_conntrack_ftp
	return $ret;
}

set_kernel_params()
{
	if [ ! -e /proc/conntrack/ct_max ] ; then
		return 1;
	fi

	get_param $NETINI ct_max
	if [ "$((gvalue))" -gt 0 ]; then
		CT_MAX=$gvalue
	fi
	echo $CT_MAX > /proc/conntrack/ct_max 2> /dev/null
	if [ ! -e /proc/conntrack/ct_redir_max ] ; then
		return 1;
	fi
	get_param $NETINI ct_redir_max
	if [ "$((gvalue))" -gt 0 ]; then
		CT_REDIR_MAX=$gvalue
	fi
	echo $CT_REDIR_MAX > /proc/conntrack/ct_redir_max 2> /dev/null
	return 0
}

# F_ini_get_https_decrypting
# Get the value of a key in an INI file
# Usage: Catch the value using an variable.
# e.g. VAR=`F_ini_get_https_decrypting "file" "section" "key"`
#
# param: full path to an INI file
# param: section name excluding '[' ']'
# param: key name
#
# print: the ini value or error
#
F_ini_get_https_decrypting()
{
  INI_F="$1"
  INI_SECTION="$2"
  INI_KEY="$3"

  if [ ! -f "$INI_F" ] ; then
    echo "Error: ${INI_F} not found."
    return 1
  fi

  RANGE_START="/^\[${INI_SECTION}\]/"
  RANGE_END="/^\[.*\]/"

  MY_SEDCMD_1="${RANGE_START},${RANGE_END}p"
  #MY_SEDCMD_2="/${INI_KEY}=/p"
  MY_SEDCMD_2="/^${INI_KEY}=/p"

  #_R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} | sed -n -e "${MY_SEDCMD_2}" | awk -F= '{ printf $2 }'`
  _R=`sed -n -e "${MY_SEDCMD_1}" ${INI_F} | sed -n -e "${MY_SEDCMD_2}" | cut -d'=' -f2-`
  printf "%s\n" "$_R"
  if [ -z "$_R" ] ; then
    return 1
  else
    return 0
  fi
}

PROC_RPOLICY=/proc/conntrack/rpolicy
PROC_FTP_PORTS=/proc/conntrack/ftp_ports

load_redirect_rules ()
{
	if [ ! -f $PROC_RPOLICY ]; then
		dbg 2 "The $PROC_RPOLICY isn't existed"
		return 1
	fi

	load_ifname

	if [ "x$bridgemode" = "x0" ]; then
		echo > $PROC_RPOLICY
		return 0
	fi

	#check bypass setting
	get_param $ISCANINI lanbypass
	if [ "$gvalue" = "rpolicy_bypass" ]; then
		return 0
	fi

	RHTTP_PORTS="`cat /etc/iscan/IWSSPIProtocolHttpProxy.pni | grep interested_port | cut -d'=' -f2`"
	RFTP_PORTS="`cat /etc/iscan/IWSSPIProtocolFtp.pni | grep interested_port | cut -d'=' -f2`"
	RHTTPS_PORTS="`cat /etc/iscan/IWSSPIProtocolHttpProxy.pni | grep interested_https_port | cut -d'=' -f2`"
    RHTTPS_UPSTREAM_PORTS="`cat /etc/iscan/IWSSPIProtocolHttpProxy.pni | grep upstream_proxy_port | cut -d'=' -f2`"

    #add support for smtp/pop3
    isMailEnabled=`cat /opt/trend/imss/config/imss.ini | grep mail_enabled | cut -d'=' -f2 | tr -d ' '`
    if [ "yes" = "$isMailEnabled" ] ; then
	    RSMTP_PORTS="`cat /opt/trend/imss/config/imss.ini | grep proxy_port | cut -d'=' -f2 | tr -s '\n' ' ' `"
    fi

	get_param $ISCANINI app_control_notification_for_https
	if [ "$gvalue" = "yes" ]; then
		NOTIFICATION_FOR_HTTPS="yes"
    else
        NOTIFICATION_FOR_HTTPS="no"
	fi
	
	get_param $ISCANINI app_control_notification
	if [ "$gvalue" = "yes" ]; then
		APP_NOTIFICATION="yes"
    else
        APP_NOTIFICATION="no"
	fi

#	echo RHTTP_PORTS=$RHTTP_PORTS
#	echo RHTTPS_PORTS=$RHTTPS_PORTS

    isHttpsDecrypt=`F_ini_get_https_decrypting $HTTPSDECRYPTINI "https-decrypting" "enabled"`
	isAppControl=`F_ini_get_https_decrypting $ISCANINI "app-control" "enable"`
    if [ "yes" != "$isHttpsDecrypt" ] ; then
		if [ "yes" = "$isAppControl" -a "yes" = "$APP_NOTIFICATION" -a "yes" = "$NOTIFICATION_FOR_HTTPS" ] ; then
			redirect_pots=`echo "$RHTTP_PORTS $RFTP_PORTS $RHTTPS_PORTS  $RSMTP_PORTS"| tr -s ';' ' '`		
		else
			redirect_pots=`echo "$RHTTP_PORTS $RFTP_PORTS  $RSMTP_PORTS"| tr -s ';' ' '`
		fi
    else
            redirect_pots=`echo "$RHTTP_PORTS $RFTP_PORTS $RHTTPS_PORTS  $RSMTP_PORTS"| tr -s ';' ' '`
    fi

#	redirect_pots=`echo "$RHTTP_PORTS $RFTP_PORTS"| tr -s ';' ' '`
	ftp_pots=`echo "$RFTP_PORTS"| tr -s ';' ' '`

	tmp_rp="/tmp/tmrp.rules"
    tmp_ftp_ports="/tmp/tmpftpports.rules"
	pre_rp_rules_create "$tmp_rp" "$redirect_pots"
	echo ' ' >> $tmp_rp
	sudo cat $tmp_rp > $PROC_RPOLICY 2>/dev/null

	# the maximun of rpolicy table is 128
	# if the redirect policy table is full, it will return 207 error
	if [ $? -eq 207 ]; then
		i18n_rport_outrange=`gettext 'The number of ports exceeds the maximum value 128.'` ;
		echo "$i18n_rport_outrange"
	fi

	rm -f $tmp_rp
	dbg 2 "Loading IWSVA redirect rules(ret=$ret)"

    for i in $ftp_pots; do
        echo "ADD $i" >> $tmp_ftp_ports
        ret=$(($ret | $?))
    done

    sudo cat $tmp_ftp_ports > $PROC_FTP_PORTS 2>/dev/null
	rm -f $tmp_ftp_ports

	dbg 2 "Loading IWSVA ftp ports rules(ret=$ret)"
	return $ret
}

pre_network_ini ()
{
	if [ ! -f $NETINIDEF ]; then
		return 0
	fi
	if [ -f $NETINI ]; then
		load_ifname 1
	fi

	cp -af $NETINIDEF $NETINI

	set_param $NETINI ifname_internal $INT
	set_param $NETINI ifname_external $EXT
	set_param $NETINI ifname_others $OTH
	set_param $NETINI port_group $port_group

	set_param $NETINI ifname_proxy $PROXY
	set_param $NETINI ifname_proxy_internal $PROXY_IN
	set_param $NETINI ifname_proxy_external $PROXY_EX

	set_param $NETINI ifname_mgmt $MGMT
	set_param $NETINI vid $VID
	rm -f $NETINIDEF
}

init_install()
{
	install_nethook
	update_squid_conf
	return $?
}

vlan_stop ()
{
	if [ "$debug" -gt 0 ]; then
		dbg 2 "skip vlan stop";
		return 0;
	fi
	dbg 2 "vlan stop, remove all vlan ids"

	DEVICE="$BRI.$VID"
	ifconfig | grep -q "^$DEVICE"
	if [ "$?" -eq 0 ]; then
		ifconfig $DEVICE down
	fi
	remove_vlan
}

# use to hook the iwsva network to system network
install_nethook()
{
	if [ -L $NETSH_ORG ]; then
		LNNAME=`readlink $NETSH_ORG`
		if [ "$LNNAME" = "$IWSVANETSH" ]; then
			dbg 2 "iwsva network has hooked"
			return 0;
		fi
	fi

	if [ -f $NETSH_ORG ]; then
		cp -af $NETSH_ORG $NETSH_SYS
	else
		dbg 2 "missing the $NETSH_ORG"
	fi

	ln -sf $IWSVANETSH $NETSH_ORG

	dbg 2 "iwsva network hooked"

	return 0;
}

get_real_nic()
{
	# get original nics
	gorealnics=`ls /sys/class/net | grep "^eth[0-9]*$"`
	grealnics=""
	for i in $gorealnics ; do
		grealnics="$grealnics$i "
	done
	if [ $debug -ne 0 ]; then
		grealnics="eth0 eth1 eth3"
	fi
	return 0
}

get_real_nic_count()
{
	nic_count=`ls /sys/class/net | grep '^eth[0-9]*$' | wc -l`
	if [ $debug -ne 0 -a $nic_count -eq 1 ]; then
		nic_count=$(($nic_count+1))
	fi
	return $nic_count
}

# param : <ininame> <section>
write_header_ini()
{
	dbg 1 "generate $1"
	echo "# "`date`" This file is generated by IWSVA system." > $1
	echo "[$2]" > $1
}

create_networkini ()
{
	local nic1=$1
	local nic2=$2

	write_header_ini $NETINI "network"
	set_param $NETINI ifname_internal $nic1

	set_param $NETINI ifname_proxy $nic1
	set_param $NETINI ifname_proxy_internal ""
	set_param $NETINI ifname_proxy_external ""

	if [ -z "$nic2" -o $nic2 = "none" ]; then
		i18n_no_2nd_nic=`gettext "No second network interface exists. Switch to proxy mode manually."`
		echo $i18n_no_2nd_nic
		set_param $ISCANINI bridgemode 0
		set_param $NETINI ifname_external none
#add manamgenet port
		set_param $NETINI separate_mgmt_port no
	else
		set_param $ISCANINI bridgemode 1
		set_param $NETINI ifname_external $nic2
	fi
}

get_nic_2 ()
{
	local idx=0
	for i in $grealnics ; do
		case $idx in
		0)
			gnic1="$i"
			;;
		1)
			gnic2="$i"
			return 0;
			;;
		2)
			gnic3="$i"
			return 0;
			;;
		esac

		idx=$(($idx + 1))
	done

	if [ -z "$gnic2" ]; then
		gnic2=none
	fi

	if [ -z "$gnic3" ]; then
		gnic3=none
	fi

	return 0
}

init_networkini ()
{
	local gnic1=""
	local gnic2=""
	local gnic3=""

	get_real_nic

	dbg 2 "all intefaces: $grealnics"

	get_nic_2

	# if not existed $NETINI
	if [ ! -f $NETINI ]; then
		create_networkini $gnic1 $gnic2
		return 0
	fi

	load_ifname

	# Case: to auto swtich to proxy mode
	if [ "$bridgemode" -eq 1 -a "$gnic2" = "none" ]; then
		i18n_no_2nd_nic=`gettext "No second network interface exists. Switch to proxy mode manually."`
		echo $i18n_no_2nd_nic
		set_param $ISCANINI bridgemode 0
	fi

	#disable seprated managment port when we only have one nic
	if [ "$separate_mgmt" = "yes" -a "$gnic2" = "none" ]; then
		i18n_no_2nd_nic=`gettext "No second network interface exists. Disable seprated managmenet interface."`
		echo $i18n_no_2nd_nic
		set_param $NETINI separate_mgmt_port no
	fi

	#disable seprated managment port when we only have two nic and bridge is enable
	if [ "$separate_mgmt" = "yes" -a "$gnic3" = "none" -a "$bridgemode" -eq 1 ]; then
		i18n_no_2nd_nic=`gettext "No third network interface exists when bridge mode enable. Disable seprated managmenet interface."`
		echo $i18n_no_2nd_nic
		set_param $NETINI separate_mgmt_port no
	fi

	if [ "$separate_mgmt" = "yes" -a "$bridgemode" -eq 1 ]; then
		if [ "$INT" = "$MGMT" -o "$EXT" = "$MGMT" ]; then
			i18n_no_2nd_nic=`gettext "Management port has been used by bridge. Disable seprated managmenet interface."`
			echo $i18n_no_2nd_nic
			set_param $NETINI separate_mgmt_port no
		fi
	else
		for inf in $PROXY_ALL; do
			if [ "$inf" = "$MGMT" ]; then
				i18n_no_2nd_nic=`gettext "Management port has been used by proxy. Disable seprated managmenet interface."`
				echo $i18n_no_2nd_nic
				set_param $NETINI separate_mgmt_port no
				break
			fi
		done
	fi

	idx=0
	# check the current ifname is correct or not
	while [ $idx -lt 4 ]; do
		case $idx in
		0) # INT
			fdname=ifname_internal
			mnic="$INT"
			;;
		1) # EXT
			fdname=ifname_external
			mnic="$EXT"
			;;
		2) # PROXY
			fdname=ifname_proxy
			mnic="$PROXY"
			;;
		3) # MGMT
			fdname=ifname_mgmt
			mnic="$MGMT"
			;;
		esac

		is_match=0
		for i in $grealnics ; do
			# if inteface change, does it have to log error?
			dbg 4 "try match $mnic $i"

			if [ "$mnic" = "$i" ]; then
				is_match=1
#				break;
			fi
		done

		echo "$mnic" | grep -q "^bond[0-9]*$"
		if [ "$?" -eq 0 ]; then
			is_match=1
		fi

		if [ $is_match -eq 0 ]; then
			case $idx in
			0) # INT
				target_nic=$gnic1
				;;
			1) # EXT
				target_nic=$gnic2
				;;
			2) # PROXY
				target_nic=$gnic1
				;;
			esac

			if [ "$target_nic" = "none" ]; then
				dbg 2 "$fdname=$mnic means no second interface"
			else
				dbg 2 "$fdname=$mnic is not a existed interface. set $fdname=$target_nic"
			fi

			set_param $NETINI $fdname $target_nic
		fi

		idx=$(($idx+1))
	done

	return 0
}

# fixed the NIC order changed after anconda applied NIC settings
# It runs while the HWADDR_eth<n> is not existed in $NETINI
init_hwaddr ()
{
	grep -q "^HWADDR_" $NETINI
	if [ "$?" = "0" ]; then
		# skip to save the HWADDR
		return 0
	fi

	# get MAC ADDRESS to set to $NETINI
	for i in $grealnics ; do
		ifcfg="$cfgdir/ifcfg-$i"

		get_param $ifcfg HWADDR
		if [ -n "$gvalue" ]; then
			OHWADDR="$gvalue"
		fi

		if [ -n "$OHWADDR" ]; then
			add_param $NETINI "HWADDR_$i" "$OHWADDR"
		fi
	done

	return 0
}

init_startup ()
{
	get_real_nic
	#init_hwaddr

	# init and check network.ini
	init_networkini
	pre_network_ini

	if [ ! -f $NETINI ]; then
		dbg 1 "init_startup: not ready"
	fi

	load_conntrack_kmod
	ret1=$?

	load_redirect_rules
	ret2=$?

	# check ifcfg-* is existed or not
	# if not the generate in advance


	# usb nic will be listed a known issue
	# check interface mapping in NETINI
	# auto switch to proxy mode
	# select the first mode

	load_ifname

	# while in full transparent proxy mode, you must to turn off tcp segment offload
	if [ $bridgemode -eq 1 ]; then
		ethtool -K $INT tso off
		ethtool -K $INT gso off
		ethtool -K $INT gro off
		ethtool -K $EXT tso off
		ethtool -K $EXT gso off
		ethtool -K $EXT gro off

		for intf in $OTH
		do
			ethtool -K $intf tso off
		done

		####### add code here if $OTH is not null
	fi

	hook_vlan

	return $(($ret1 | $ret2))
}

init_uninstall ()
{
	uninstall_nethook
	ret=$?
}

uninstall_nethook ()
{
	if [ -L $NETSH_ORG ]; then
		LNNAME=`readlink $NETSH_ORG`
		if [ "$LNNAME" != "$IWSVANETSH" ]; then
			dbg 2 "iwsva network don't hook"
			return 0;
		fi
	else
		return 0;
	fi

	if [ -f $NETSH_SYS ]; then
		rm -f $NETSH_ORG
		cp -af $NETSH_SYS $NETSH_ORG
		rm -f $NETSH_SYS

		dbg 2 "iwsva network hook removed"
	else
		dbg 2 "$NETSH_SYS is missing files"
	fi
	return 0
}

dbg()
{
	level="$1"
	
	if [ -z "$level" -o $level -gt $verbose ]; then
		return
	fi
	
	shift;
	echo -e "$@" >> $debug_dev
}

# get_param <cfgname> <fd name>
# return gvalue
get_param ()
{
	local cfgname="$1"
	local fdname="$2"
	local gline=""

	gvalue=""
	if [ -z "$cfgname" ]; then
		echo "Error missing the configuraion $cfgname"
	fi
	if [ -z "$fdname" ]; then
		echo "Error missing argument: fdname"
	fi

	# fixed the duplicate entry
	gline=`grep "$fdname[[:space:]]*=" $cfgname 2>/dev/null | grep -v "^#" | head -n 1`
	if [ -z "$gline" ]; then
		return 1;
	fi

	gvalue=`echo "$gline" | cut -d"=" -f2`
	if [ -z "$gvalue" ]; then
		return 1
	fi

	echo "$gvalue" | grep -q '^\"'
	if [ "$?" -eq 0 ] ; then
		gvalue="${gvalue:1}"
	fi

	echo "$gvalue" | grep -q '\"[[:space:]]*'
	if [ "$?" -eq 0 ] ; then
		gvalue="${gvalue%\"*}"
	fi

	return 0;
}

# set_param <cfgname> <fd name> <fd value>
set_param ()
{
	local cfgname=""
	local fdname=""
	local fdvalue=""

	cfgname="$1"
	if [ -z "$cfgname" ]; then
		echo "Error: missing argument: config name"
	fi

	fdname="$2"
	if [ -z "$fdname" ]; then
		echo "Error: missing argument: field name"
	fi

	fdvalue="$3"
	if [ -z "$fdvalue" ] ; then
		return 0;
	fi

	local line="`grep \"[[:space:]]*$fdname[[:space:]]*=\" $cfgname 2>/dev/null | grep -v "^#" 2>/dev/null`"
	if [ -n "$line" ]; then
		sed -i "s%\([[:space:]]*$fdname[[:space:]]*\)=.*%\1=$fdvalue%g" $cfgname
	else
		echo "$fdname=$fdvalue" >> $cfgname
	fi

	return 0;
}

load_ifname ()
{
	local load_vid=$(($1))
	if [ ! -f $NETINI ]; then
		echo "missing the configuraion $NETINI"
	fi
	if [ ! -f $ISCANINI ]; then
		echo "missing the configuraion $ISCANINI"
	fi

	get_param $ISCANINI bridgemode
	if [ -n $gvalue ]; then
		bridgemode=$(($gvalue))
	else
		bridgemode=0
	fi

	get_param $NETINI ifname_internal
	if [ -n $gvalue ]; then
		INT=$gvalue
	fi

	get_param $NETINI ifname_external
	if [ -n $gvalue ]; then
		EXT=$gvalue
	fi

	get_param $NETINI ifname_others
	if [ -n "$gvalue" ]; then
        OTH=$gvalue
	fi

	get_param $NETINI port_group
	if [ -n "$gvalue" ]; then
        port_group=$gvalue
	fi

	get_param $NETINI ifname_proxy
	if [ -n "$gvalue" ]; then
		PROXY=$gvalue
	fi

	PROXY_ALL="$PROXY"

	get_param $NETINI ifname_proxy_internal
	if [ -n "$gvalue" ]; then
		PROXY_IN=$gvalue
	fi

	PROXY_ALL="$PROXY_ALL $PROXY_IN"

	get_param $NETINI ifname_proxy_external
	if [ -n "$gvalue" ]; then
		PROXY_EX=$gvalue
	fi

	PROXY_ALL="$PROXY_ALL $PROXY_EX"

	get_param $NETINI vid
	if [ -n "$gvalue" ]; then
		VID="$(($gvalue))"
	else
		VID=0
	fi

	if [ "$bridgemode" -eq 0 -a $load_vid -eq 0 ]; then
		VID=0;
	fi

#add mgmt interface
	get_param $NETINI separate_mgmt_port
	if [ "$gvalue" = "yes"  ]; then
		separate_mgmt="yes"
		get_param $NETINI ifname_mgmt

		if [ -n $gvalue ]; then
			MGMT=$gvalue
		fi
	else
		MGMT=none
	fi

	get_param $NETINI enable_stp
	if [ "$gvalue" = "yes" ]; then
		enable_stp="yes"
	else
		enable_stp="no"
	fi
}

load_kmod()
{
	insmod /opt/TrendMicro/Common/lib/netbase.ko
	insmod /opt/TrendMicro/Common/lib/conntrack.ko
	insmod /opt/TrendMicro/Common/lib/bypass_ftphelper.ko
}

unload_kmod()
{
	rmmod bypass_ftphelper.ko
	rmmod conntrack.ko
	rmmod netbase.ko
}

is_arg_val ()
{
	for i in $2 ; do
	if [ "$1" = "$i" ] ; then
		dbg 3 "$1 existed"
		return 1;
	fi
	done
	return 0;
}

add_param()
{
	local cfgname="$1"
	local fdname="$2"
	local fdvalue="$3"

	if [ -z $fdvalue ] ; then
		return;
	fi

	echo "$fdname=$fdvalue" >> $cfgname
	return 0;
}

del_param()
{
	local cfgname="$1"
	local fdname="$2"
	if [ -z "$cfgname" -o -z "fdname" ]; then
		dbg 2 "The argument is missing"
	fi

	cat $cfgname | grep -q "^${fdname}[[:space:]]*="
	if [ $? -eq 1 ]; then
		# not fould, skip it
		return 0;
	fi

	cp -af $cfgname ${cfgname}.bak
	grep -v "^${fdname}[[:space:]]*=" ${cfgname}.bak > $cfgname
	rm -f ${cfgname}.bak

	return 0;
}

get_ipinfo_cfgname ()
{
	dbg 2 "check old bridge ifcfg is existed or not"

	if [ -f "$tmpcfgdir/ifcfg-$BRI" ]; then
		ipcfgname=`find $tmpcfgdir | grep "ifcfg-$BRI\." | head -n 1`

		# bridge mode enable
		if [ -n "$ipcfgname" ]; then
			# change vid
			ODEVICE="$ipcfgname";
		else
			ODEVICE="$tmpcfgdir/ifcfg-$BRI";
		fi

		return 0
	fi

	if [ -z "$OPROXY" ]; then
		OPROXY="$PROXY"
	fi

	ipcfgname=$tmpcfgdir/ifcfg-$OPROXY
	if [ -f "$ipcfgname" ]; then
		ODEVICE="$ipcfgname"
		return 0
	fi

	ipcfgname="`find $tmpcfgdir | sort | grep "ifcfg-eth[0-9]*"| head -n 1`"
	if [ -z "$ipcfgname" ]; then
		return 1
	fi

	# change vid
	ODEVICE="$ipcfgname"
	return 0
}

tmpcfgdir="/tmp/iwsvaifcfg"
load_ifcfg_old ()
{
	OBOOTPROTO=""
	ONETMASK=""
	OIPADDR=""
	OGATEWAY=""
	OIPV6ADDR=""
	OIPV6INIT=""
	OIPV6_DEFAULTGW=""
	ODHCPV6C=""

	if [ -z "$2" ]; then
	    get_ipinfo_cfgname
	else
	    ODEVICE="$2"
	fi

	dbg 2 "load ipaddr info config"

	ifcfg="$ODEVICE"
	if [ -z "$ODEVICE" ]; then
		return 0
	fi

	get_param $ifcfg BOOTPROTO
	if [ -n "$gvalue" ]; then
		OBOOTPROTO="$gvalue"
	fi

	get_param $ifcfg IPV6INIT
	if [ -n "$gvalue" ]; then
		OIPV6INIT="$gvalue"
	fi

	get_param $ifcfg IPV6ADDR
	if [ -n "$gvalue" ]; then
		OIPV6ADDR="$gvalue"
	fi

	get_param $ifcfg IPV6_DEFAULTGW
	if [ -n "$gvalue" ]; then
		OIPV6_DEFAULTGW="$gvalue"
	fi

	get_param $ifcfg DHCPV6C
	if [ -n "$gvalue" ]; then
		ODHCPV6C="$gvalue"
	fi

	IPV6INIT="$OIPV6INIT"
	IPV6ADDR="$OIPV6ADDR"
	IPV6_DEFAULTGW="$OIPV6_DEFAULTGW"
	DHCPV6C="$ODHCPV6C"

	BOOTPROTO="$OBOOTPROTO"
	if [ "$OBOOTPROTO" = "dhcp" ]; then
		return 0;
	fi

	get_param $ifcfg IPADDR
	if [ -n "$gvalue" ]; then
		OIPADDR="$gvalue"
	fi

	get_param $ifcfg NETMASK
	if [ -n "$gvalue" ]; then
		ONETMASK="$gvalue"
	fi

	get_param $ifcfg GATEWAY
	if [ -n "$gvalue" ]; then
		OGATEWAY="$gvalue"
	fi

	NETMASK="$ONETMASK"
	IPADDR="$OIPADDR"
	GATEWAY="$OGATEWAY"

	get_param $CFGNETWORK GATEWAY
	DEFAULT_GATEWAY="$gvalue"

	return 0
}

generate_ifcfg_vid ()
{
	ifcfgvid="$cfgdir/ifcfg-$1"

	ONBOOT="yes"
	if [ -z $BOOTPROTO ]; then
		BOOTPROTO="dhcp"
	fi

	rm -f $ifcfgvid

	write_header_ifcfg $ifcfgvid

	add_param $ifcfgvid "DEVICE" "$1"
	add_param $ifcfgvid "ONBOOT" "yes"
	add_param $ifcfgvid "TYPE"  "ETHERNET"
	add_param $ifcfgvid "BOOTPROTO" "$BOOTPROTO"

	add_param $ifcfgvid "NETMASK" "$NETMASK"
	add_param $ifcfgvid "IPADDR" "$IPADDR"
	add_param $ifcfgvid "GATEWAY" "$GATEWAY"
	add_param $ifcfgvid "IPV6INIT" "$IPV6INIT"
	add_param $ifcfgvid "IPV6ADDR" "$IPV6ADDR"
	add_param $ifcfgvid "VLAN" "yes"
	add_param $ifcfgvid "ISALIAS" "no"
}

chk_vid ()
{
	if [ -z "$VID" -o "$VID" = "0" ]; then
		VID=0
		return 0
	fi
	return 1
}

create_ifcfg_vid()
{
	chk_vid

	if [ $? -eq 0 ]; then
		return 0
	fi

	if [ "$target_bridgemode" -eq 1 ]; then
		TYPE="Bridge"
		DEVICE="$BRI.$VID"
	else
		TYPE="Ethernet"
		DEVICE="$PROXY.$VID"
	fi

	dbg 2 "create ifcfg with $DEVICE"
	generate_ifcfg_vid $DEVICE
}

create_ifcfg_main ()
{
	local iftargetcfg

	iftargetcfg="$cfgdir/ifcfg-$DEVICE"

	write_header_ifcfg $iftargetcfg

	# added parameter
	add_param $iftargetcfg "DEVICE" "$DEVICE"
	add_param $iftargetcfg "ONBOOT" "yes"

	add_hwaddr $iftargetcfg  $DEVICE

	if [ $target_bridgemode -eq 1 ]; then
		add_param $iftargetcfg "TYPE" "Bridge"

		# generate ifcfg int, ext and oth
		# except bonding device for the system script ifup-eth
		# cannot bring up the bonding device when the device is
		# a slave of the bridge
		for interface in $INT $EXT $OTH; do
			echo "$interface" | grep -q "^bond"
			if [ "$?" -ne 0 ]; then
				create_brifcfg "$interface"
			fi
		done

		add_param $iftargetcfg "ISALIAS" "no"
	else
		#case 1-2 change mode proxy -> bridge
		add_param $iftargetcfg "TYPE" "Ethernet"
	fi

	load_ifcfg_old
	chk_vid
	if [ $? -eq 1 ]; then
		create_ifcfg_vid
		add_param $iftargetcfg "BOOTPROTO" "none"
		return 0
	fi

	if [ -z "$BOOTPROTO" ]; then
		BOOTPROTO="dhcp"
	fi

	add_param $iftargetcfg "BOOTPROTO" "$BOOTPROTO"
	add_param $iftargetcfg "NETMASK" "$NETMASK"
	add_param $iftargetcfg "IPADDR" "$IPADDR"
	add_param $iftargetcfg "GATEWAY" "$GATEWAY"
	add_param $iftargetcfg "IPV6INIT" "$IPV6INIT"
	add_param $iftargetcfg "DHCPV6C" "$DHCPV6C"
	add_param $iftargetcfg "IPV6ADDR" "$IPV6ADDR"
	add_param $iftargetcfg "IPV6_DEFAULTGW" "$IPV6_DEFAULTGW"

	if [ "$BOOTPROTO" = "dhcp" -a -n "$DEFAULT_GATEWAY" ]; then
		# clear default gateway setting
		del_param "$CFGNETWORK" GATEWAY
	fi

	if [ $target_bridgemode -eq 1 ]; then
        get_param $NETINI enable_multilink
        if [ "$gvalue" = "yes" ]; then
			local duelbridgecfg
			duelbridgecfg="$cfgdir/ifcfg-$BRII"

			write_header_ifcfg $duelbridgecfg
        	add_param $duelbridgecfg "DEVICE" "$BRII"
        	add_param $duelbridgecfg "ONBOOT" "yes"
		    add_hwaddr $duelbridgecfg $BRII
        	add_param $duelbridgecfg "TYPE" "Bridge"

        	for interface in $DUELINT $DUELEXT ; do
               	echo "$interface" | grep -q "^bond"
               	if [ "$?" -ne 0 ]; then
               		create_brifcfg "$interface" "$BRII"
               		fi
        	done

        	add_param $duelbridgecfg "ISALIAS" "no"
        	add_param $duelbridgecfg "BOOTPROTO" "none"
		fi
	fi

	return 0;
}

create_ifcfg_main_mgmt ()
{
	local iftargetcfg

	if [ -n "$MGMT" ]; then
		iftargetcfg="$cfgdir/ifcfg-$MGMT"
	else
		return 1;
	fi

	write_header_ifcfg $iftargetcfg
	# added parameter
	add_param $iftargetcfg "DEVICE" "$MGMT"
	add_param $iftargetcfg "ONBOOT" "yes"

	add_hwaddr $iftargetcfg  $MGMT
	add_param $iftargetcfg "TYPE" "Ethernet"

	BOOTPROTO=""
	NETMASK=""
	IPADDR=""
	IPV6INIT=""
	IPV6ADDR=""
	DHCPV6C=""
	IPV6_DEFAULTGW=""

	ifcfg_mgmt="$tmpcfgdir/ifcfg-$MGMT"
	get_param $ifcfg_mgmt BOOTPROTO
	if [ -n "$gvalue" ]; then
		BOOTPROTO="$gvalue"
	fi

	get_param $ifcfg_mgmt IPADDR
	if [ -n "$gvalue" ]; then
		IPADDR="$gvalue"
	fi
	get_param $ifcfg_mgmt NETMASK
	if [ -n "$gvalue" ]; then
		NETMASK="$gvalue"
	fi
	get_param $ifcfg_mgmt IPV6INIT
	if [ -n "$gvalue" ]; then
		IPV6INIT="$gvalue"
	fi
	get_param $ifcfg_mgmt IPV6ADDR
	if [ -n "$gvalue" ]; then
		IPV6ADDR="$gvalue"
	fi
	get_param $ifcfg_mgmt IPV6_DEFAULTGW
	if [ -n "$gvalue" ]; then
		IPV6_DEFAULTGW="$gvalue"
	fi
	get_param $ifcfg_mgmt DHCPV6C
	if [ -n "$gvalue" ]; then
		DHCPV6C="$gvalue"
	fi

	if [ -z "$BOOTPROTO" ]; then
		BOOTPROTO="dhcp"
	fi

	add_param $iftargetcfg "BOOTPROTO" "$BOOTPROTO"
	add_param $iftargetcfg "NETMASK" "$NETMASK"
	add_param $iftargetcfg "IPADDR" "$IPADDR"
	add_param $iftargetcfg "IPV6INIT" "$IPV6INIT"
	add_param $iftargetcfg "IPV6ADDR" "$IPV6ADDR"
	add_param $iftargetcfg "IPV6_DEFAULTGW" "$IPV6_DEFAULTGW"
	add_param $iftargetcfg "DHCPV6C" "$DHCPV6C"

	return 0;
}

backup_ifcfg()
{
	dbg 2 "backup ifcfg files"
	mkdir -p $tmpcfgdir

	find $cfgdir | grep "ifcfg\-" | ( while read fn ; do
		FNAME=`basename $fn`
		case $FNAME in
		ifcfg-eth*)
			cp -af $cfgdir/ifcfg-eth* $tmpcfgdir
			dbg 2 "backup ethernet ifcfg"
			;;
		ifcfg-${BRI}*)
			cp -af $cfgdir/ifcfg-${BRI}* $tmpcfgdir
			dbg 2 "backup bridge ifcfg"
			;;
		ifcfg-bond*)
			cp -af $cfgdir/ifcfg-bond* $tmpcfgdir
			dbg 2 "backup bonding ifcfg"
			;;
		ifcfg-${BRII}*)
			cp -af $cfgdir/ifcfg-${BRII}* $tmpcfgdir
	  		dbg 2 "backup duel bridge ifcfg"
			;;
		esac
	done
	)

	return 0
}

remove_ifcfg_proxy()
{
	find $cfgdir | grep "ifcfg\-" |( while read fn ; do
		FNAME=`basename $fn`
		
		is_found=0
		
		for inf in $PROXY_ALL; do
			if [ "$FNAME" = "ifcfg-$inf" ]; then
				is_found=1
				break
			fi
		done		
		
		if [ "$FNAME" = "ifcfg-$PROXY" ]; then
			is_found=0
		fi
		
		dbg 2 "remove_ifcfg_proxy is_found=$is_found"
		
		if [ "$is_found" = "0" ]; then
			dbg 2 "remove_ifcfg_proxy check and remove $FNAME"

			case $FNAME in
			ifcfg-eth*)
				dbg 2 "removing $fn"
				rm -f "$fn"
				;;
			ifcfg-${BRI}*)
				dbg 2 "removing $fn"
				rm -f "$fn"
				;;
			ifcfg-bond*)
				dbg 2 "removing $fn"
				rm -f "$fn"
				;;
			ifcfg-${BRII}*)
				dbg 2 "removing $fn"
				rm -f "$fn"
				;;
			esac
		fi
	done
	)

	return 0
}

remove_ifcfg()
{
	find $cfgdir | grep "ifcfg\-" |( while read fn ; do
		FNAME=`basename $fn`

		case $FNAME in
		ifcfg-eth*)
			dbg 2 "removing $fn"
			rm -f "$fn"
			;;
		ifcfg-${BRI}*)
			dbg 2 "removing $fn"
			rm -f "$fn"
			;;
		ifcfg-bond*)
			dbg 2 "removing $fn"
			rm -f "$fn"
			;;
		ifcfg-${BRII}*)
			dbg 2 "removing $fn"
			rm -f "$fn"
			;;
		esac
	done
	)

	return 0
}

destory_old_data_interface()
{
#andy add
	DATA_INTERFACE=""
	load_ifname

	if [ $bridgemode -eq 1 ]; then
		get_param $NETINI ifname_proxy

		if [ -n $gvalue ]; then
			DATA_INTERFACE=$gvalue
		fi

		get_param $NETINI ifname_proxy_internal

		if [ -n $gvalue ]; then
			DATA_INTERFACE="$DATA_INTERFACE $gvalue"
		fi

		get_param $NETINI ifname_proxy_external

		if [ -n $gvalue ]; then
			DATA_INTERFACE="$DATA_INTERFACE $gvalue"
		fi

		for inf in $DATA_INTERFACE; do
			destroy_route $inf
		done
	else
		get_param $NETINI vid
        if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
            destroy_route "br0.$gvalue"
        else
	        destroy_route "br0"
        fi
	fi
}

# turnbirdge on <target_int> <target_ext>
# turnbirdge off <target proxy>
turnbridge ()
{
	local ifname;
	local ifcfgname;

	init_networkini
	# load interface name
	load_ifname

	OPROXY="$PROXY"

	# get target mode
	if [ "$1" = "on" ]; then
		target_bridgemode=1
		if [ "$EXT" = "none" ]; then
			dbg 2 "unable to turn on bridge with only one NIC"
			return 1
		fi
	else
		target_bridgemode=0
	fi

	# fixed ui birdgemode problem
	ifcfgbri=`ls $cfgdir/ifcfg-* | grep $BRI`
	if [ -z "$ifcfgbri" ]; then
		bridgemode=0
	else
		bridgemode=1
	fi

	dbg 2 "bridgemode is changed from ${bridgemode} to $target_bridgemode"

	if [ "$bridgemode" -eq 1 ]; then
		# load configue in ifname_proxy
		ifname="$BRI"
	else
		ifname="$PROXY"
	fi

	if [ "$1" = "on" ]; then
		iftarget="$BRI"
		target_int="$2"
		target_ext="$3"

		param_num="$#"
		shift;shift;shift

		target_oth=$*

		#"turnbridge on eth1 eth2" for two port bridge setting
		if [ "$param_num" -eq 3 ]; then
			OTH=""
			del_param $NETINI ifname_others
		fi
	else
		iftarget="$PROXY"
		target_proxy="$2"
	fi

	ifcfgname="$cfgdir/ifcfg-$ifname"
	ifcfgname_int="$cfgdir/ifcfg-$INT"
	ifcfgname_ext="$cfgdir/ifcfg-$EXT"
	ifcfgname_proxy="$cfgdir/ifcfg-$PROXY"
	iftargetcfg="$cfgdir/ifcfg-$iftarget"

	tmpifcfgname="$tmpcfgdir/ifcfg-$ifname"

	# before change configure
	uninit_portgroup
	uninit_bonding

	stopnet
	# backup all related configuration in advance

	backup_ifcfg

	# remove all related configuration
	is_proxy_reload=0

	if [ "$bridgemode" -eq "$target_bridgemode" ]; then
		if [ $target_bridgemode -eq 0 ]; then
			is_proxy_reload=1
		else
			del_param $NETINI ifname_proxy_internal ""
			del_param $NETINI ifname_proxy_external ""

			PROXY_ALL=$PROXY
			PROXY_IN=
			PROXY_EX=
		fi
	else
		del_param $NETINI ifname_proxy_internal ""
		del_param $NETINI ifname_proxy_external ""

		PROXY_ALL=$PROXY
		PROXY_IN=
		PROXY_EX=
	fi

	if [ $is_proxy_reload -eq 0 ]; then
		remove_ifcfg
	else
		remove_ifcfg_proxy
	fi

	DEVICE=""
	BOOTPROTO=""
	NETMASK=""
	IPADDR=""
	GATEWAY=""
	IPV6INIT=""
	IPV6ADDR=""
	DHCPV6C=""
	IPV6_DEFAULTGW=""

	# case 0: apply int, ext, proxy change
	if [ -n "$target_int" ]; then
		set_param $NETINI ifname_internal $target_int
		INT="$target_int"
	fi

	if [ -n "$target_ext" ]; then
		set_param $NETINI ifname_external $target_ext
		EXT="$target_ext"
	fi

	if [ -n "$target_proxy" ]; then
		set_param $NETINI ifname_proxy $target_proxy
		PROXY="$target_proxy"
	fi

	if [ -n "$target_oth" ] ; then
		set_param $NETINI ifname_others "$target_oth"
		OTH="$target_oth"
	fi

	if [ "$bridgemode" -ne "$target_bridgemode" ]; then
		destory_old_data_interface

		if [ $target_bridgemode -eq 1 ]; then
			DEVICE=$BRI
			#case 1: change mode proxy -> bridge
			dbg 1 "switch mode from proxy to bridge"
			get_param $NETINI vid
			VID="$((gvalue))"
		else
			#case 2: change mode bridge -> proxy
			DEVICE=$PROXY
			dbg 1 "switch mode from bridge to proxy"
			VID=0
	#		rename_sroute $BRI
		fi

		# change mode
		set_param $ISCANINI bridgemode $target_bridgemode
	else
		if [ $target_bridgemode -eq 1 ]; then
			#case 3: bridge mode is reloading
			dbg 1 "bridge mode is reloading."
			DEVICE=$BRI
			#reload lanbypass configuration when bridge setting changed
			#rm -f /etc/lanbypass.conf
			/etc/init.d/lanbypass restart > /dev/null 2>&1
		else
			#case 4: proxy mode is reloading
			dbg 1 "proxy mode is reloading."
			DEVICE=$PROXY
			VID=0
	#		rename_sroute $BRI
		fi
	fi

  	get_param $NETINI enable_multilink
	multilink="$gvalue"

  	if [ "$multilink" = "yes" ]; then
		get_param $NETINI ifduel_internal
		DUELINT="$gvalue"
		get_param $NETINI ifduel_external
		DUELEXT="$gvalue"

		ethtool -K $DUELINT tso off
		ethtool -K $DUELINT gso off
		ethtool -K $DUELINT gro off
		ethtool -K $DUELEXT tso off
		ethtool -K $DUELEXT gso off
		ethtool -K $DUELEXT gro off
	fi

	create_ifcfg_main

#add mgmt interface
	if [ $separate_mgmt = "yes" ]; then
		if [ -f $tmpcfgdir/ifcfg-$MGMT ]; then
#			mv $tmpcfgdir/ifcfg-$MGMT $cfgdir/ifcfg-$MGMT
			create_ifcfg_main_mgmt
			if [ $? -eq 1 ]; then
				set_param $NETINI separate_mgmt_port no
			fi
		else
			set_param $NETINI separate_mgmt_port no
		fi
	fi

	init_bonding

	bridgemode=$((target_bridgemode))
	load_redirect_rules

	# restart firewall rules related to scanning service
	/usr/iwss/iwsvafw.sh scanning_service
	/usr/iwss/iwsvafw.sh local_service

	# after configure
	iwsva_deploy_route 0
	startnet
	iwsva_check_route_table

	init_portgroup
	add_bonding_to_bridge

	rm -rf $tmpcfgdir
	dhcpuphooks

	return 0
}

init_portgroup()
{
    get_param $NETINI port_group
    pg_setting="$gvalue"

    pg_index=1
    pg_group=`echo $pg_setting | cut -d'|' -f$pg_index`
    if [ "$pg_group" != "" ]; then
        reload_pg_module
    else
        uninit_portgroup
    fi

    multiple=`echo $pg_setting | grep "|"`
	new_setting=""
    while [ "$pg_group" != "" ] ; do
        pg_index=`expr $pg_index + 1`
        $DEVCONF -c "$pg_group" 2>/dev/null >/dev/null
        ret="$?"

        if [ "$ret" -eq 0 ]; then
            if [ "$new_setting" = "" ]; then
                new_setting="$pg_group"
            else
                new_setting="$new_setting|$pg_group"
            fi
        fi

        if [ "$multiple" != "" ] ; then
            pg_group=`echo $pg_setting | cut -d'|' -f$pg_index`
        else
            break
        fi
    done

    if [ -n "$new_setting" ]; then
        set_param $NETINI port_group "$new_setting"
    else
        del_param $NETINI port_group
        uninit_portgroup
    fi
}

uninit_portgroup()
{
    lsmod | grep -q "devctrl"
    if [ "$?" -eq 0 ]; then
        rmmod devctrl.ko 2>/dev/null >/dev/null
    fi
}

reload_pg_module()
{
    lsmod | grep -q "devctrl"
    if [ "$?" -eq 0 ]; then
        rmmod devctrl.ko 2>/dev/null >/dev/null
    fi

    insmod $LIB_DIR/devctrl.ko 2>/dev/null >/dev/null
}

load_pg_module()
{
    lsmod | grep -q "devctrl"
    if [ "$?" -ne 0 ]; then
        insmod $LIB_DIR/devctrl.ko 2>/dev/null >/dev/null
    fi
}

judge_interface()
{
	local find;
	local Interface;

	Interface=$1
	if [ -z $Interface ]; then
	    echo "please provide interface name"
	    return 1
	fi

	gorealnics=`ls /sys/class/net | grep "^eth[0-9]*$"`

	find="no"
	for i in $gorealnics ; do
        if [ "$Interface" = "$i" ]; then
                find="yes"
        fi
	done

    # if the vlan is configured, change interface to VLAN interface
    local vlan_interface="br0"

    get_param $NETINI vid
    if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
            vlan_interface="br0.$gvalue"
    fi

	if [ "$find" = "no" -a "$Interface" != "$vlan_interface" ]; then
	        echo "unknown interface $Interface: No such device"
	        return 1
	fi

	return 0
}

setinfo ()
{
	dbg 2 "set $@"

	ret=0
	action=$1
	shift

	case $action in
	bridgemode)
		turnbridge $1
		ret=$?

		#update lanbypass setting
		if [ "yes" == `/etc/init.d/lanbypass bypassable 2>/dev/null` ];then
	        /etc/init.d/lanbypass start > /dev/null 2>&1
		fi
		;;
	bridge)
		# set bridge interface <ifname_internal> <ifname_external>
		i18n_br_assign_ifnames=`gettext "The interface name for internal and external must be assigned."`
		i18n_br_setings=`gettext "Bridge setting is saved, but it will not be activated until the mode is changed to bridge mode."`

		if [ "$1" = "interface" ]; then
			if [ -z "$2" -o -z "$3"  ]; then
				echo $i18n_br_assign_ifnames
				return 1
			fi

			is_bridge_enable

			if [ "$?" -eq 1 ]; then
				shift
				turnbridge on $*
				dbg 1 "bridge settings is active"
			else
				set_param $NETINI ifname_internal $1
				set_param $NETINI ifname_external $2
				shift;shift
				set_param $NETINI ifname_others $*
				echo $i18n_br_setings
			fi
		fi
		;;
	# port group setting
	portgroup)
		set_portgroup $*
		;;
	# bonding setting
	bonding)
        set_bonding $*
		;;
	proxy)
		# set proxy interface <ifname_proxy>
		if [ "$1" = "interface" ]; then
			if [ -z "$2" ]; then
				i18n_pr_assign_ifnames=`gettext "The interface name for proxy must be assigned."`
				echo "$i18n_pr_assign_ifnames"
			fi

			is_bridge_enable

			if [ "$?" -eq 0 ]; then
				DATA_INTERFACE=""
				get_data_interface

				if [ "$2" != $DATA_INTERFACE ]; then
					destroy_route $DATA_INTERFACE
				fi

				turnbridge off $2
				dbg 1 "proxy settings is active"
			else
				i18n_pr_settings=`gettext "The proxy setting is saved, but it will not be activated until the mode is changed to proxy mode."`
				set_param $NETINI ifname_proxy $2
				echo $i18n_pr_settings
			fi
		fi
		;;
	ethernet)
		case "$1" in
		    static) # set static ipv4/ipv6 <eth0|eth1> <ipaddr> <netmask>/<prefix> [vid]
				case "$2" in
				    ipv4)
						if [ -z "$4" -o -z "$5" ]; then
						    usageset_eth_static
						    return 1
						fi
						setip ipv4 static $3 $4 $5 $6
						;;
				    ipv6)
						if [ -z "$4" -o -z "$5" ]; then
						    usageset_eth_static
						    return 1
						fi
						setip ipv6 static $3 $4 $5 $6
						;;
				esac
				;;
		    dhcp) # set dhcp <ifname> <vid>
				setip ipv4 dhcp $2 $3
				;;
		    dhcpv6)
				setip ipv6 dhcpv6 $2 $3
				;;
		    *)
				usageset_eth_usage
				usageset_eth_dhcp
				usageset_eth_static
				return 1
			;;
		esac
		;;
	redirect)
		set_redirect $@
		ret=$?
		;;
	reload_redirect)
		load_ifname

        if [ "$bridgemode" = "0" ]; then
            i18n_no_iports_in_proxymode="`gettext 'Proxy Mode does not allow you to reload redirection ports.'`"
            echo "$i18n_no_iports_in_proxymode"
			return 1
        else
			load_redirect_rules
			ret=$?
        fi
		;;
	routedel)
		if [ -z "$1" -o -z "$2" ]; then
			usageset_routedel
			return 1
		fi

	    # if the vlan is configured, change interface to VLAN interface
	    IFNAME=$1
	    if [ "$IFNAME" = "br0" ]; then
	        get_param $NETINI vid
	        if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
	            IFNAME="$IFNAME.$gvalue"
	        fi
	    fi

		judge_interface $IFNAME
		if [ $? -eq 1 ] ; then
			usageset_routedel
			return 1
		fi

		del_route $IFNAME $2
		;;

#added by andy wang
	uiroute)
		# uiroute ifname <ifname> index <new index>  <new route value>
		if [ "$1" != "ifname" ]; then
			usageset_uiroute
			return 1
		fi

		IFNAME=$2

	    # if the vlan is configured, change interface to VLAN interface
	    if [ "$IFNAME" = "br0" ]; then
	        get_param $NETINI vid
	        if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
	            IFNAME="$IFNAME.$gvalue"
	        fi
	    fi

		judge_interface $IFNAME
		if [ $? -eq 1 ] ; then
			usageset_uiroute
			return 1
		fi

		if [ "$3" = "index" ]; then
		 	index="$4"
			if [ -z "$index" ]; then
				index=-1; # append route
			fi
			ROUTE_VALUE=$5
			TMOPTION=$6
		else
			index=-1; # append route
			ROUTE_VALUE=$3
			TMOPTION=$4
		fi

		if [ -z "$ROUTE_VALUE" ]; then
			usageset_uiroute
			return 1
		fi

#		echo "index=$index"
		# uiroute ifname <ifname> [index <new index>] ADDRESS:GATEWAY:NETMASK TMOPTION
		# set uiroute ifname <ifname>  '<address>:<gateway>:<netmask>'
		# considering ipv6 ,value is like address|gateway|netmask,liangji

    	ADDRESS="`echo $ROUTE_VALUE | cut -d'|' -f1`"
 		GATEWAY="`echo $ROUTE_VALUE | cut -d'|' -f2`"
   		NETMASK="`echo $ROUTE_VALUE | cut -d'|' -f3`"

#		echo "ADDRESS=$ADDRESS"
#		echo "NETMASK=$NETMASK"
#		echo "GATEWAY=$GATEWAY"
#		echo "ROUTE_VALUE=$ROUTE_VALUE"

		if [ -z "$ADDRESS" -o -z "$NETMASK" -o -z "GATEWAY" ]; then
			usageset_uiroute
			return 1
		fi

		#conf ipv4 and ipv6 separatly,liangji
		#echo $ADDRESS | grep -q ':'

        route_conf $IFNAME $index $ADDRESS $NETMASK $GATEWAY 2
		;;
	webserver)
		case $1 in
		port)
			set_webserver_port $2
			;;
		*)
			;;
		esac
		;;
	hostname)
		if [ -z $1 ]; then
			usageset_hostname
			return 1
		fi

		set_hostname $1
		ret=$?
		;;
	dns)
		update_dns "$1" "$2" "$3"
		ret=$?
		;;
	*)
		echo "unknown set cmd: $@"
		usageset
		;;
	esac

	return $ret
}

set_redirect ()
{
	if [ -z "$2" ]; then
		usageset_redirect_ftpports
		usageset_redirect_httpports
		return 1
	fi

	load_ifname
	if [ "$bridgemode" = "0" ]; then
		i18n_no_iports_in_proxymode="`gettext 'Proxy Mode does not allow you to input redirection ports.'`"
		echo "$i18n_no_iports_in_proxymode"
		return 0
	fi

	if [ "$1" = "ftpports" ]; then
		ftp_daemon_stop=true
		if [ -e $FTP_PID_FILE ]; then
			ftp_daemon_stop=false
		fi

		if [ "true" == "$ftp_daemon_stop" ]; then
			echo "FTP daemon is stopped."
			set_param /etc/iscan/IWSSPIProtocolFtp.pni redirect_ftp_ports_before_bypass "$2"
		else
			set_param /etc/iscan/IWSSPIProtocolFtp.pni interested_port "$2"
			load_redirect_rules
			/etc/iscan/S99ISftp stop 2>/dev/null >/dev/null
			/etc/iscan/S99ISftp start
		fi
		ret=$?
	elif [ "$1" = "httpports" ]; then
		set_interested_port=$2
		redirect_pots=`echo "$2"| tr -s ';' ' '`
		http_default_port=80
		find_http_default_port=0

    	for i in $redirect_pots; do
			if [ "$i" = "$http_default_port" ];then
				find_http_default_port=1
			fi
        done

		if [ $find_http_default_port = 0  ];then
			set_interested_port="80;$set_interested_port"
		fi

		http_daemon_stop=true
		if [ -e $HTTP_PID_FILE ]; then
			http_daemon_stop=false
		fi

		if [ "true" == "$http_daemon_stop" ]; then
			echo "HTTP daemon is stopped."
			set_param /etc/iscan/IWSSPIProtocolHttpProxy.pni redirect_http_ports_before_bypass "$set_interested_port"
		else
			set_param /etc/iscan/IWSSPIProtocolHttpProxy.pni interested_port "$set_interested_port"
			load_redirect_rules
			/etc/iscan/S99ISproxy stop 2>/dev/null >/dev/null
			/etc/iscan/S99ISproxy start
		fi
		ret=$?
    elif [ "$1" = "httpsports" ]; then
        set_interested_port=$2
        redirect_pots=`echo "$2"| tr -s ';' ' '`
        https_default_port=443
        find_https_default_port=0

        for i in $redirect_pots; do
#			echo $i
#			echo $https_default_port
            if [ "$i" = "$https_default_port" ];then
                    find_https_default_port=1
            fi
#			echo $find_https_default_port
		done

		if [ $find_https_default_port = 0  ];then
			set_interested_port="443;$set_interested_port"
		fi

		http_daemon_stop=true
		if [ -e $HTTP_PID_FILE ]; then
			http_daemon_stop=false
		fi

		if [ "true" == "$http_daemon_stop" ]; then
			echo "HTTP daemon has stopped."
			set_param /etc/iscan/IWSSPIProtocolHttpProxy.pni redirect_https_ports_before_bypass "$set_interested_port"
		else
			set_param /etc/iscan/IWSSPIProtocolHttpProxy.pni interested_https_port "$set_interested_port"
			load_redirect_rules
			/etc/iscan/S99ISproxy stop 2>/dev/null >/dev/null
			/etc/iscan/S99ISproxy start
		fi
		ret=$?
	else
		dbg 2 "command $1 is not support for redirection"
		ret=1
	fi

	return $ret
}

usageset_hostname()
{
	echo "usage: set hostname <hostname>"
}

usageset_webserver ()
{
	echo "usage: set webserver port <port>"
}

usageset_dns ()
{
	echo "usage: set dns <dns1> [dns2]"
}

usageset_routedel ()
{
	echo "usage: set routedel <ifname> <index>"
}

usageset ()
{
	usageset_webserver
	usageset_uiroute
	usageset_routedel
#	usageset_routeadd
	usageset_hostname
	usageset_dns
}

get_webserver_port ()
{
	get_param $ISCANINI iscan_web_server
	WEBSERVER_PORT="$gvalue"
    WEBSERVER_PORT=`echo $WEBSERVER_PORT | cut -d':' -f2`

	return 0
}

# set webserver port 1812
set_webserver_port ()
{
	local port="$1"

	# set webserver port 1812
	if [ -z "$port" ]; then
		usageset_webserver
		return 1
	fi

	set_param $ISCANINI iscan_web_server "$port"
	sed -i "s/Connector port=\"[0-9]*\"/Connector port=\"$port\"/g" /usr/iwss/AdminUI/tomcat/conf/server.xml
	/etc/init.d/IWSS restart
	/etc/iptables restart

	return $?
}

get_br_mngt ()
{
	load_ifname

	if [ "$((VID))" -gt 0 ]; then
		MNGT="$BRI.$VID"
	else
		MNGT=$BRI
	fi
}

# route config <IFNAME> <INDEX> <ADDRESS> <NETMASK> <GATEWAY> <TMOPTION>
# if index = -1 ; add route record
# if index > 0 ; modify route record with index
#	route_conf $index $ADDRESS $NETMASK $GATEWAY $TMOPTION
#	route_conf $IFNAME $index $ADDRESS $NETMASK $GATEWAY $TMOPTION
route_conf()
{
	add_to_rtfile $1 $2 $3 $4 $5 $6
	return $ret
}

# route del <ifname> <index>
del_route ()
{
#	get_br_mngt
	del_item_rtfile $1 $2
	ret=$?
	return $ret
}

usageset_uiroute()
{
#	echo "usage: set route '<dev>:<address>:<gateway>:<netmask>'"
	echo "usage: set uiroute ifname <ifname> [index <indexname>] '<address>|<gateway>|<netmask>'"
}

usageset_routeadd()
{
	echo "usage: set route <ifname> <address> <netmask> <gateway>"
}

usageset_redirect_ftpports ()
{
	echo "usage: set redirect ftpports \"<1-65535>;<1-65535>;...\""
}

usageset_redirect_httpports ()
{
	echo "usage: set redirect httpports \"<1-65535>;<1-65535>;...\""
}

usageset_eth_static ()
{
	echo "usage: set ethernet static ipv4/ipv6 <ifname> <ip address> <netmask> <gateway> [vid]"
}

usageset_eth_dhcp ()
{
	echo "usage: set ethernet dhcp  <ifname> [vid]"
}

usageset_eth_usage ()
{
	echo "usage: set ethernet <static | dhcp >"
}

write_header_ifcfg ()
{
	if [ -z "$1" ]; then
		echo "write_header_ifcfg:Error missing config name"
		return 1
	fi

	dbg 1 "generate $1"
	echo "# "`date`" This file is generated by IWSVA system." > $1
	return 0
}

add_hwaddr ()
{
	if [ -z "$2" ] ; then
		return 0
	fi

	get_param $NETINI HWADDR_$2
	if [ -z "$gvalue" -a  "$2" != "br0" -a "$2" != "br1"  ]; then
		gvalue=`/sbin/ifconfig $2 | grep HWaddr | awk '{split($0,mac_address,"HWaddr ");print mac_address[2];}'`
	fi

	add_param $1 "HWADDR" "$gvalue"
	return 1
}

create_brifcfg ()
{
	local ifcfgname="$cfgdir/ifcfg-$1"
	if [ -z "$1" ]; then
		echo "write_header_ifcfg:Error missing config name"
		return 1
	fi

	write_header_ifcfg $ifcfgname
	echo "DEVICE=$1" >> $ifcfgname
	if [ -z "$2" ]; then
	    echo "BRIDGE=$BRI" >> $ifcfgname
	else
	    echo "BRIDGE=$2" >> $ifcfgname
	fi

	echo "ONBOOT=yes" >> $ifcfgname
	echo "TYPE=Ethernet" >> $ifcfgname
	echo "BOOTPROTO=none" >> $ifcfgname
	add_hwaddr $ifcfgname $1
	return 0
}

create_duelbrifcfg ()
{
    local ifcfgname="$cfgdir/ifcfg-$1"
    if [ -z "$1" ]; then
            echo "write_header_ifcfg:Error missing config name"
            return 1
    fi

    write_header_ifcfg $ifcfgname
    echo "DEVICE=$1" >> $ifcfgname
    echo "BRIDGE=$BRII" >> $ifcfgname
    echo "ONBOOT=yes" >> $ifcfgname
    echo "TYPE=Ethernet" >> $ifcfgname
    echo "BOOTPROTO=none" >> $ifcfgname
    add_hwaddr $ifcfgname $1

    return 0
}

remove_vlan ()
{
	# only bridge mode have vlan id
	if [ ! -d /proc/net/vlan ]; then
		return 0
	fi

	local VIDS="`ls /proc/net/vlan | grep $BRI`"
	for i in  $VIDS ; do
		vconfig rem $i  2>/dev/null
	done
	return 0
}

rename_sroute ()
{
	local vdevice="$1"
	src_rt="`find $cfgdir | grep "\/route-" | head -n 1`"
	if [ -z "$src_rt" -o ! -f "$src_rt" ]; then
		return 0;
	fi

	dst_rt="$cfgdir/route-$vdevice"
	if [ -n "$src_rt" -a ! "$src_rt" = "$dst_rt" ]; then
		mv -f $src_rt $dst_rt
		dbg 2 "rename $src_rt -> $dst_rt"
	fi
}

setip ()
{
	local NVID;
	local cfgvid;

	PROTO="$1"
	BOOTPROTO=""
	BOOT="$2"
	IPADDR=""
	NETMASK=""
	IPV6INIT=""
	IPV6ADDR=""
	IPV6PREFIX=""
	DHCPV6C=""
	IPV6_DEFAULTGW=""
	DEVICE="$3"

	if [ "$BOOT" = "static" ]; then
	    if [ "$PROTO" = "ipv4" ]; then
			IPADDR="$4"
			NETMASK="$5"
			NVID="$(($6))"
		elif [ "$PROTO" = "ipv6" ]; then
			IPV6ADDR="$4/$5"
			IPV6PREFIX="$5"
			NVID="$(($6))"
			IPV6INIT="yes"
	    fi
	elif [ "$BOOT" = "dhcp" ]; then
	    NVID="$(($4))"
	elif [ "$BOOT" = "dhcpv6" ]; then
	    NVID="$(($4))"
	else
	    NVID=0
	fi

	load_ifname

	if [ "$NVID" -gt 0 -a "$bridgemode" -eq 0 ]; then
		i18n_no_vid_with_proxy_mode="`gettext 'Proxy Mode does not allow you to input a VLan ID.'`"
		echo "$i18n_no_vid_with_proxy_mode"
		return 1;
	fi

	stopnet
	is_bridge_enable
	bridgemode="$?"

	if [ "$bridgemode" -eq 1 ]; then
		ifname="$BRI"
		TYPE=Bridge
	else
		ifname="$3"
		TYPE=ETHERNET
	fi

	if [ -f $cfgdir/ifcfg-$ifname ]; then
	    if [ "$PROTO" = "ipv4" ]; then
			get_param $cfgdir/ifcfg-$ifname IPV6ADDR
			if [ -n "$gvalue" ]; then
			    IPV6ADDR=$gvalue
			    IPV6PREFIX="`echo $IPV6ADDR | cut -d'/' -f2`"
			fi

			get_param $cfgdir/ifcfg-$ifname DHCPV6C
			if [ -n "$gvalue" ]; then
				DHCPV6C=$gvalue
			fi

			get_param $cfgdir/ifcfg-$ifname IPV6INIT
			if [ -n "$gvalue" ]; then
				IPV6INIT=$gvalue
			fi

			BOOTPROTO="$BOOT"
	    elif [ "$PROTO" = "ipv6" ]; then
			get_param $cfgdir/ifcfg-$ifname IPADDR
			if [ -n "$gvalue" ]; then
			    IPADDR="$gvalue"
			fi

			get_param $cfgdir/ifcfg-$ifname NETMASK
			if [ -n "$gvalue" ]; then
			    NETMASK="$gvalue"
			fi
			get_param $cfgdir/ifcfg-$ifname BOOTPROTO
			if [ -n "$gvalue" ]; then
			    BOOTPROTO="$gvalue"
			fi
	    fi
        else
            BOOTPROTO="$BOOT"
	fi

	get_param $cfgdir/ifcfg-$ifname IPV6INIT
	if [ -n "$gvalue" ]; then
		IPV6INIT=$gvalue
	fi

	get_param $cfgdir/ifcfg-$ifname IPV6_DEFAULTGW
	if [ -n "$gvalue" ]; then
		IPV6_DEFAULTGW=$gvalue
	fi

	rm -f $cfgdir/ifcfg-$ifname\.*

	if [ "$bridgemode" -eq 1 ]; then
        get_param $NETINI vid
        if [ -n "$gvalue" -a "$gvalue" != "$NVID" ]; then
            local temp_intf=""
            if [ "$gvalue" = "0" ]; then
                temp_intf="br0"
            else
                temp_intf="br0.$gvalue"
            fi
            destroy_route $temp_intf
        fi
    fi

	VID="$NVID"
	if [ "$((NVID))" -gt 0 ] ; then
		VDEVICE="${ifname}.$VID"
		cfgvid="$cfgdir/ifcfg-$VDEVICE"
	else
		VDEVICE="${ifname}"
	fi

	set_param $NETINI vid $VID
#	if [ "$bridgemode" -eq 1 ]; then
#		rename_sroute $VDEVICE
#	fi

	cfgname="$cfgdir/ifcfg-${ifname}"
	write_header_ifcfg $cfgname
	set_param $cfgname "DEVICE" "$ifname"
	set_param $cfgname "ONBOOT" "yes"
	set_param $cfgname "TYPE" "$TYPE"

	if [ "$VID" -eq 0 ]; then # generate ifcfg-xxx
	    set_param $cfgname "BOOTPROTO" "$BOOTPROTO"
	    set_param $cfgname "NETMASK" "$NETMASK"
	    set_param $cfgname "IPADDR" "$IPADDR"
	    set_param $cfgname "IPV6INIT" "$IPV6INIT"
	    set_param $cfgname "IPV6ADDR" "$IPV6ADDR"
	    set_param $cfgname "IPV6_DEFAULTGW" "$IPV6_DEFAULTGW"
#		add_hwaddr $cfgname $DEVICE
	    add_hwaddr $cfgname ${ifname}
	else # generate ifcfg-xxx.vid
	    set_param $cfgname "ISALIAS" "no"
	    set_param $cfgname "BOOTPROTO" "none"
	    write_header_ifcfg $cfgvid
	    set_param $cfgvid "DEVICE" "$VDEVICE"
	    set_param $cfgvid "ONBOOT" "yes"
	    set_param $cfgvid "TYPE" "ETHERNET"
	    set_param $cfgvid "VLAN" "yes"
	    set_param $cfgvid "BOOTPROTO" "$BOOTPROTO"
	    set_param $cfgvid "NETMASK" "$NETMASK"
	    set_param $cfgvid "IPADDR" "$IPADDR"
	    set_param $cfgvid "IPV6INIT" "$IPV6INIT"
	    set_param $cfgvid "IPV6ADDR" "$IPV6ADDR"
	    set_param $cfgvid "IPV6_DEFAULTGW" "$IPV6_DEFAULTGW"
	    set_param $cfgvid "ISALIAS" "no"
	fi

	# set default gateway
	if [ "$BOOT" = "static" ]; then
	    #set_param $CFGNETWORK GATEWAY "$GATEWAY"
        echo >/dev/null;
	elif [ "$BOOT" = "dhcp" ]; then
	    # remove the default gateway while bootproto is dhcp.
	    sed -i "/^GATEWAY=.*/d" $CFGNETWORK
	elif [ "$BOOT" = "dhcpv6" ]; then
	    # remove the IPV6 default gateway while bootproto is dhcpv6
	    #set_param $cfgvid "IPV6INIT" "yes"
	    sed -i "/^IPV6_DEFAULTGW=.*/d" $CFGNETWORK
	    sed -i "/^IPV6ADDR=.*/d" $CFGNETWORK
	    set_param $cfgname "DHCPV6C" "yes"
	fi

	startnet

    # After setting ip address, check the route table item
    iwsva_check_route_table

    #Because of VLAN in bridge mode, the interface name may be changed
    #So reload the related firewall rule.
    if [ "$bridgemode" -eq 1 ]; then
        sudo /usr/iwss/iwsvafw.sh service_all
    fi

	# after ip configued, trigger ip change event.
	if [ "$BOOT" = "static" ]; then
		hook_ip_changed
	fi

    # Restart the wccp daemon after reconfigure ip address
    wccp_mode=`/usr/bin/rwconfig -r -d '=' -f /etc/iscan/IWSSPIProtocolHttpProxy.pni -s 'transparency'`
    if [ XX$wccp_mode = "XXwccp" ] ; then
	    /usr/iwss/S99ISWCCPd restart
    fi

	return 0
}

getinfo()
{
	ret=0
	dbg 2 "get $@"

	action="$1"
	shift;

	case "$action" in
	bridgemode)
		is_bridge_enable
		if [ $? -eq 1 ] ; then
			echo "on"
		else
			echo "off"
		fi
		;;
	bridge)
		if [ "$1" = "interface" ]; then
			load_ifname
			echo "ifname_internal=$INT"
			echo "ifname_external=$EXT"
			if [ "$OTH" != "" ]; then
				echo "ifname_others=$OTH"
			fi
			return 0;
		else
			usage_getinfo $@
		fi
		;;
	mgmt_interface)
		show_mgmt_info
		;;
	data_interface)
		show_data_info
		;;
	proxy)
		if [ "$1" = "interface" ]; then
			load_ifname
			echo "ifname_proxy=$INT"
			return 0;
		else
			usage_getinfo $@
		fi
		;;
	# get ip <interface|address|netmask|gateway>
	ip)
		case "$1" in
		interface)
			show_mngt vid nvid
			;;
		address)
			show_mngt_info $1
			;;
		ipv6addr)
			show_mngt_info $1
			;;
		bootproto)
			get_bootproto
			echo "$BOOTPROTO"
			;;
		ipv6bootp)
			get_bootproto
			echo "$DHCPV6C"
			;;
		vid)
			show_vid
			;;
		netmask)
			show_mngt_info $1
			;;
		gateway)
			show_gateway
			;;
	    datagatewayv6)
	        show_gateway_v6 data
	        ;;
	    mgmtgatewayv6)
	        show_gateway_v6 mgmt
	        ;;
		mgmtinterfaceipv6)
			show_mgmt_interface_v6
			;;
		*)
			usage_ipinfo $@
			return 1;
		esac

		return 0;
		;;
	# get real_nics
	real_nics)
		get_real_nic
		echo "$grealnics"
		;;
	# get real_nics_count
	# hardware information
	real_nics_count)
		get_real_nic_count
		echo "$?"
		;;
	# get interfaces
	# the active interfaces
	interfaces)
		echo "2"
		;;
	route)
		if [ -z "$1" -o -z "$2" ]; then
			usage_route
			return 1
		fi

		#alway get from bridge interface for WEB UI
#		get_br_mngt

   		local rt_interface="$2"

	    # if the vlan is configured, change interface to VLAN interface
	    if [ $2 = "br0" ]; then
	        get_param $NETINI vid
	        if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
	            rt_interface="br0.$gvalue"
	        fi
	    fi

		judge_interface $rt_interface
		if [ $? -eq 1 ] ; then
			usage_route
			return 1
		fi

		# get route count <ifname>
		if [ "$1" = "count" ]; then
			get_count_from_rtfile $rt_interface
			echo -n "$?"
			ret=0
		# get route value <ifname> <index>
		elif [ "$1" = "value" -a ! "x$3" = "x" ]; then
			get_from_rtfile $rt_interface $3
			ret=$?
		# get route option <ifname> <index>
		elif [ "$1" = "option" -a ! "x$3" = "x" ]; then
			get_opt_from_rtfile $rt_interface $3
			ret=$?
		else
			usage_route
			return 1;
		fi
		;;
	mngt)
		show_mngt vid
		;;
	redirect)
		if [ "$1" = "ftpports" ]; then
			get_param /etc/iscan/IWSSPIProtocolFtp.pni interested_port
		elif [ "$1" = "httpports" ]; then
			get_param /etc/iscan/IWSSPIProtocolHttpProxy.pni interested_port
		else
			usageget_redirect
			return 1

		fi

		if [ -n "$gvalue" ]; then
			echo "$gvalue"
		fi
		;;
	bonding)
		get_param $NETINI bonding
		if [ -n "$gvalue" ]; then
			echo "$gvalue"
		fi
		;;
	*)
		usage_getinfo $@
		;;
	esac

	return $ret
}

show_gateway ()
{
	DEFAULTGW="`/sbin/ip route | grep default | cut -d' ' -f3`"
	echo -n "$DEFAULTGW"
}

show_gateway_v6 ()
{
    MGMT_INTERFACE=""
    DATA_INTERFACE=""
    INTERFACE=""

    if [ "data" = "$1" ];then
    	get_data_interface
        INTERFACE="$DATA_INTERFACE"
    else
        get_mgmt_interface
        INTERFACE="$MGMT_INTERFACE"
    fi

    if [ -z "$INTERFACE" ]; then
            return 1;
    fi

    GATEWAY="`/sbin/ip -6 route | grep default | grep "$INTERFACE" |cut -d' ' -f3`"
    echo -n "$GATEWAY"
    return 0
}

show_mgmt_interface_v6()
{
	mgmt_ipv6_addr=""
	get_mgmt_interface

	local MGNT="$MGMT_INTERFACE"
	if [ -n "$MGNT" ];then
		mgmt_ipv6_addr="`/sbin/ip -6 addr show $MGNT | grep "inet6.*global" | awk '{ print $2 }'`"
	fi

	echo -n "$mgmt_ipv6_addr"
}

get_bootproto ()
{
	local ifcfg=""
	get_mngt vid nvid

	ifcfg="$cfgdir/ifcfg-$MNGT"
	if [ ! -f "$ifcfg" ]; then
		dbg 3 "$ifcfg is missing"
		return 0;
	fi

	get_param $ifcfg BOOTPROTO
	BOOTPROTO="$gvalue"
	get_param $ifcfg DHCPV6C
	DHCPV6C="$gvalue"
	return 0;
}

show_bootproto ()
{
	get_bootproto
	echo -n ${BOOTPROTO}
}

# get_mngt_info <address|netmask|ipv6addr>
get_mngt_info ()
{
	local opt=0
	local fdnum=4

	# export out of function
	IPINFO=""
	if [ "$1" = "netmask" ]; then
	    opt=1
	elif [ "$1" = "address" ]; then
	    opt=2
	    fdnum=2
	else
	    opt=3
	fi

	get_mngt vid nvid
	ifcfg="$cfgdir/ifcfg-$MNGT"
	if [ ! -f "$ifcfg" ]; then
		dbg 3 "$ifcfg is missing"
		return 0;
	fi

	case $opt in
	1|2)
		get_param $ifcfg BOOTPROTO
		BOOTPROTO="$gvalue"
		if [ "$BOOTPROTO" = "dhcp" ]; then
			if [ "$opt" -eq 2 ]; then
			    IPINFO="`ifconfig $MNGT | grep "inet addr:" | cut -d":" -f$fdnum | cut -d' ' -f1`"
			else
			    IPINFO="`ifconfig $MNGT | grep "Mask:" | cut -d":" -f4 | cut -d' ' -f1`"
			fi
		elif [ "$BOOTPROTO" = "static" ]; then
			if [ "$opt" -eq 2 ]; then
			    get_param $ifcfg IPADDR
			else
			    get_param $ifcfg NETMASK
			fi
			IPINFO="$gvalue"
		fi
		;;
	3)
		get_param $ifcfg DHCPV6C
		DHCPV6C="$gvalue"
		if [[ "${DHCPV6C}" = [Yy1]* ]]; then
		    IPINFO="`/sbin/ip -6 addr show $MNGT | grep "inet6.*global" | awk '{ print $2 }'`"
		else
		    get_param $ifcfg IPV6ADDR
		    IPINFO="$gvalue"
		fi
		;;
	esac
	return 0
}

show_mngt_info ()
{
	get_mngt_info $@
	echo -n "$IPINFO"
	return 0
}

show_mgmt_info ()
{
	MGMT_INTERFACE=""
	get_mgmt_interface
	if [ "$MGMT_INTERFACE" != "" ]; then
	echo -n "$MGMT_INTERFACE"
	return 0
	fi
	echo -n "$MGMT_INTERFACE"
	return 1
}

show_data_info ()
{
	DATA_INTERFACE=""
	get_data_interface

	#for spport VLAN
	DATA_INTERFACE=`echo "$DATA_INTERFACE" | cut -d'.' -f 1`

	echo -n "$DATA_INTERFACE"
	return 0
}

show_vid ()
{
	load_ifname
	echo -n "$VID"
	return 0
}

# get_mngt [vid] [nvid]
get_mngt ()
{
	load_ifname

	if [ $bridgemode -eq 1 ]; then
	 	MNGT="$BRI"
	else
	 	MNGT="$PROXY"
	fi

	if [ "$1" = "vid" ]; then
		if [ ! "$2" = "nvid" -o "$VID" -gt 0 ]; then
			MNGT="$MNGT.$VID"
		fi
	fi
	return 0
}

# show_mngt [vid] [nvid]
show_mngt ()
{
	get_mngt $1 $2
	echo -n "$MNGT"
	return 0;
}

ECHOE="echo -e"

usage_route ()
{
	$ECHOE "usage: get route value <ifname> <index>"
	$ECHOE "usage: get route count <ifname>"
	$ECHOE "usage: get route option <ifname> <index>"
}

usage_ipinfo()
{
	$ECHOE "usage: get ip address"
	$ECHOE "usage: get ip netmask"
	$ECHOE "usage: get ip gateway"
	$ECHOE "usage: get ip interface"
	$ECHOE "usage: get ip bootproto"
	$ECHOE "ex:\teth0\n"
}

usageget_redirect ()
{
	$ECHOE "usage: set redirect <httpports|ftpports>"
}

usage_getinfo()
{
	$ECHOE "usage: get bridgemode"
	$ECHOE "ex:\ton | off\n"
	$ECHOE "usage: get bridge interface"
	$ECHOE "ex:\nifname_internal=eth0"
	$ECHOE "ifname_external=eth1\n"
	$ECHOE "usage: get proxy interface"
	$ECHOE "ex:\teth1"

	usage_ipinfo
	usage_route
	usageget_redirect
}

restore_proxy_setting()
{
	return 0;
}

is_bridge_enable()
{
	get_param $ISCANINI bridgemode

	if [ -z "$gvalue" ]; then
		return 0;
	fi

	if [ "$gvalue" -eq 0 ]; then
		return 0;
	fi

	return 1
}

showinfo()
{
	verbose=2
	ret=0

	case $1 in
	all)
		/sbin/ifconfig | grep -v '^ ' | grep "^[a-z]" | cut -d' ' -f1 | grep -v "lo"
		;;
	status)
		is_bridge_enable
		return $?
		;;
	*)
		/sbin/ifconfig | grep -v '^ ' | grep "^[a-z]" | cut -d' ' -f1 | grep -v "lo" | grep -v "bridge"
		;;
	esac

	return $ret
}

do_ifrename()
{
	if [ -f "/etc/iftab" ]; then
		/sbin/ifrename -p -t
	fi

	return $?
}

startnet()
{
	if [ $debug -eq 0 ]; then
#		create_mgt_cfg
		do_ifrename
		$NETSH_SYS start

		if [ $bridgemode -eq 1 ]; then
			if [ "$enable_stp" == "yes" ] ; then
				brprio=`/usr/bin/rwconfig -r -f /etc/iscan/network.ini -d '=' -s stp_brprio`
				/usr/sbin/brctl setbridgeprio ${BRI} ${brprio} >/dev/null 2>&1
				/usr/sbin/brctl stp ${BRI} on
			else
		       	/usr/sbin/brctl stp ${BRI} off
			fi

			ethtool -K $BRI tso off
			ethtool -K $BRI gso off
			ethtool -K $BRI ufo off

			if [ $VID -ne 0 ]; then
				ethtool -K $BRI.$VID gso off
				ethtool -K $BRI.$VID gro off
			fi
		fi

        # Shutdown internal interface in HA mode
        # Setup bridge settings and STP
        HA_CONF=/etc/iscan/cluster.ini
        if [[ ( -f ${HA_CONF} && \
            "x$(F_ini_get $HA_CONF 'cluster' '^cluster_id')" != "x" ) ]] ; then
            if [ "$enable_stp" == "yes" ] ; then
                /usr/sbin/brctl setfd ${BRI} 4
                /usr/sbin/brctl sethello ${BRI} 1
                /usr/sbin/brctl setmaxage ${BRI} 6
                /usr/sbin/brctl setportprio ${BRI} ${INT} 2 >/dev/null 2>&1
                /usr/sbin/brctl setportprio ${BRI} ${EXT} 1 >/dev/null 2>&1
            fi

            ROLE=`/usr/iwss/haagntcmd GetRole`
			if [ "$ROLE" != "parent" ] ; then
				# /sbin/ifconfig $INT down
				/usr/sbin/brctl delif ${BRI} $INT >/dev/null 2>&1
				/sbin/ifconfig $INT down >/dev/null 2>&1
				/sbin/ifconfig $INT up >/dev/null 2>&1
			else
				CLUSTER_IP="$(F_ini_get $HA_CONF 'cluster' '^cluster_ip')"
				MGMT_MASK=`ip addr show ${MGMT} | grep inet | cut -d '/' -f 2 | cut -f 1  -d ' '`
				/sbin/ip addr add dev ${MGMT} ${CLUSTER_IP}/${MGMT_MASK}
	        fi
        fi
	else
		dbg 1 "debug: skip $NETSH_SYS start"
	fi
}

create_mgt_cfg ()
{
 	# robin suport management port
	get_param $NETINI ifname_mgt
	if [ -n "$gvalue" ]; then
		mgt=$gvalue
	else
		# skip mgt function
		return 0;
	fi

	get_param $NETINI mgt_ip
	if [ -n "$gvalue" ]; then
		mgt_ip=$gvalue
	else
		# skip mgt function
		return 0;
	fi

	get_param $NETINI mgt_netmask
	if [ -n "$gvalue" ]; then
		mgt_netmask=$gvalue
	else
		# skip mgt function
		return 0;
	fi

	get_param $NETINI mgt_gw
	if [ -n "$gvalue" ]; then
		mgt_gw=$gvalue
	fi

	#ifconfig $mgt down
	#ifconfig $mgt up $mgt_ip $mgt_netmask
	#route del -net 0.0.0.0
	#route add -net 0.0.0.0 netmask 0.0.0.0 gw $mgt_gw

	cfgmgtname="$cfgdir/ifcfg-$mgt"

	echo "# Create by Experimental function of IWSVA" > $cfgmgtname
	add_param $cfgmgtname "DEVICE" "$mgt"
	add_param $cfgmgtname "ONBOOT" "yes"
	add_param $cfgmgtname "TYPE"  "ETHERNET"
	add_param $cfgmgtname "IPADDR"  "$mgt_ip"
	add_param $cfgmgtname "NETMASK"  "$mgt_netmask"

	# set default gateway
	if [ -n "$mgt_gw" ]; then
		set_param $CFGNETWORK GATEWAY "$mgt_gw"
	fi

	# generate route-xxx
	#/etc/sysconfig/network-scripts/ifup-routes $mgt
	#rtmgtname=/etc/sysconfig/network-scripts/routes-$mgt
	#echo "# Create by Experimental function of IWSVA" > $rtmgtname
	#add_param $rtmgtname "ADDRESS0"  "$mgt_addres0"
	#add_param $rtmgtname "NETMASK0"  "$mgt_netmask"
	#add_param $rtmgtname "GATEWAY0"  "$mgt_gw"
}

stopnet()
{
	if [ $debug -eq 0 ]; then
		$NETSH_SYS stop
		vlan_stop
	else

		dbg 1 "debug: skip $NETSH_SYS stop"
	fi
}

stop ()
{
	uninit_bonding
	stopnet
}

status ()
{
	$NETSH_SYS status
}

start()
{
	init_startup
	set_kernel_params

	if [ $bridgemode -eq 1 ]; then
		turnbridge on
	else
		turnbridge off

		# for proxy mode, turn off tso & gso for app control purpose
		for inf in $PROXY_ALL; do
			/sbin/ethtool -K $inf tso off
			/sbin/ethtool -K $inf gso off
		done
	fi

    # reset the eth nic option
    for NIC in `ls /sys/class/net | grep '^eth[0-9]*$'`; do
         eth_opts=`/usr/bin/iniconfig -r -f /etc/iscan/network.ini -s network -o ${NIC}_opts`
         [ XX != "$eth_opts"XX ] &&  /sbin/ethtool -s $NIC $eth_opts
    done

    if [ -e /etc/iscan/.dw_reboot_with_timesync ]; then
        /etc/iscan/AdminUI/deployment_helper.sh SyncNTPServer
    fi

    rm -rf /etc/iscan/.dw_reboot*

    return 0;
}

# for debug the shell script
shell_debug_log()
{
    local shell_debug_log_on="no";
    local shell_debug_log_file="/etc/iscan/log/shell_running.log";

    if [ "$shell_debug_log_on" = "yes" -a -n "$1" ]; then
        running_date=`date`
        echo "$running_date [$1]" >> $shell_debug_log_file
    fi
}

destroy_routev4()
{
	if [ -z $1 ]; then
		return 1;
	fi

	routecfg="$cfgdir/route-$1"
	if [ ! -f $routecfg ]; then
		return 0;
	fi

    sed -i "s/^ADDRESS/TMPADDRESS/g" $routecfg
    sed -i "s/^NETMASK/TMPNETMASK/g" $routecfg
    sed -i "s/^GATEWAY/TMPGATEWAY/g" $routecfg
	sed -i "s/^TMOPTION.*$/TMPTMOPTION=2/g" $routecfg
}

destroy_routev6()
{
	if [ -z $1 ]; then
		return 1;
	fi

	route6cfg="$cfgdir/route6-$1"
	if [ ! -f $route6cfg ]; then
	    return 0
	fi

	# comment all of them
	sed -i "s/\(.*\)/\#\1/g" $route6cfg
}

#destory route file
#destory_route <ifname>
destroy_route()
{
	if [ -z $1 ]; then
		return 1;
	fi

	destroy_routev4 $1
	destroy_routev6 $1
}

# usage: <cfg> <idx> <network address> <netmask> <gateway> [tmoption]
add_tmp_entry_route()
{
	local rtcfg="$1"
	local idx="$2"
	local naddr="$3"
	local mask="$4"
	local gw="$5"
	local opt="$6"
	echo "TMPADDRESS$idx=$naddr" >> $rtcfg
	echo "TMPNETMASK$idx=$mask" >> $rtcfg
	echo "TMPGATEWAY$idx=$gw" >> $rtcfg
#	if [ -n "$opt" ] ; then
#	echo "TMPTMOPTION$idx=$opt" >> $rtcfg
#	fi
	echo "TMPTMOPTION$idx=2" >> $rtcfg
	return $?
}

# usage: <cfg> <idx> <network address> <netmask> <gateway> [tmoption]
add_entry_route()
{
	local rtcfg="$1"
	local idx="$2"
	local naddr="$3"
	local mask="$4"
	local gw="$5"
	local opt="$6"

	echo "ADDRESS$idx=$naddr" >> $rtcfg
	echo "NETMASK$idx=$mask" >> $rtcfg
	echo "GATEWAY$idx=$gw" >> $rtcfg

	if [ -n "$opt" ] ; then
		echo "TMOPTION$idx=$opt" >> $rtcfg
	fi
	return $?
}

# usage: <ifname> <index> <network address> <netmask> <gateway>
# for ipv6 route ,like route6-eth0
add_to_rtfile()
{
	local INDEX="$2"
	local ADDRESS="$3"
	local NETMASK="$4"
	local GATEWAY="$5"
	local TMOPTION="$6"
	local found=0
	local rtnum=0
	local route6=0

	echo $ADDRESS | grep -q ':' && route6=1 || route6=0
	# write to route-eth*
	if [ $route6 -eq 0 ];then

		rtfile="$cfgdir/route-$1"

		if [ ! -f $rtfile ]; then
			add_tmp_entry_route $rtfile $rtnum $ADDRESS $NETMASK $GATEWAY $TMOPTION
			return 0
		fi
		. $rtfile

		local num=0
		echo "#`date`: Generate by IWSVA network scripts" > $rtfile
		while [ "x$(eval echo '$'ADDRESS$num)x" != "xx" -o "x$(eval echo '$'TMPADDRESS$num)x" != "xx" ]; do
			if [ "$INDEX" -eq $num -o "$ADDRESS" = "$(eval echo '$'ADDRESS$num)" -o "$ADDRESS" = "$(eval echo '$'TMPADDRESS$num)" ]; then
				add_tmp_entry_route $rtfile $rtnum $ADDRESS $NETMASK $GATEWAY $TMOPTION
				found=1
			else
				ADDRESS_O="$(eval echo '$'ADDRESS$num)"
				NETMASK_O="$(eval echo '$'NETMASK$num)"
				GATEWAY_O="$(eval echo '$'GATEWAY$num)"
				TMOPTION_O="$(eval echo '$'TMOPTION$num)"

				if [ -n "$ADDRESS_O" ]; then
					add_entry_route $rtfile $rtnum $ADDRESS_O $NETMASK_O $GATEWAY_O $TMOPTION_O
				else
					ADDRESS_O="$(eval echo '$'TMPADDRESS$num)"
					NETMASK_O="$(eval echo '$'TMPNETMASK$num)"
					GATEWAY_O="$(eval echo '$'TMPGATEWAY$num)"
					TMOPTION_O="$(eval echo '$'TMPTMOPTION$num)"
					add_tmp_entry_route $rtfile $rtnum $ADDRESS_O $NETMASK_O $GATEWAY_O $TMOPTION_O
				fi
			fi

			rtnum=$(($rtnum+1))
			num=$((num+1));
		done

		if [ "$found" -eq 0 ]; then
			add_tmp_entry_route $rtfile $rtnum $ADDRESS $NETMASK $GATEWAY $TMOPTION
			dbg 2 "add_tmp_entry_route $rtfile $rtnum $ADDRESS $NETMASK $GATEWAY $TMOPTION"
		fi
	else
		# write to route6-eth*
		echo "write to route6"
		rtfile="$cfgdir/route-$1"

		if [ ! -f $rtfile ]; then
			rt4num=0
		else
			rt4num=`cat $rtfile | grep 'ADDRESS' | wc -l`
		fi

		rtfile="$cfgdir/route6-$1"
		tmprtfile="$cfgdir/tmprt6"
		> $tmprtfile
		tmpstr="$ADDRESS/$NETMASK via $GATEWAY"

		if [ $INDEX -eq -1 ];then
			if [ ! -f $rtfile ];then
				echo "#`date`: Generate by IWSVA network scripts" > $rtfile
			fi

        	if cat $rtfile | grep '^[[:space:]]*[0-9a-fA-F:]*/[0-9]*' | grep -q "$tmpstr" ; then
				echo "Configured route already exists in the current route table."
            	return 1
        	fi

			echo "TMOPTION=$(eval echo '$'TMOPTION)" >> $rtfile
    		echo "$tmpstr" >> $rtfile
			return 0
		else
			echo "change index"
			INDEX=$(($INDEX-$rt4num))
			echo "$INDEX"

			if [ ! -f $rtfile ];then
				echo "error:cant change index"
				return 0
			fi

			rt6num=`cat $rtfile |sed 's/#.*//g' | grep 'via' |wc -l`
			if [ $INDEX -ge 0 -a $INDEX -lt $rt6num ];then
				tar=1
				INDEX=$(($INDEX+1))

				while [ $tar -le $rt6num ];do
					if [ $tar -eq $INDEX ];then
						echo "TMOPTION=$(eval echo '$'TMOPTION)" >> $tmprtfile
						echo "$tmpstr"
						echo "$tmpstr" >>$tmprtfile
					else
						stayopt=`cat $rtfile |sed 's/#.*//g' |grep 'TMOPTION' | sed -n "$(eval echo '$'tar)p" `
						echo "$stayopt">>$tmprtfile
						staystr=`cat $rtfile | sed 's/#.*//g'| grep 'via' | sed -n "$(eval echo '$'tar)p" `
						echo "$staystr"
						echo "$staystr">>$tmprtfile
					fi

					tar=$(($tar+1))
				done

				if [ -f $tmprtfile ];then
					mv -f $tmprtfile $rtfile
				fi

				return 0
			fi
		fi
	fi

	return  0
}

del_item_rtfile()
{
	local ADDRESS=""
	local NETMASK=""
	local GATEWAY=""
	local TMOPTION="0"
	local skipnum="$2"
	local rtnum=0
	local num=0
	local rt4num=1


	rtfile="$cfgdir/route-$1"
	if [ ! -f $rtfile ]; then
		rt4num=0
	fi

	grep -q "^ADDRESS$skipnum" $rtfile
	rtret=$?
	grep -q "^TMPADDRESS$skipnum" $rtfile

	if [ "$rtret" -eq 1 -a  "$?" -eq 1 ]; then
		# no this item
		rt4num=0
	fi

	if [ $rt4num -ne 0 ]; then
		. $rtfile
		echo "#`date`: Generate by IWSVA network scripts" > $rtfile

		while [ "x$(eval echo '$'ADDRESS$rtnum)x" != "xx" -o "x$(eval echo '$'TMPADDRESS$rtnum)x" != "xx" ]; do
			if [ "$skipnum" -ne "$rtnum" ]; then
				ADDRESS="$(eval echo '$'ADDRESS$rtnum)"
				NETMASK="$(eval echo '$'NETMASK$rtnum)"
				GATEWAY="$(eval echo '$'GATEWAY$rtnum)"
				TMOPTION="$(eval echo '$'TMOPTION$rtnum)"

				if [ -n "$ADDRESS" ]; then # deploy route entry
					add_entry_route $rtfile $num $ADDRESS $NETMASK $GATEWAY $TMOPTION
				else # not depoly route entry
					ADDRESS="$(eval echo '$'TMPADDRESS$rtnum)"
					NETMASK="$(eval echo '$'TMPNETMASK$rtnum)"
					GATEWAY="$(eval echo '$'TMPGATEWAY$rtnum)"
					TMOPTION="$(eval echo '$'TMPTMOPTION$rtnum)"
					add_tmp_entry_route $rtfile $num $ADDRESS $NETMASK $GATEWAY $TMOPTION
				fi
				num=$(($num+1))
			fi

			rtnum=$(($rtnum+1))
		done

		if [ "$num" -eq 0 ]; then
			rm -f $rtfile
		fi

		return  0
	fi
	# rt4num = 0 ,find in route6

	if [ ! -f $rtfile ]; then
		rt4num=0
	else
		rt4num=`cat $rtfile | grep 'ADDRESS' | wc -l`
	fi

	echo "find in route6,route4 is $rt4num"

	if [ $skipnum -ge $rt4num ];then
		# get value from route6
		targetnum=1
		rtfile="$cfgdir/route6-$1"
		tmprtfile="$cfgdir/rt6"
		> $tmprtfile

		if [ ! -f $rtfile ]; then
			return 0
		fi

		skipnum=$(($skipnum-$rt4num+1))
		rt6num=`cat $rtfile |sed 's/#.*//g'|grep 'via' |wc -l`

		if [ $rt6num -ge $skipnum ];then
			while [ $targetnum -le $rt6num ];do
				if [ $targetnum -ne $skipnum ];then
					rt4str=`cat $rtfile |sed 's/#.*//g'| grep 'via' | sed -n "$(eval echo '$'targetnum)p" `
					ADDRESS=`echo $rt4str|awk -F/ '{print $1}'`
					GATEWAY=`echo $rt4str|awk '{print $3}'`
					NETMASK=`echo $rt4str|awk '{print $1}'|awk -F/ '{print $2}'`
					TMOPTION=`cat $rtfile |sed 's/#.*//g' | grep 'TMOPTION' | sed -n "$(eval echo '$'targetnum)p"|awk -F= '{print $2}' `
					if [ -z "$TMOPTION" ]; then
						TMOPTION=2
					fi

					echo "TMOPTION=$(eval echo '$'TMOPTION)" >> $tmprtfile
					echo "$ADDRESS/$NETMASK via $GATEWAY" >> $tmprtfile
				fi
				targetnum=$(($targetnum+1))
			done

			if [ -f $tmprtfile ];then
				mv -f $tmprtfile $rtfile
				rt6num=`cat $rtfile |sed 's/#.*//g'|grep 'via' |wc -l`
				if [ $rt6num -eq 0 ];then
					rm -f $rtfile
				fi
			fi
			return 0
		else
			echo -e "\nError: This route does not exist in current route table.\n"
       		return 1
		fi
	fi

	return  0
}

# get_from_rtfile:$1=dev,$2=index
# for exp:eth0 has 5 route,3 in route-eth0,2 in route6-eth0
# we need get value in route6-eth0
# liangji
get_from_rtfile()
{
	local ADDRESS=""
	local NETMASK=""
	local GATEWAY=""
	local targetnum="$2"
	local TMOPTION=0

	rtnum=0
	rtfile="$cfgdir/route-$1"
	if [ ! -f $rtfile ]; then
		#return 0
		rt4num=0
	else
		rt4num=`cat $rtfile | grep 'ADDRESS' | wc -l`
	fi

	if [ $rt4num -gt 0 ]; then

	. $rtfile

	while [ "x$(eval echo '$'ADDRESS$rtnum)x" != "xx" -o "x$(eval echo '$'TMPADDRESS$rtnum)x" != "xx" ]; do
		if [ "$rtnum" -eq "$targetnum" ] ; then
			ADDRESS="$(eval echo '$'ADDRESS$rtnum)"
			NETMASK="$(eval echo '$'NETMASK$rtnum)"
			GATEWAY="$(eval echo '$'GATEWAY$rtnum)"
			TMOPTION="$(eval echo '$'TMOPTION$rtnum)"

			if [ -z "$ADDRESS" ]; then # get not deploy entry
				ADDRESS="$(eval echo '$'TMPADDRESS$rtnum)"
				NETMASK="$(eval echo '$'TMPNETMASK$rtnum)"
				GATEWAY="$(eval echo '$'TMPGATEWAY$rtnum)"
				TMOPTION="$(eval echo '$'TMPTMOPTION$rtnum)"
			fi

	#		if [ -z "$TMOPTION" ]; then
	#			TMOPTION=0
	#		fi

			if [ -z "$TMOPTION" ]; then
				TMOPTION=2
			fi

			#IWSx-02379 [Bridge]Static route will add/delete fail when IWSVA management IP has VLAN
			#IFDEV=$1
			#if [ "$IFDEV" = "${IFDEV%\.*}" ]; then
			#	IFDEV="$IFDEV.$VID"
			#fi

			#Spidey need remove Bridge VLAN, so we only use origianl dev
			#IFDEV=$1

	        # add the support for VLAN in bridge mode
	        	IFDEV=$1
	        	IFDEV=`echo "$IFDEV" | cut -d'.' -f 1`

			#echo -n "$ADDRESS:$GATEWAY:$NETMASK:$IFDEV ${TMOPTION}"
			echo -n "$ADDRESS|$GATEWAY|$NETMASK|$IFDEV ${TMOPTION}"
			return 0;
		fi

		rtnum=$(($rtnum+1))
		done
	fi
	# get route6 value

	#rt4num=`cat $rtfile | grep '^ADDRESS*' | wc -l`
	if [ ! -f $rtfile ]; then
		rt4num=0
	else
		rt4num=`cat $rtfile | grep 'ADDRESS' | wc -l`
	fi

	if [ $targetnum -ge $rt4num ];then
		# get value from route6
		targetnum=$(($targetnum-$rt4num+1))
		rt6file="$cfgdir/route6-$1"
		if [ ! -f $rt6file ]; then
			return 0
		fi

		if [ `cat $rt6file | grep 'via' |wc -l` -ge $targetnum ];then
			rt4str=`cat $rt6file | grep 'via' | sed -n "$(eval echo '$'targetnum)p" `
			ADDRESS=`echo $rt4str|awk -F/ '{print $1}'`
			GATEWAY=`echo $rt4str|awk '{print $3}'`
			NETMASK=`echo $rt4str|awk '{print $1}'|awk -F/ '{print $2}'`
    		IFDEV=$1
    		IFDEV=`echo "$IFDEV" | cut -d'.' -f 1`

			TMOPTION=`cat $rt6file | sed 's/#.*//g' |grep 'TMOPTION' | sed -n "$(eval echo '$'targetnum)p"|awk -F= '{print $2}' `
			if [ -z "$TMOPTION" ]; then
				TMOPTION=2
			fi

			echo -n "$ADDRESS|$GATEWAY|$NETMASK|$IFDEV ${TMOPTION}"
			return 0
		fi
	fi

	return 1
}

get_opt_from_rtfile ()
{
	local targetnum="$2"
	local TMOPTION=0
	local found=0
	rtfile="$cfgdir/route-$1"

	if [ ! -f $rtfile ]; then
		return 0
	fi

	rtnum=0
	. $rtfile

	while [ "x$(eval echo '$'ADDRESS$rtnum)x" != "xx" -o "x$(eval echo '$'TMPADDRESS$rtnum)x" != "xx" ]; do
 		if [ "$rtnum" -eq "$targetnum" ] ; then
			TMOPTION="$(eval echo '$'TMOPTION$rtnum)"
			if [ -z "$TMOPTION" ]; then
				TMOPTION="$(eval echo '$'TMPTMOPTION$rtnum)"
				found=1
			fi

			if [ -z "$TMOPTION" ]; then
				TMOPTION=2
			fi
			echo -n "$((TMOPTION))"
		fi

        rtnum=$(($rtnum+1))
    done

	# get route6 option

	if [ ! -f $rtfile ]; then
		rt4num=0
	else
		rt4num=cat $rtfile | grep 'ADDRESS' | wc -l 2>/dev/null
	fi

	if [ $targetnum -ge $rt4num ];then
		# get option from route6
		targetnum=$(($targetnum-$rt4num+1))
		rtfile="$cfgdir/route6-$1"
		if [ ! -f $rtfile ]; then
			return 0
		fi

		if [ `cat $rtfile |sed 's/#.*//g' | grep 'TMOPTION' |wc -l` -ge $targetnum ];then
			TMOPTION=`cat $rtfile | sed 's/#.*//g' |grep 'TMOPTION' | sed -n "$(eval echo '$'targetnum)p"|awk -F= '{print $2}' `
			if [ -z "$TMOPTION" ]; then
				TMOPTION=2
			fi

			found=1
			echo -n "$((TMOPTION))"
			return 0
		fi

	fi

	if [ $found -eq 1 ]; then
		return 0
	else
		return 1
	fi
}

get_count_from_rtfile ()
{
	local ADDRESS=""

	rtnum=0
	rtfile="$cfgdir/route-$1"
	if [ -f $rtfile ]; then
		. $rtfile

		while [ "x$(eval echo '$'ADDRESS$rtnum)x" != "xx" -o "x$(eval echo '$'TMPADDRESS$rtnum)x" != "xx" ]; do
			rtnum=$(($rtnum+1))
		done
	fi

	# add ipv6 route num:route6-$1,liangji

	rtfile="$cfgdir/route6-$1"
	if [ ! -f $rtfile ]; then
		return $rtnum
	fi

	rt6num=`cat $rtfile |sed 's/#.*//g'|grep 'via'|wc -l`
	rtnum=$(($rtnum+$rt6num))

	dbg 2 "get route count=$rtnum"
	return $rtnum
}

# it is used to ui deploy setting
deploysetting ()
{
	dbg 2 "depoly $@"

	action="$1"
	shift

	case $action in
	route)
		iwsva_deploy_route 1
		ret=$?
		iwsva_check_route_table
		;;
	pre_route)
		pre_deploy_route $@
		;;
	check_route)
		iwsva_check_route_table
		;;
	destroy_route)
		case "$1" in
		mgmt) #
			MGMT_INTERFACE=""
			get_mgmt_interface
			if [ "$MGMT_INTERFACE" != "" ]; then
				destroy_route $MGMT_INTERFACE
			fi
			;;
		data) #
			DATA_INTERFACE=""
			get_data_interface

			if [ $bridgemode -eq 1 ]; then
				destroy_route $DATA_INTERFACE
			else
				for int in $PROXY_ALL; do
					destroy_route $inf
				done
			fi
			;;
		esac
	;;
	dhcpuphooks)
		dhcpuphooks $@
		ret=$?
		;;
	update_issue)
		update_issue_file $@
		;;
	conntrack_ftp)
		# added for Web console modified ports
		load_ip_conntrack_ftp
		;;
	esac

	return $ret
}

#pre_deploy_route_v4 <ifname>
pre_deploy_route_v4 ()
{
	# clear route by default
	route | grep -v "\*" | grep "^[1-9]" | grep $1 | (
		while read ADDRESS GATEWAY NETMASK EOL ; do
	        route del -net $ADDRESS netmask $NETMASK
		done )

	mroutecfg="$cfgdir/route-$1"
#	echo "$mroutecfg"
	if [ ! -f $mroutecfg ]; then
		return 0
	fi

	# apply the tmp route
    if grep -q "^TMP" $mroutecfg ; then
		sed -i "s/^TMP//g" $mroutecfg
	fi

#	if [ -f $mroutecfg  ]; then
#		$cfgdir/ifup-routes $1
#	fi
}

pre_deploy_route_v6 ()
{
    local IPV6ADDRPREFIX=""
    local IPV6GW=""

    # clear ipv6 route by default
    /sbin/ip -6 route | grep $1 | grep "^[0-9a-fA-F:]*/[0-9]*[[:space:]]via[[:space:]].*" |
    sed -n -e 's/^\([0-9a-fA-F:]*\/[0-9]*\) via \([0-9a-fA-F:]*\) .*/\1 \2/p' | (
		while read IPV6ADDRPREFIX IPV6GW ; do
		    /sbin/ip -6 route del $IPV6ADDRPREFIX via $IPV6GW
		done )

    local mroutecfg="$cfgdir/route6-$1"
    if [ ! -f $mroutecfg ]; then
		return 0
    fi

    sed -i 's/^#\([0-9a-fA-F:]*\/[0-9]* via [0-9a-fA-F:]*\)/\1/g' $mroutecfg
}

#pre_deploy_route <ifname>
pre_deploy_route ()
{
    pre_deploy_route_v4 $1
    pre_deploy_route_v6 $1
}

#deploy_route_v4 <ifname>
deploy_route_v4 ()
{
	mroutecfg="$cfgdir/route-$1"
	if [ -f $mroutecfg ]; then
		$cfgdir/ifup-routes $1
	fi
}

#deploy_route_v6 <ifname>
deploy_route_v6 ()
{
    local mroutecfg="$cfgdir/route6-$1"

    if [ -f $mroutecfg ]; then
		cat $mroutecfg | sed 's/#.*//g' | grep -v '^[[:space:]]*$' | while read line; do
	    		/sbin/ip -6 route add $line
			done
    fi
}

#deploy_route <ifname>
deploy_route ()
{
    deploy_route_v4 $1
    deploy_route_v6 $1
}

get_data_interface ()
{
	load_ifname

	if [ $bridgemode -eq 1 ]; then
		DATA_INTERFACE="$BRI"

		get_param $NETINI vid
		if [ -n "$gvalue" -a "$gvalue" != "0" ]; then
	    	DATA_INTERFACE="$BRI.$gvalue"
		fi
	else
	 	DATA_INTERFACE="$PROXY"
	fi
}

#
get_mgmt_interface ()
{
	get_param $NETINI separate_mgmt_port
	if [ "$gvalue" = "yes"  ]; then
		get_param $NETINI ifname_mgmt
		if [ -n $gvalue ]; then
			MGMT_INTERFACE=$gvalue
		fi
	fi
}

#Usage: iwsva_check_route_table
iwsva_check_route_table ()
{
	MGMT_INTERFACE=""
	DATA_INTERFACE=""
	get_data_interface

	check_router_table $DATA_INTERFACE
	get_mgmt_interface

	if [ "$MGMT_INTERFACE" != "" ]; then
		check_router_table $MGMT_INTERFACE
	fi
}

#Usage: iwsva_deploy_route <deploy> deploy=0: do no do ifup-routes deply=1: do ifup-routes
iwsva_deploy_route ()
{
	DEPLOY="$1"
	if [ -z $DEPLOY ]; then
		$DEPLOY=1
	fi

	MGMT_INTERFACE=""
	DATA_INTERFACE=""

	get_data_interface
#	echo "data interface $DATA_INTERFACE"

	if [ $bridgemode -eq 1 ]; then
		pre_deploy_route $DATA_INTERFACE
	else
		for inf in $PROXY_ALL; do
			pre_deploy_route $inf
		done
	fi

	get_mgmt_interface
#	echo "mgmt_interfac $MGMT_INTERFACE"
	if [ "$MGMT_INTERFACE" != "" ]; then
		pre_deploy_route $MGMT_INTERFACE
	fi

	if [ $DEPLOY -eq 1 ]; then
		if [ $bridgemode -eq 1 ]; then
			deploy_route $DATA_INTERFACE
		else
			for inf in $PROXY_ALL; do
				deploy_route $inf
			done
		fi

		if [ "$MGMT_INTERFACE" != "" ]; then
			deploy_route $MGMT_INTERFACE
		fi
	fi
}

update_issue_file ()
{
	local msg=""
	local port=""
	local proto="http"
	MGMT_IPINFO=""

	# reset the issue file
	get_mngt_info address
	if [ -n "$IPINFO" ]; then
		ipaddr="$IPINFO"
	else
		ipaddr="resolving.address"
	fi

	get_mngt_info ipv6addr
	if [ -n "$IPINFO" ]; then
	    ipv6addr="`echo $IPINFO | cut -d'/' -f1`"
	else
	    ipv6addr="resolving.address"
	fi

#andy add for mgmt interface
	get_param $NETINI separate_mgmt_port
	if [ "$gvalue" = "yes"  ]; then
		get_param $NETINI ifname_mgmt
		if [ -n $gvalue ]; then
			MGMT_IPINFO="`ifconfig $gvalue | grep "inet addr:" | cut -d":" -f2 | cut -d' ' -f1`"
			if [ -n "$MGMT_IPINFO" ]; then
				ipaddr="$MGMT_IPINFO"
			else
				ipaddr="resolving.address"
			fi

			MGMT_IPINFO="`/sbin/ip -6 addr show $gvalue | grep "inet6.*global" | awk '{ print $2 }' | cut -d '/' -f 1`"
			if [ -n "$MGMT_IPINFO" ]; then
			    ipv6addr="$MGMT_IPINFO"
			else
			    ipv6addr="resolving.address"
			fi
		fi
	fi

	get_webserver_port
	if [ -n "$WEBSERVER_PORT" ]; then
		port="$WEBSERVER_PORT"
	fi

	get_param $ISCANINI iscan_web_protocol
	if [ -n "$gvalue" ]; then
		proto=$gvalue
	fi

	msg="$proto://$ipaddr:$port"
	msgv6="$proto://[$ipv6addr]:$port"
	if [ -f /etc/issue ]; then
		dbg 2 "update /etc/issue with '$msg'"
		sed -i "s%^\([\t ]*\)http[s]*://[0-9a-zA-Z\.]*:[0-9]*.*%\\1$msg%g" /etc/issue
		dbg 2 "update /etc/issue with '$msgv6'"
		sed -i "s%^\([\t ]*\)http[s]*://\[[0-9a-zA-Z\.:]*\]:[0-9]*.*%\\1$msgv6%g" /etc/issue
	fi
}

if [ $debug -eq 0 ]; then
	CFGHOSTS="/etc/hosts"
else
	CFGHOSTS="./hosts"
fi

update_hosts ()
{
	get_mngt_info address
	if [ -z "$IPINFO" ]; then
		dbg 2 "resolving address for update $CFGHOSTS"
		return 1;
	fi

	get_param $CFGNETWORK "HOSTNAME"
	if [ -z "$gvalue" ]; then
		dbg 2 "hostname is missing in $CFGNETWORK"
	fi

	FQDN="$gvalue"
	HOSTNAME="`echo $FQDN | cut -d'.' -f1`"

	if [ "x$HOSTNAME" = "x$FQDN" ]; then
		SETTINGS="$IPINFO\t\t$HOSTNAME"
	else
		SETTINGS="$IPINFO\t\t$FQDN $HOSTNAME"
	fi

	# check entry
	local MAGIC='# IWSVA'
	local MAGICTAIL="$MAGIC Network added entry"
	local ret=0

	if [ -f $CFGHOSTS ]; then
		grep -q "$MAGIC" $CFGHOSTS
		ret=$?
	else
		ret=1
	fi

	if [ "$ret" -eq 0 ]; then
		SETTINGS="$SETTINGS\t\t$MAGIC"
		sed -i "s/.*$MAGIC/$SETTINGS/g" $CFGHOSTS
	else
		SETTINGS="$SETTINGS\t\t$MAGICTAIL"
		echo -e "$SETTINGS" >> $CFGHOSTS
	fi

	# remove the specific hostname in 127.0.0.1 to avoid Web UI conflict
	grep -q "127.0.0.1[[:space:]]*.*localhost" $CFGHOSTS
	if [ "$?" -eq 0 ]; then
		sed -i "s/127\.0\.0\.1[[:space:]]*.*/127.0.0.1\tlocalhost.localdomain localhost/g" $CFGHOSTS
	fi

    # update ipv6 address into /etc/hosts
    get_mngt_info ipv6addr
    IPINFO=`echo $IPINFO | cut -d'/' -f1`
    if [ -n "$IPINFO" ]; then
        if [ "x$HOSTNAME" = "x$FQDN" ]; then
            IPv6SETTINGS="$IPINFO\t\t$HOSTNAME"
        else
            IPv6SETTINGS="$IPINFO\t\t$FQDN $HOSTNAME"
        fi

        local MAGICTAIL="# IWSx IPv6 network added entry"
        local rc=1;
        if [ -f $CFGHOSTS ]; then
            grep -q "$MAGICTAIL" $CFGHOSTS
            rc=$?
        fi

        IPv6SETTINGS="$IPv6SETTINGS\t\t$MAGICTAIL"
        if [ "$rc" -eq 0 ]; then
            sed -i "s/.*$MAGICTAIL/$IPv6SETTINGS/g" $CFGHOSTS
        else
            echo -e "$IPv6SETTINGS" >> $CFGHOSTS
        fi
    fi

	dbg 2 "$CFGHOSTS updated"
	return 0
}

CFGSQUID="/etc/squid/squid.conf"
# update the field visible_hostname of squid.conf
update_squid_conf ()
{
	if [ ! -f "$CFGSQUID" ]; then
		dbg 2 "$CFGSQUID is not existed"
		return 0;
	fi

	get_param $CFGNETWORK "HOSTNAME"
	if [ -z "$gvalue" ]; then
		dbg 2 "hostname is missing in $CFGNETWORK"
		HOSTNAME=localhost.localdomain
	fi

	FQDN="$gvalue"
	HOSTNAME="`echo $FQDN | cut -d'.' -f1`"
	SETTINGS="visible_hostname $HOSTNAME"

	# check entry
	local MAGICTAIL="# IWSVA Network added entry"
	local ret=0

	if [ -f $CFGSQUID ]; then
		grep -q "^visible_hostname[[:space:]]*" $CFGSQUID
		ret=$?
	else
		ret=1
	fi

	if [ "$ret" -eq 0 ]; then
		SETTINGS="$SETTINGS $MAGICTAIL"
		sed -i "s/^visible_hostname[[:space:]]*.*/$SETTINGS/g" $CFGSQUID
	else
		SETTINGS="$SETTINGS $MAGICTAIL"
		echo "$SETTINGS" >> $CFGSQUID
	fi

	dbg 2 "$CFGSQUID updated"
	return 0
}

# solve issus of the nscd cached IWSVA IP address
update_nscd ()
{
	if [ -e /var/lock/subsys/nscd ]; then
		/etc/init.d/nscd restart 2>/dev/null >/dev/null
		dbg 2 "nscd restart, ret=$?"
	else
		/etc/init.d/nscd start 2>/dev/null >/dev/null
		dbg 2 "nscd restart, ret=$?"
	fi

	# clear nscd cache
	nscd -i hosts
	return 0
}

CHKIP=/var/iwss/iwsvanet_ip_check.tmp
# Either IPv4 address or IPv6 address changed, then return 1.
check_same_ip ()
{
    local OIPV4=""
    local OIPV6=""
    local ipv4same="y"
    local ipv6same="y"

    if [ -f "$CHKIP" ]; then
		OIPV4=`cat $CHKIP | grep '^ipv4' | cut -d' ' -f2`
		OIPV6=`cat $CHKIP | grep '^ipv6' | cut -d' ' -f2`
    fi

    get_mngt_info address
    if [ -n "$IPINFO" -a "$OIPV4" != "$IPINFO" ]; then
        # ip has been changed
        if [ -f $CHKIP ]; then
            sed -i "/^ipv4 .*/d" $CHKIP
        else
            touch $CHKIP
        fi

        echo "ipv4 $IPINFO" >> $CHKIP
        ipv4same="n"
    fi

    IPINFO=""
    get_mngt_info ipv6addr
    if [ -n "$IPINFO" -a "$OIPV6" != "$IPINFO" ]; then
        if [ -f $CHKIP ]; then
            sed -i "/^ipv6 .*/d" $CHKIP
        else
            touch $CHKIP
        fi

        echo "ipv6 $IPINFO" >> $CHKIP
        ipv6same="n"
    fi

    if [ "$ipv4same" = "y" -a "$ipv6same" = "y" ]; then
        return 0
    fi

    return 1
}

iwss_restart ()
{
  	if [ ! -e /etc/iscan/.pgpass ]; then
		return 0;
	fi

	#If reboot by deployment wizard, need not to restart service
	if [ -e /etc/iscan/.dw_reboot -o -e /etc/iscan/.dw_reboot_with_timesync ]; then
		return 0
	fi

	/etc/iscan/S99ISftp stop 2>/dev/null >/dev/null
	/etc/iscan/S99ISftp start 2>/dev/null >/dev/null
	/etc/iscan/S99ISproxy stop >/dev/null 2>&1
	/etc/iscan/S99ISproxy start >/dev/null 2>&1
	return 0;
}

hook_ip_changed ()
{
	# except it from ip check, for proto update
	update_issue_file

	check_same_ip
	if [ $? -eq 0 ]; then
		dbg 2 "hook_ip_changed: ip is unchanged"
		return 0
	fi

	update_hosts
	update_nscd

	# IWSx-02453  squid -- set squid as downstream proxy, then change ip or get another IP via DHCP, the traffic disconnected
	/usr/iwss/squid.sh reload 2>/dev/null >/dev/null
	iwss_restart
	#/etc/iscan/S99IScanHttpd restart 2>/dev/null >/dev/null
	#IWSx-02368  CLI [configure ethernet static] after change IP address, ntpd still use old IP address
#	/etc/init.d/ntpd restart 2>/dev/null >/dev/null
	return 0;
}

# This function will be called,
# if dhclient run the /etc/dhclient-up-hooks
dhcpuphooks ()
{
	hook_ip_changed dhcp
}

set_hostname ()
{
	local HOSTNAME="$1"
        get_param "$CFGNETWORK" HOSTNAME
	if [ "$gvalue" = "$HOSTNAME" ]; then
		dbg 3 "hostname is applied the same value."
		return 0;
	fi

    set_param "$CFGNETWORK" HOSTNAME "${HOSTNAME}"
	hostname "${HOSTNAME}"
	hook_hostname_changed "${HOSTNAME}"
	return $?
}

# Once the hostname has been changed,
# this hook function will be called.
hook_hostname_changed ()
{
	dbg 2 "hostname($1) has been changed"
	update_hosts
	update_squid_conf

	# restart proxy to rest hostname cache
	iwss_restart
	return 0;
}

hook_vlan ()
{
	for i in ifup ifdown-eth ; do
		targetcfg=/etc/sysconfig/network-scripts/$i
		grep -q "|bond)" $targetcfg
		if [ "$?" -eq 0 ]; then
			sed -i "s/|bond)/|bond|br)/g"  $targetcfg
		fi
	done
}


update_dns ()
{
    local proto="$1"
	local dns1="$2"
	local dns2="$3"
	local ipv4dns=""
	local ipv6dns=""

	if [ -z "$dns1" -a -z "$dns2" ]; then
		return 1;
	fi

	rscf=/etc/resolv.conf

	get_bootproto
	if [ "$BOOTPROTO" = "dhcp" -a "$proto" = "ipv4" ]; then
		i18n_not_dns_in_dhcp="Cannot set DNS under DHCP mode"
		echo "$i18n_not_dns_in_dhcp"
		return 1;
	fi

	if [[ "${DHCPV6C}" = [Yy1]* ]] && [ "$proto" = "ipv6" ]; then
		i18n_not_dns_in_dhcpv6="Cannot set DNS under DHCPV6 mode"
		echo "$i18n_not_dns_in_dhcpv6"
		return 1;
	fi

	if [ -f $rscf ]; then
	    cp -af $rscf $rscf.bak
	    if [ "$proto" = "ipv4" ]; then
			#ipv6dns="`cat $rscf | grep '^[[:space:]]*nameserver[[:space:]]\[[0-9a-fA-F:]*\]' | awk '{ print $2 }' | sed 's/\[\(.*\)\]/\1/g'`"
			ipv6dns="`cat $rscf |  grep -E '^[[:space:]]*nameserver[[:space:]][0-9a-fA-F]{1,4}:' | awk '{ print $2 }'`"
	    else
			ipv4dns="`cat $rscf | grep -E '^[[:space:]]*nameserver[[:space:]][0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' | awk '{ print $2 }'`"
	    fi

	    cat $rscf.bak | sed -e '/^[[:space:]]*search[[:space:]]/d' -e '/^[[:space:]]*nameserver[[:space:]]/d' > $rscf
	    rm -f $rscf.bak
	fi

	if [ "$proto" = "ipv4" ]; then
	    echo "nameserver ${dns1}" >> $rscf
	    if [ -n "${dns2}" ]; then
			echo "nameserver ${dns2}" >> $rscf
	    fi

	    if [ -n "${ipv6dns}" ]; then
			for i in ${ipv6dns}
			do
			    echo "nameserver ${i}" >> $rscf
			done
	    fi
	else  # set ipv6 DNS server
	    if [ -n "${ipv4dns}" ]; then
			for i in ${ipv4dns}
			do
			    echo "nameserver ${i}" >> $rscf
			done
	    fi

	    echo "nameserver ${dns1}" >> $rscf
	    if [ -n "${dns2}" ]; then
			echo "nameserver ${dns2}" >> $rscf
	    fi
	fi

	update_nscd
	return 0
}

#usage:find_recorder_from_route_table IFNAME ADDRESS NETMASK GATEWAY
find_recorder_from_route_table ()
{
	local IFNAME="$1"
	local ADDRESS="$2"
	local NETMASK="$3"
	local GATEWAY="$4"
# 	echo "find_recorder_from_route_table $IFNAME $ADDRESS $NETMASK $GATEWAY"
	local found=0
	local num=0

	route -n > /tmp/route.tmp

	while   read   LINE;   do
	   	if [ $num -gt 1 ]; then
			R_ADDRESS=`echo   $LINE | cut -d" " -f1`
			R_MASK=`echo   $LINE |  cut -d" " -f3`
			R_ROUTER=`echo   $LINE |  cut -d" " -f2`
			R_IFNAME=`echo   $LINE |  cut -d" " -f8`
		   	index=$((index+1));
#		   	echo "Recoder: $R_IFNAME $R_ADDRESS $R_MASK $R_ROUTER "

	        if [ "$R_IFNAME" = "$IFNAME" -a "$R_ADDRESS" = "$ADDRESS" -a "$R_MASK" = "$NETMASK" -a "$R_ROUTER" = "$GATEWAY" ]; then
	   	   		found=1
#	   	   		echo "find Recoder:  $R_IFNAME $R_ADDRESS $R_MASK $R_ROUTER  "
	   		fi
	   	fi

	   	num=$((num+1));
  	done   <   /tmp/route.tmp

  	if [ $found -eq 1 ]; then
  	  	return 1
  	else
  	  	return 0
  	fi
}

#usage:check_router_table IFNAME
check_router_table_v4 ()
{
	IFNAME="$1"
	local found=0
	local rtnum=0
	local num=0
	rtfile="$cfgdir/route-$IFNAME"

	if [ ! -f $rtfile ]; then
		return 1
	fi

	. $rtfile

    # get the number of the route item in $rtfile
    local route_count=`cat $rtfile | grep -e "^ADDRESS[[:digit:]]" | wc -l`

    #generate the new file
    echo "#`date`: Generate by IWSVA network scripts" > $rtfile
    while [ $route_count -gt 0 ]; do
        route_count=$((route_count-1))

        ADDRESS_O="$(eval echo '$'ADDRESS$num)"
        NETMASK_O="$(eval echo '$'NETMASK$num)"
        GATEWAY_O="$(eval echo '$'GATEWAY$num)"
        TMOPTION_O="$(eval echo '$'TMOPTION$num)"

        find_recorder_from_route_table $IFNAME $ADDRESS_O $NETMASK_O $GATEWAY_O
        if [ $? -eq 1 ]; then
            TMOPTION=0
        else
            TMOPTION=1
        fi

        add_entry_route $rtfile $rtnum $ADDRESS_O $NETMASK_O $GATEWAY_O $TMOPTION
        rtnum=$(($rtnum+1))
        num=$((num+1));
    done
}

check_router_table_v6 ()
{
    local IFNAME="$1"
    local rtfile="$cfgdir/route6-$IFNAME"

    if [ ! -f $rtfile ]; then
		return 1
    fi

    local ipv6addrprefix=""
    local ipv6addr=""
    local ipv6gw=""
    sed -i '/^TMOPTION/d' $rtfile

    cat $rtfile | sed 's/#.*//g' | grep -v '^[[:space:]]*$' | while read line ; do
			ipv6addrprefix=`echo $line | awk '{ print $1 }'`
			ipv6addr=`echo $ipv6addrprefix | cut -d'/' -f1`
			ipv6gw=`echo $line | awk '{ print $3 }'`

			if /sbin/ip -6 route show | grep "$IFNAME" | grep "$ipv6addr" | grep -q "$ipv6gw" ; then
			    sed -i "/^$ipv6addr/i TMOPTION=0" $rtfile
			else
			    sed -i "/^$ipv6addr/i TMOPTION=1" $rtfile
			fi
	    done
    return 0
}

check_router_table ()
{
    check_router_table_v4 ${1}
    check_router_table_v6 ${1}
}

check_mgmt_network_number()
{
    mgmt_ip=${1}

    if [ -z "${mgmt_ip}" ]; then
        echo "Please specify the management interface address."
        return 0
    fi

    data_ip=`sudo /usr/iwss/iwsvanetwork.sh get ip address`
    data_netmask=`sudo /usr/iwss/iwsvanetwork.sh get ip netmask`

    [ -z ${data_ip} ] && return 0

    if [ -z ${data_netmask} ]; then
        echo "Error: Cannot get the netmask of data interface"
        return 0
    fi

    data_network=`ipcalc -n ${data_ip} ${data_netmask}`

    network=`ipcalc -n ${mgmt_ip} ${data_netmask}`

    if [ "${network}" = "${data_network}" ]; then
        echo "Warning: The IP addresses of the data and management interfaces should not be in the same network segment."
        return 0
    else
        return 0
    fi
}

check_data_network_number()
{
    data_ip=${1}

    if [ -z "${data_ip}" ]; then
        echo "Please specify the data interface address."
        return 0
    fi

    # get mgmt interface's adress
    get_param $NETINI separate_mgmt_port
    [ "$gvalue" != "yes"  ] && return 0

    get_param $NETINI ifname_mgmt
    if [ -n "${gvalue}" ]; then
        mgmt_interface=${gvalue}
    else
        echo "Error: Cannot get management interface name."
        return 0
    fi

    mgmt_ip=`ifconfig "${mgmt_interface}" | grep -e "inet addr:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`
    mgmt_netmask=`ifconfig "${mgmt_interface}" | grep -e "Mask:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`

    [ -z ${mgmt_ip} ] && return 0
    if [ -z "$mgmt_netmask" ]; then
        echo "Error: Cannot get management interface netmask."
        return 0
    fi

    # check the network number
    mgmt_network=`ipcalc -n ${mgmt_ip} ${mgmt_netmask}`
    network=`ipcalc -n ${data_ip} ${mgmt_netmask}`

    if [ "$mgmt_network" = "$network" ]; then
        echo "Warning: The IP addresses of the data and management interfaces should not be in the same network segment."
        return 0
    else
        return 0
    fi
}

check_dhcp_network()
{
    local status=`/usr/iwss/iwsvanetwork_mgmt.sh get mgmt enable`

    [ "${status}" != "yes" ] && return 0

    current_dev=${1}
    get_data_interface
    get_mgmt_interface

    ip=`ifconfig "${current_dev}" | grep -e "inet addr:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`
    if [ -z ${ip} ]; then
        echo "Error: the dhcp client get nothing." # the program should not run to here
        return 0
    fi

    if [ ${current_dev} = ${DATA_INTERFACE} ]; then
        mgmt_ip=`ifconfig "${MGMT_INTERFACE}" | grep -e "inet addr:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`
        mgmt_netmask=`ifconfig "${MGMT_INTERFACE}" | grep -e "Mask:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`

        [ -z ${mgmt_ip} ] && return 0

        if [ -z ${mgmt_netmask} ]; then
            echo "Error: Cannot get the netmask of management interface."
            return 0
        fi

        mgmt_network=`ipcalc -n ${mgmt_ip} ${mgmt_netmask} | cut -d '=' -f2`
        network=`ipcalc -n ${ip} ${mgmt_netmask} | cut -d '=' -f2`

        [ ${mgmt_network} = ${network} ] && return 0

    elif [ ${current_dev} = ${MGMT_INTERFACE} ]; then
        data_ip=`ifconfig "${DATA_INTERFACE}" | grep -e "inet addr:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`
        data_netmask=`ifconfig "${DATA_INTERFACE}" | grep -e "Mask:[0-9]*.[0-9]*.[0-9]*.[0-9]*" -o | cut -d':' -f2`

        [ -z ${data_ip} ] && return 0

        if [ -z ${data_netmask} ]; then
            echo "Error: Cannot get the netmask of data interface."
            return 0
        fi

        data_network=`ipcalc -n ${data_ip} ${data_netmask} | cut -d '=' -f2`
        network=`ipcalc -n ${ip} ${data_netmask} | cut -d '=' -f2`

        [ ${data_network} = ${network} ] && return 0
    fi

    return 0
}

set_portgroup()
{
    get_param $NETINI port_group
    pg_setting="$gvalue"

    operation=$1
    shift
    pg_name=$1

    case $operation in
        add)
            shift

            if [ "$pg_setting" != "" ]; then
                echo "$pg_setting" | grep -q "$pg_name"
                if [ "$?" -eq 0 ]; then
                    i18n_pg_exist=`gettext "The port group is already exist."`
                    echo "$i18n_pg_exist"
                    return 1
                else
                    pg_setting="$pg_setting|$pg_name:off $*"
                fi
            else
                pg_setting="$pg_name:off $*"
            fi

            #check interface
            if [ $# -lt 2 ]; then
                i18n_pg_nic_one=`gettext "At least 2 interfaces are needed to add a port group."`
                echo "$i18n_pg_nic_one"
                return 1
            fi

            for i in $*; do
                local nic=`echo "$i" | awk -F ':' '{print $1}'`
                local ct=`echo "$*" | grep -o "$nic" | wc -l`
                if [ $ct -gt 1 ]; then
                    i18n_pg_nic_dup=`gettext "Duplicated interface found:"`
                    echo "$i18n_pg_nic_dup $nic"
                    return 1
                fi
            done

            load_pg_module

            $DEVCONF -c "$pg_name:off $*"
            if [ "$?" -eq 0 ]; then
                set_param $NETINI port_group "$pg_setting"
            fi
	        ;;
        remove)
            if [ "$pg_setting" != "" ]; then
                echo "$pg_setting" | grep -q "$pg_name"
                if [ "$?" -eq 0 ]; then
                    pg_setting=`echo $pg_setting | sed 's/\('$pg_name'[^|]*|\)\|\('$pg_name'[^|]*$\)\|\(|'$pg_name'[^|]*$\)//g'`

                    $DEVCONF -r "$pg_name" 2>/dev/null >/dev/null
                    if [ "$pg_setting" = "" ]; then
                        del_param $NETINI port_group
                        uninit_portgroup
                    else
                        set_param $NETINI port_group "$pg_setting"
                    fi
                else
                    i18n_pg_not_exist=`gettext "The port group is not exist."`
                    echo "$i18n_pg_not_exist"
                fi
            else
                i18n_pg_not_exist=`gettext "The port group is not exist."`
                echo "$i18n_pg_not_exist"
            fi
    	    ;;
        linkloss)
            if [ "$pg_setting" != "" ]; then
                echo "$pg_setting" | grep -q "$pg_name"
                if [ "$?" -eq 0 ]; then
                    pg_group=`echo $pg_setting | sed 's/.*\('$pg_name'[^|]*\).*/\1/g'`
                    pg_group=`echo $pg_group | sed 's/'$pg_name':[onf]*\(.*\)/'$pg_name':'$2'\1/g'`

                    pg_setting=`echo $pg_setting | sed "s/$pg_name[^|]*/$pg_group/g"`

                    $DEVCONF -r "$pg_name" 2>/dev/null >/dev/null
                    $DEVCONF -c "$pg_group"
                    ret="$?"
                    if [ "$ret" -eq "0" ]; then
                        set_param $NETINI port_group "$pg_setting"
                    fi
                else
                    i18n_pg_not_exist=`gettext "The port group is not exist."`
                    echo "$i18n_pg_not_exist"
                fi
            else
                i18n_pg_not_exist=`gettext "The port group is not exist."`
                echo "$i18n_pg_not_exist"
            fi
        	;;
        vlan)
            if [ "$pg_setting" != "" ]; then
                echo "$pg_setting" | grep "$pg_name"
                if [ "$?" != "" ]; then
                    pg_group=`echo $pg_setting | sed 's/.*\('$pg_name'[^|]*\).*/\1/g'`
                    pg_group=`echo $pg_group | sed 's/\('$pg_name':[onf]*\)[^ ]*\(.*\)/\1:'$2'\2/g'`

                    pg_setting=`echo $pg_setting | sed "s/$pg_name[^|]*/$pg_group/g"`
                    $DEVCONF -r "$pg_name" 2>/dev/null >/dev/null

                    $DEVCONF -c "$pg_group"
                    if [ "$?" -eq 0 ]; then
                        set_param $NETINI port_group "$pg_setting"
                    fi
                else
                    i18n_pg_not_exist=`gettext "The port group is not exist."`
                    echo "$i18n_pg_not_exist"
                fi
            else
                i18n_pg_not_exist=`gettext "The port group is not exist."`
                echo "$i18n_pg_not_exist"
            fi
        	;;
    esac
}

init_bonding()
{
    get_param $NETINI bonding
    bonding_settings="$gvalue"

    if [ -n "$bonding_settings" ]; then
        # create ifcfg if bonding is used by bridge or proxy mode
        interfaces=""

        if [ "$bridgemode" -eq 0 ]; then
            get_param $NETINI ifname_proxy
            interfaces="$gvalue"

            get_param $NETINI ifname_proxy_internal
            interfaces="$interfaces $gvalue"

            get_param $NETINI ifname_proxy_external
            interfaces="$interfaces $gvalue"
        else
            get_param $NETINI ifname_internal
            interfaces="$gvalue"

            get_param $NETINI ifname_external
            interfaces="$interfaces $gvalue"

            get_param $NETINI ifname_others
            interfaces="$interfaces $gvalue"
        fi

        bonding_index=1
        bonding_setting=`echo "$bonding_settings" | cut -d'|' -f$bonding_index`
        multiple=`echo "$bonding_settings" | grep "|"`

        while [ "$bonding_setting" != "" ]; do
            is_inuse=0
            bond_name=`echo "$bonding_setting" | cut -d':' -f1`
            for interface in $interfaces; do
                if [ "$interface" = "$bond_name" ]; then
                    is_inuse=1
                fi
            done

            bonding_setting=`echo "$bonding_setting" | sed 's/\(.*\):\(.*\)/\1 \2/g'`
            if [ "$is_inuse" -eq 1 ]; then
                create_ifcfg_bonding $bonding_setting
            fi

            bonding_index=`expr $bonding_index + 1`
            if [ -n "$multiple" ]; then
                bonding_setting=`echo "$bonding_settings" | cut -d'|' -f$bonding_index`
            else
                break
            fi
        done
    fi
}

uninit_bonding()
{
	get_param $ISCANINI bridgemode
	if [ -n $gvalue ]; then
        bridgemode=$(($gvalue))
	else
        bridgemode=0
	fi

    if [ "$bridgemode" -eq 0 ]; then
        get_param $NETINI ifname_proxy
        interfaces="$gvalue"

        get_param $NETINI ifname_proxy_internal
        interfaces="$interfaces $gvalue"

        get_param $NETINI ifname_proxy_external
        interfaces="$interfaces $gvalue"
    else
        get_param $NETINI ifname_internal
        interfaces="$gvalue"

        get_param $NETINI ifname_external
        interfaces="$interfaces $gvalue"

        get_param $NETINI ifname_others
        interfaces="$interfaces $gvalue"
    fi

    for intf in $interfaces; do
        echo $intf | grep -q "^bond"
        if [ "$?" -eq 0 ]; then
            slaves=`cat /sys/class/net/$intf/bonding/slaves`
            for slave in $slaves; do
                echo -$slave > /sys/class/net/$intf/bonding/slaves
            done
        fi
    done
}

set_bonding()
{
    if [ "$1" == "add" -a $# -lt 3 ];then
        i18n_bonding_noIF=`gettext "There should be at least one interface."`
        echo "$i18n_bonding_noIF"
        return 1
    fi

    get_param $NETINI bonding
    bonding_settings="$gvalue"

    operation=$1
    shift

    case $operation in
        add)
            bonding_name=$1
            shift

            tmp_bonding=`echo "$bonding_settings" | sed "s/$bonding_name:.*|//"`
            tmp_bonding=`echo "$tmp_bonding" | sed "s/$bonding_name:.*$//"`
            get_real_nic

            for if_name in $* ; do
                echo "$tmp_bonding" | grep -q "$if_name"
                if [ "$?" -eq 0 ]; then
                    i18n_bonding_exist=`gettext "$if_name already used with another bonding interface."`
                    echo "$i18n_bonding_exist"
                    return 1
                fi

                echo "$grealnics" | grep -q "$if_name"
                if [ "$?" -ne 0 ]; then
                    i18n_bonding_nonic=`gettext "Cannot find $if_name."`
                    echo "$i18n_bonding_nonic"
                    return 1
                fi
            done

            if [ "$bonding_settings" = "" ]; then
                bonding_settings="$bonding_name:$*"
            else
                echo "$bonding_settings" | grep -q "$bonding_name:"
                if [ "$?" -eq 0 ]; then
                    bonding_setting=`echo $bonding_settings | sed 's/\('$bonding_name'\):\([^|]*\)/\1 \2/g'`
                    bonding_settings=`echo $bonding_settings | sed "s/\($bonding_name:[^|]*\)/$bonding_name:$*/g"`
                else
                    bonding_settings="$bonding_settings|$bonding_name:$*"
                fi
            fi

            add_bonding $bonding_name $*
            set_param $NETINI bonding "$bonding_settings"
	        ;;
        remove)
            bonding_name=$1

            echo "$bonding_settings" | grep -q "$bonding_name:"
            if [ "$?" -eq 0 ]; then
                bonding_setting=`echo $bonding_settings | sed 's/.*\('$bonding_name'\):\([^|]*\).*/\1 \2/g'`
                bonding_settings=`echo $bonding_settings | sed 's/\('$bonding_name':[^|]*|\)\|\('$bonding_name':[^|]*$\)\|\(|'$bonding_name':[^|]*$\)//g'`

                remove_bonding $bonding_setting

                if [ "$bonding_settings" != "" ]; then
                    set_param $NETINI bonding "$bonding_settings"
                else
                    del_param $NETINI bonding
                fi

            else
                i18n_bonding_notexist=`gettext "unknown interface $bonding_name: No such device."`
                echo "$i18n_bonding_notexist"
                return 1
            fi
	        ;;
        options)
            option=$1
            option_value=$2
            set_bonding_options "$option" "$option_value"
    	    ;;
    esac
}

create_ifcfg_bonding()
{
    name=$1
    shift

    ifcfg_bonding="$cfgdir/ifcfg-$name"
    ifcfg_tmp="$tmpcfgdir/ifcfg-$name"

    if [ ! -e $ifcfg_bonding ]; then
        write_header_ifcfg $ifcfg_bonding
    fi

    get_param $NETINI "bonding_options"
    bonding_options="$gvalue"

    if [ -z "$bonding_options" ]; then
        bonding_options="mode=4 miimon=100 xmit_hash_policy=1"
    else
        bonding_options=`echo $bonding_options | sed 's/:/=/g' | sed 's/|/ /g'`
    fi

	set_param $ifcfg_bonding "BONDING_OPTS" "\"$bonding_options\""
	set_param $ifcfg_bonding "DEVICE" "$name"
	set_param $ifcfg_bonding "ONBOOT" "yes"
	set_param $ifcfg_bonding "TYPE"  "ETHERNET"
	set_param $ifcfg_bonding "BOOTPROTO" "none"
	set_param $ifcfg_bonding "USERCTL" "none"

    get_param $ifcfg_tmp BONDING_OPTS
    if [ -n "$gvalue" ]; then
        OBOOTPROTO="$gvalue"
        set_param $ifcfg_bonding "BONDING_OPTS" "$OBONDING_OPTS"
    fi

    get_param $ifcfg_tmp BOOTPROTO
    if [ -n "$gvalue" ]; then
        OBOOTPROTO="$gvalue"
        set_param $ifcfg_bonding "BOOTPROTO" "$OBOOTPROTO"
    fi

    get_param $ifcfg_tmp IPADDR
    if [ -n "$gvalue" ]; then
        OIPADDR="$gvalue"
        set_param $ifcfg_bonding "IPADDR" "$OIPADDR"
    fi

    get_param $ifcfg_tmp NETMASK
    if [ -n "$gvalue" ]; then
        ONETMASK="$gvalue"
        set_param $ifcfg_bonding "NETMASK" "$ONETMASK"
    fi

    get_param $ifcfg_tmp GATEWAY
    if [ -n "$gvalue" ]; then
        OGATEWAY="$gvalue"
        set_param $ifcfg_bonding "GATEWAY" "$OGATEWAY"
    fi

    for eth in $*; do
        ifcfg="$cfgdir/ifcfg-$eth"
        write_header_ifcfg $ifcfg
        set_param $ifcfg "DEVICE" "$eth"
        set_param $ifcfg "ONBOOT" "yes"
        set_param $ifcfg "USERCTL" "no"
        set_param $ifcfg "MASTER" "$name"
        set_param $ifcfg "SLAVE" "yes"
        set_param $ifcfg "BOOTPROTO" "none"
        add_hwaddr $ifcfg "$eth"
    done
}

add_bonding()
{
    name=$1
    shift
    grep -q "^[[:space:]]*alias[[:space:]]\+$name[[:space:]]\+bonding" $MODCONF
    if [ "$?" -ne 0 ]; then
        echo "alias $name bonding" >> $MODCONF
    fi
}

add_bonding_to_bridge()
{
    if [ "$bridgemode" -eq 0 ]; then
        get_param $NETINI ifname_proxy
        interfaces="$gvalue"

        get_param $NETINI ifname_proxy_internal
        interfaces="$interfaces $gvalue"

        get_param $NETINI ifname_proxy_external
        interfaces="$interfaces $gvalue"
    else
	    get_param $NETINI ifname_internal
	    interfaces="$gvalue"

	    get_param $NETINI ifname_external
        interfaces="$interfaces $gvalue"

	    get_param $NETINI ifname_others
        interfaces="$interfaces $gvalue"
    fi

    for intf in $interfaces; do
        echo $intf | grep -q "^bond"
        if [ "$?" -eq 0 ]; then
            brctl addif $BRI $intf >/dev/null 2>&1
            set_param "$cfgdir/ifcfg-$intf" "BRIDGE" "$BRI"
        fi
    done

    echo $interfaces | grep -q "bond"
    if [ "$?" -eq 0 ]; then
        brctl stp $BRI on >/dev/null 2>&1
    else
        get_param $NETINI enable_stp
        if [ "$gvalue" = "no" ]; then
            brctl stp $BRI off >/dev/null 2>&1
        fi
    fi
}

remove_bonding()
{
    name=$1
    shift

    config=`grep -v "^[[:space:]]*alias[[:space:]]\+$name[[:space:]]\+bonding" $MODCONF`
    echo "$config" > $MODCONF

    #rm -f "$cfgdir/ifcfg-$name"
}

set_bonding_options()
{
    load_ifname

    bonding_option=$1
    option_value=$2

    get_param $NETINI bonding_options
    bonding_options="$gvalue"

    if [ -z "$bonding_options" ]; then
        bonding_options="mode:4|miimon:100|xmit_hash_policy:1"
    fi

    case $bonding_option in
        miimon)
            bonding_options=`echo $bonding_options | sed 's/miimon:[^|]*/miimon:'$option_value'/g'`
        ;;
        xmit_hash_policy)
            bonding_options=`echo $bonding_options | sed 's/xmit_hash_policy:[^|]*/xmit_hash_policy:'$option_value'/g'`
        ;;
        mode)
            bonding_options=`echo $bonding_options | sed 's/mode:[^|]*/mode:'$option_value'/g'`
        ;;
    esac

    set_param $NETINI bonding_options "$bonding_options"

    if [ "$bridgemode" -eq 0 ]; then
        get_param $NETINI ifname_proxy
        interfaces="$gvalue"

        get_param $NETINI ifname_proxy_internal
        interfaces="$interfaces $gvalue"

        get_param $NETINI ifname_proxy_external
        interfaces="$interfaces $gvalue"
    else
        get_param $NETINI ifname_internal
        interfaces="$gvalue"

        get_param $NETINI ifname_external
        interfaces="$interfaces $gvalue"

        get_param $NETINI ifname_others
        interfaces="$interfaces $gvalue"
    fi

    for intf in $interfaces; do
        echo $intf | grep -q "^bond"
        if [ "$?" -eq 0 ]; then
            case $bonding_option in
                miimon)
                    echo "$option_value" > /sys/class/net/$intf/bonding/$option
                ;;
                xmit_hash_policy)
                    ifconfig $intf down
                    echo "$option_value" > /sys/class/net/$intf/bonding/$option
                    ifconfig $intf up
                ;;
                mode)
                    ifconfig $intf down
                    echo "$option_value" > /sys/class/net/$intf/bonding/$option
                    ifconfig $intf up
                ;;
            esac
        fi
    done
}


# the function will query nic statistic information
# those infomation will be displayed in webui

extract_nic_statistic_info()
{
    device_list=`/sbin/ifconfig -a | grep "Link encap:Ethernet" | grep "eth[0-9]*" | cut -d' ' -f1`
    current_time=`date "+%s"`
    device_status=`/usr/iwss/AdminUI/uihelper "/usr/bin/shownic -o not_up_interface"`

    READ_INI_TOOL="/usr/bin/iniconfig -r -f "
    NETWORK_INI="/etc/iscan/network.ini"
    INTSCAN_INI="/etc/iscan/intscan.ini"
    NODE_INI="/etc/iscan/node.ini"

    data_interfaces=""

    bridge_mode=`$READ_INI_TOOL $INTSCAN_INI -s "bridge" -o "bridgemode"`
    if [ "$bridge_mode" -eq 0 ]; then
        data_interfaces=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_proxy"`
        p1=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_proxy"`
        p2=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_proxy"`

        data_interfaces="$data_interfaces $p1 $p2"
    else
        data_interfaces=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_internal"`
        p1=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_external"`
        data_interfaces="$data_interfaces $p1"
    fi

    mgmt_interfaces=""
    enable_mgmt=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "separate_mgmt_port"`
    if [ "$enable_mgmt" = "yes" ]; then
        mgmt_interfaces=`$READ_INI_TOOL $NETWORK_INI -s "network" -o "ifname_mgmt"`
    fi

    ha_interfaces=""
    machine_role=`/usr/iwss/haagntcmd GetRole`
    if [ "$machine_role" != "single" -a -f $NODE_INI ]; then
        ha_interfaces=`$READ_INI_TOOL $NODE_INI -s "node" -o "heartbeat_port"`
    fi

    devices_name=""

    for iterator in $device_list; do
        contents=`/sbin/ifconfig $iterator`

        rx_packets=`echo $contents|egrep "RX packets:[0-9]*" -o|cut -d':' -f2`
        tx_packets=`echo $contents|egrep "TX packets:[0-9]*" -o|cut -d':' -f2`

        rx_errors=`echo $contents|egrep "RX packets:[0-9]* errors:[0-9]*" -o|cut -d':' -f3`
        tx_errors=`echo $contents|egrep "TX packets:[0-9]* errors:[0-9]*" -o|cut -d':' -f3`

        collisions=`echo $contents|egrep "collisions:[0-9]*" -o|cut -d':' -f2`

        rx_bytes=`echo $contents|egrep "RX bytes:[0-9]*" -o|cut -d':' -f2`
        tx_bytes=`echo $contents|egrep "TX bytes:[0-9]*" -o|cut -d':' -f2`

        hwaddr=`echo $contents|egrep "HWaddr [0-9a-zA-Z:]*" -o|cut -d' ' -f2`

        inet_addr=`echo $contents|egrep "inet addr:[0-9.]*" -o |cut -d':' -f2`
        inet_mask=`echo $contents|egrep "Mask:[0-9.]*" -o |cut -d':' -f2`

        status=`echo $device_status|egrep "$iterator: [a-zA-Z]*" -o|cut -d" " -f2`

        if [ -z "$inet_addr" -o -z "$inet_mask" ]; then
            ipv4_addr=""
        else
            ipv4_addr="$inet_addr/`ipcalc -p $inet_addr $inet_mask|cut -d'=' -f2`"
        fi

        mtu=`echo $contents|egrep "MTU:[0-9]*" -o|cut -d':' -f2`

        if [ -z "$devices_name" ]; then
            devices_name=$iterator
        else
            devices_name="$devices_name|$iterator"
        fi

        # output the statistic information
        echo "${iterator}_rx_packets=$rx_packets"
        echo "${iterator}_tx_packets=$tx_packets"
        echo "${iterator}_rx_errors=$rx_errors"
        echo "${iterator}_tx_errors=$tx_errors"
        echo "${iterator}_collisions=$collisions"
        echo "${iterator}_rx_bytes=$rx_bytes"
        echo "${iterator}_tx_bytes=$tx_bytes"

        echo "${iterator}_hwaddr=$hwaddr"
        echo "${iterator}_ipv4_addr=$ipv4_addr"
        echo "${iterator}_mtu=$mtu"
        echo "${iterator}_status=$status"

        dev_type="NULL"
        echo $data_interfaces | grep -q "$iterator"
        [ $? -eq 0 ] && dev_type="D"

        echo $mgmt_interfaces | grep -q "$iterator"
        [ $? -eq 0 ] && dev_type="M"

        echo $ha_interfaces | grep -q "$iterator"
        [ $? -eq 0 ] && dev_type="H"

        echo "${iterator}_type=${dev_type}"
        echo ""
    done

    echo "devices_name=$devices_name"
    echo "current_time=$current_time"
}

# the following function will implement the package capture
# the webui will call those function at last

packet_capture_progress="/tmp/capture_packet_progress.log"
packet_store_directory="/var/iwss/UserDumps"
packet_capture_time=`date '+%Y%m%d%H%M%S'`
pcap_file_pattern="$packet_store_directory/capture*$packet_capture_time.pcap"
packet_capture_stop_command_file="/tmp/.stop_capture_packet_command"

WEBUI_HELPER="/usr/iwss/AdminUI/uihelper"

finish_capture()
{
    process_count=`ps -ef | grep tcpdump | grep -v grep | wc -l`
    [ $process_count -ne 0 ] && $WEBUI_HELPER "killall -SIGINT tcpdump"

    package_name="$packet_store_directory/capture-$packet_capture_time.tgz"
    $WEBUI_HELPER "chown iscan:iscan $pcap_file_pattern"

    ls $pcap_file_pattern >/dev/null 2>&1
    [ $? -eq 0 ] && tar -zcf $package_name $pcap_file_pattern

    rm -f $pcap_file_pattern
    echo "[finish] capture package name: `basename $package_name`" >>$packet_capture_progress
}

start_capture()
{
    rm -f $packet_capture_stop_command_file >/dev/null 2>&1

    echo "[start] capture packet ..." >$packet_capture_progress
    if [ $# -lt 1 ]; then
        echo "please specify the capture interface" >>$packet_capture_progress
        return 1
    fi

    device_name="$*"
    filename_list=""

    trap 'finish_capture' 0

    # by default, the max capture file size can be 10G
    max_capture_size=`cat /etc/iscan/network.ini| grep max_capture_size  |cut -d'=' -f2`
    [ -z "$max_capture_size" ] &&  max_capture_size=10737418240

    for iterator in $device_name; do
        [ ! -e "/sys/class/net/$iterator" ] && continue

        packet_file="$packet_store_directory/capture-$iterator-$packet_capture_time.pcap"
        filename_list="$filename_list $packet_file"
        $WEBUI_HELPER "/usr/sbin/tcpdump -i $iterator -s 0 -w $packet_file &"
    done

    # wait to finish capture opteration
	start_capture_time=`date "+%s"`
	echo "[start] start time=$start_capture_time" >>$packet_capture_progress
	echo "[start] devices=$device_name">>$packet_capture_progress

    time2finish=0

    while [ 1 ]; do
        sleep 1

        # file size reach the max
        for iterator in $filename_list; do
            if [ -e "$iterator" ]; then
                file_size=`ls -l $iterator|cut -d' ' -f5`
                [ -n "$file_size" -a "$file_size" -gt "$max_capture_size" ] && time2finish=1
            fi
        done

        # disk free size is less 1G, quit capture
        disk_free_size=`df -lP -B 1 $packet_store_directory | awk '{print $4}'| egrep '[0-9]*' -o`
        [ "$disk_free_size" -lt 1073741824 ] && time2finish=1

        # stop capture packet
        [ -f "$packet_capture_stop_command_file" ] && time2finish=1

        [ "$time2finish" -eq 1 ] && break
    done
}

stop_capture()
{
    $WEBUI_HELPER "killall -SIGINT tcpdump"

    touch $packet_capture_stop_command_file
    chown iscan:iscan $packet_capture_stop_command_file
}

print_capture_status()
{
	device=""
	start_time=""

	matched_line=`cat $packet_capture_progress | grep "\[start\] start time="`
	if [ -n "$matched_line" ]; then
		start_time=`echo $matched_line | cut -d'=' -f2`
	fi

	matched_line=`cat $packet_capture_progress | grep "\[start\] devices="`
	if [ -n "$matched_line" ]; then
		device=`echo $matched_line | cut -d'=' -f2`
	fi

	matched_line=`cat $packet_capture_progress | grep "\[finish\] capture package name"`
	if [ -n "$matched_line" ]; then
		start_time=""
	fi


	if [ -n "$start_time" -a -n "$device" ]; then
		((elapsed_time=`date +%s`-$start_time))

		echo "elapsed time: $elapsed_time"
		echo "capture device: $device"
	fi
}

print_net_device_list()
{
    for iter in `ls /sys/class/net/`; do
        [ "$iter" = "bonding_master" ] && continue

        $WEBUI_HELPER "/sbin/ifconfig $iter" | grep "UP" >/dev/null 2>&1
        [ $? -eq 0 ] && echo "$iter"
	done
}
