#!/bin/sh

. /usr/iwss/bin/ini_util.sh

#IWSx-02370 [FTP Proxy] FTP passive mode can not work properly in standalone proxy mode 
#IWSx-02433 [FTP Proxy] active ftp connections to a ftp-server doesn't work. There is no data channel possible
load_ip_conntrack_ftp ()
{
	#default port
	local DEFAULT_FTP_PORT=21
	local FTP_PORT=$DEFAULT_FTP_PORT
	local FTP_PNI=/etc/iscan/IWSSPIProtocolFtp.pni
	local FTP_PORTS=""
	F_ini_get "$FTP_PNI" 'main' '^port' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		FTP_PORT="$_R"
	fi
	F_ini_get "$FTP_PNI" 'ftp' '^ftp_server_port'  >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		FTP_SERVER_PORT="$_R"
	fi

	FTP_PORTS="$FTP_PORT"
	if [ -n "$FTP_SERVER_PORT" -a ! "$FTP_PORT" = "$FTP_SERVER_PORT" ]; then
		FTP_PORTS="$FTP_PORTS,$FTP_SERVER_PORT"
	fi

	if [ "$FTP_PORT" -ne $DEFAULT_FTP_PORT -a "$FTP_SERVER_PORT" -ne $DEFAULT_FTP_PORT ]; then
		FTP_PORTS="$DEFAULT_FTP_PORT,$FTP_PORTS"
	fi

	LIBKMOD_DIR=/lib/modules/`uname -r`
	#rmmod ip_conntrack_ftp 2>/dev/null >/dev/null
	#insmod $LIBKMOD_DIR/kernel/net/ipv4/netfilter/ip_conntrack_ftp.ko ports="$FTP_PORTS"
	rmmod nf_conntrack_ftp 2>/dev/null >/dev/null
	insmod $LIBKMOD_DIR/kernel/net/netfilter/nf_conntrack_ftp.ko ports="$FTP_PORTS"
	return $?
}


rp_add_rports ()
{
	local OP_IP="$1"
	local rports="$2"
	local intface="$3"
	local ret=0
	for i in $rports; do
		echo "ADD -dport $i $OP_IP $intface " >> $tmp_rp_file
		ret=$(($ret | $?))
	done
	return $ret
}

rp_add_bypass ()
{
	local rp_file="$1"
	local type="$2"
	local ids="$3"
	local id=""
	if [ -n "$ids" ]; then
		ids=`echo "$ids" | tr -s ';' ' '`
		for id in $ids; do
			case $type in
			v*)
			echo "ADD -v $id -action pass" >> $rp_file
			;;
			s*)
			echo "ADD -sip $id -action pass" >> $rp_file
			;;
			d*)
			echo "ADD -dip $id -action pass" >> $rp_file
			;;
			esac
		done
	fi
	return 0
}

rp_add_macfilter ()
{
        local rp_file="$1"
        local ids="$2"
        local id=""
        if [ -n "$ids" ]; then
                ids=`echo "$ids" | tr -s ';' ' '`
                for id in $ids; do
                        echo "ADD -dm !$id -action pass" >> $rp_file
                done
        fi
        return 0
}

pre_rp_rules_create ()
{
	local tmp_rp_file="$1"
	local rports="$2"
	local INIFILE=/etc/iscan/network.ini
	local ret=0;

	F_ini_get "$INIFILE" 'network' '^bypass_vlan' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		bypass_vlan="$_R"
	fi
	F_ini_get "$INIFILE" 'network' '^bypass_dstIP' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		bypass_dstIP="$_R"
	fi
	F_ini_get "$INIFILE" 'network' '^bypass_srcIP' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		bypass_srcIP="$_R"
	fi
	F_ini_get "$INIFILE" 'network' '^onlyscan_srcIP' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		onlyscan_srcIP="$_R"
	fi
	F_ini_get "$INIFILE" 'network' '^onlyscan_dstIP' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		onlyscan_dstIP="$_R"
	fi
	F_ini_get "$INIFILE" 'network' '^mac_filter' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		mac_filter="$_R"
	fi
  
	F_ini_get "$INIFILE" 'network' '^onlyscan_interface' >/dev/null
	if [ "$?" = "0" -a ! -z "$_R" ] ; then
		onlyscan_interface="$_R"
	fi
	
	echo -n > $tmp_rp_file
	rp_add_macfilter $tmp_rp_file "$mac_filter"
	rp_add_bypass $tmp_rp_file vlan "$bypass_vlan"
	rp_add_bypass $tmp_rp_file srcIP "$bypass_srcIP"
	rp_add_bypass $tmp_rp_file dstIP "$bypass_dstIP"

	if [ -n "$onlyscan_srcIP" -o -n "$onlyscan_dstIP" -o -n "$onlyscan_interface" ]; then
		if [ -n "$onlyscan_srcIP" ]; then
			onlyscan_srcIP=`echo "$onlyscan_srcIP" | tr -s ';' ' '`
			for sip in $onlyscan_srcIP ; do
				OP_SIP="-sip $sip"
				rp_add_rports "$OP_SIP" "$rports"
				ret=$(($ret | $?))
			done
		fi
		
		if [ -n "$onlyscan_dstIP" ] ; then
			onlyscan_dstIP=`echo "$onlyscan_dstIP" | tr -s ';' ' '`
			for dip in $onlyscan_dstIP ; do
				OP_DIP="-dip $dip"
				rp_add_rports "$OP_DIP" "$rports"
				ret=$(($ret | $?))
			done
		fi
		
		if [ -n "$onlyscan_interface" ]; then
			onlyscan_interface=`echo "$onlyscan_interface" | tr -s ';' ' '`
			for int in $onlyscan_interface ; do
				intface="-i $int"
				rp_add_rports "" "$rports" "$intface"
				ret=$(($ret | $?))
			done
		fi
	else
		rp_add_rports "" "$rports"
		ret=$(($ret | $?))
	fi
	# trim duplication entry
	cat $tmp_rp_file | uniq > $tmp_rp_file.bak
	mv $tmp_rp_file.bak $tmp_rp_file
	return $ret;
}
