#!/usr/bin/env python
# -*- coding: utf-8 -*-

import cStringIO
import json
import zlib
import base64
import fnmatch
from summary_engine import SummaryEngine
from log_engine import LogEngine
from clfcfg import CLFCfg
from common_id import logformat
from common_id import log_datatype
from common_id import summary_indx
import logging
import time
import os
import fcntl
import PyInterface
import copy
g_logEninge = LogEngine()
g_sumEngine = SummaryEngine()

def check_log_entry(entry, filters, timeBegin, timeEnd):
    '''check whether the entry matchs the filters'''
    #check ts first
    ts = int(entry[logformat.index('CLF_UTCTimeStamp')])
    if not (ts >= timeBegin and ts < timeEnd):
        return False

    ops_int = ['>', '<', '>=', '<=']
    ops_str = ['eq', 'neq']
    for f in filters:
        try:
            c_index = logformat.index(f['col'])
        except ValueError,e:
            continue

        v = entry[c_index]
        if (log_datatype[c_index] in ['int', 'bigint']):
            if not v:
                return False
            v = int(v)

        #check number filter
        if f['op'] in ops_int:
            statment = '%s %s %s' % (v, f['op'], f['value'])
            if not eval(statement):
                return False

        # change 'value' to 'list'
        if f['op'] in ('==', '!=', 'eq', 'neq'):
            if not isinstance(f['value'], list):
                f['value'] = [f['value']]

        wildcard = False
        if f['op'] in ops_str:
            for fv in f['value']:
                if '*' in fv:
                    wildcard = True
                    break

        if wildcard:
            if f['op'] == 'eq': #Matching any one will be OK.
                matched = False
                for fv in f['value']:
                    if '*' in fv:
                        if fnmatch.fnmatch(v, fv):
                            matched = True
                            break
                    else:
                        if (v == fv) :
                            matched = True
                            break
                if not matched:
                    return False
            elif f['op'] == 'neq':  #Should match no one in value.
                for fv in f['value']:
                    if '*' in fv:
                        if fnmatch.fnmatch(v, fv):
                            return False
                    else:
                        if v == fv:
                            return False
        else:
            if f['op'] in ('==','eq') and not v in f['value']:
                return False
            elif f['op'] in ('!=','neq') and v in f['value']:
                return False

    return True

def make_cursor(cursor):
    result = ''
    if cursor:
        result = json.dumps(cursor)
        result = base64.encodestring(zlib.compress(result))

    return result


def write_sessionID_ts(sessionID, timestamp):
    if not os.path.exists('/usr/trend/commonlog/framework/sessionID.txt'):
        os.makedirs('/usr/trend/commonlog/framework/sessionID.txt')
    session_fp = open('/usr/trend/commonlog/framework/sessionID.txt','a+')
    fcntl.flock(session_fp, fcntl.LOCK_EX)
    session_info = session_fp.read()
    if session_info:
        session_info = json.loads(session_info)
    else:
        session_info = {}
    session_info[sessionID] = timestamp
    session_fp.seek(0, 0)
    session_fp.truncate()  #clean all content for this file
    session_fp.write(json.dumps(session_info))
    session_fp.flush()
    fcntl.flock(session_fp, fcntl.LOCK_UN)
    session_fp.close()

def read_sessionID_ts(sessionID):
    fp = open('/usr/trend/commonlog/framework/sessionID.txt','r')
    fcntl.flock(fp, fcntl.LOCK_EX)
    session_info = fp.read()
    if session_info:
        session_info = json.loads(session_info)
    else:
        session_info = {}
    if session_info.has_key(sessionID):
        time_stamp_now  = int(session_info[sessionID])
    else:
        time_stamp_now = 0
    fcntl.flock(fp, fcntl.LOCK_UN)
    fp.close()
    return time_stamp_now

def delete_sessionID_ts(sessionID):
    session_fp = open('/usr/trend/commonlog/framework/sessionID.txt','a+')
    fcntl.flock(session_fp, fcntl.LOCK_EX)
    session_info = session_fp.read()
    if session_info:
        session_info = json.loads(session_info)
    else:
        session_info = {}
    logging.debug('delete sessionID and session_info:%r and sessionID:%r',session_info, sessionID)
    if session_info.has_key(sessionID):
        session_info.pop(sessionID)
    logging.debug('delete sessionID and session_info:%r',session_info)
    session_fp.seek(0, 0)
    session_fp.truncate()  #clean all content for this file
    if session_info:
        session_fp.write(json.dumps(session_info))
    session_fp.flush()
    fcntl.flock(session_fp, fcntl.LOCK_UN)
    session_fp.close()



def is_timeout(sessionID, time_stamp):
    time_stamp_now = read_sessionID_ts(sessionID)
    logging.debug('$$$$$$$$$query log sessionID:%r and check time_stamp:%r',sessionID,time_stamp_now)
    if time_stamp_now and time_stamp_now != time_stamp:
        logging.info('***********888query log sessionID:%r and return duprep',sessionID)
        return json.dumps({'err_code':'dupreq'})
    if int(time.time())- time_stamp >= 295:
        logging.info('query log time out')
        #logging.debug('query log sessionID:%r and return timeout',sessionID)
        return json.dumps({'err_code':'timeout'})
    if time_stamp_now == 0 and int(time.time()) - time_stamp >= 295:
        logging.debug('query log sessionID:%r and ts == 0 return timeout',sessionID)
        return json.dumps({'err_code':'timeout'})
    return ''




def queryLogs(companyID, column, filters, timeBegin, timeEnd, skip, limit, cursor, sessionID, facet_name, scheduled_time_filter=None):

    #get summary data
    #check cursor. cursor has format: {'block_id1':123, 'block_id2':234}
    if len(cursor) > 0:
        cursor = zlib.decompress(base64.decodestring(cursor))
        cursor = json.loads(cursor)
    else:
        cursor = None
    time_stamp = int(time.time())
    logging.debug('query log sessionID:%r and ts:%r',sessionID,time_stamp)
    write_sessionID_ts(sessionID,time_stamp)

    match_value = []
    match_index = []
    msg_value = []
    msg_index = 1
    #make filter to correct format
    for i in range(0, len(filters)):
        try:
            c_index = logformat.index(filters[i]['col'])
        except ValueError,e:
            logging.exception('Error: %r. Ignore the filter: %r', e, filters[i]['col'])
            continue

        if (log_datatype[c_index] in ['int', 'bigint']):
            if filters[i]['op'] == 'eq':
                filters[i]['op'] = '=='
            elif filters[i]['op'] == 'neq':
                filters[i]['op'] = '!='
            if isinstance(filters[i]['value'], list):
                for j in range(0,len(filters[i]['value'])):
                    filters[i]['value'][j] = int(filters[i]['value'][j])
            else:
               filters[i]['value'] = int(filters[i]['value'])
        if(filters[i]['col'] == 'CLF_MsgType'):
            msg_value = copy.deepcopy(filters[i]['value'])
            logging.info('msg_value:%r',msg_value)
            msg_index = c_index
            if isinstance(msg_value, list):
                for k, msg_item in enumerate(msg_value):
                    msg_value[k] = str(msg_value[k])
            else:
                msg_value = [str(msg_value)]
        else:
            match_index.append(c_index)
            match_value.append(str(filters[i]['value']))


    logging.info('facet_name:%r',facet_name)

    logging.debug(filters)

    #round ts to minute
    ts1 = int(timeBegin)/60*60
    ts2 = int(timeEnd)/60*60
    start = 0
    rows = 10
    try:
        #summaries = g_sumEngine.findSummaries(companyID, filters, ts1, ts2, start, rows)
        summaries = g_sumEngine.findSummariesEx(companyID,facet_name,filters, ts1, ts2, start, rows, scheduled_time_filter)
    except Exception,e:
        logging.exception('solr db error:%r',e)
        delete_sessionID_ts(sessionID)
        return json.dumps({'err_code':'timeout'})

    #get raw logs
    result = []
    cnt = 0
    new_cursor = {}
    log_block_dir = os.path.join(CLFCfg.config.get('storage', 'root'),CLFCfg.config.get('storage', 'log_block'),companyID)
    while summaries:
        for s in summaries:
            for blk in s['zLnk']:
                if cursor and skip > 0:
                    # bypass blocks in cursor
                    try:
                        logs_in_blk = cursor[blk]
                        if (cnt + logs_in_blk <= skip):
                            cnt += logs_in_blk
                            logging.debug('Bypass [%s] %d records', blk, logs_in_blk)
                            new_cursor[blk] = logs_in_blk
                            continue
                    except KeyError:
                        pass

                ret = is_timeout(sessionID,time_stamp)
                if ret:
                    delete_sessionID_ts(sessionID)
                    logging.info('timeout !!!!')
                    return ret

                #raw_logs = zlib.decompress(g_logEninge.getLogBlock(companyID, blk))
                matched = 0
                file_path = log_block_dir+'/'+blk
                #file_path = '/etc/iscan/commonlog_data/log_block/0/'+blk
                logging.info('match_value:%r, and match_index:%r',match_value,match_index)

                if skip > cnt :
                    skip_count = skip - cnt
                else:
                    skip_count = 0
                logging.info('findLogEntry:%r,%r,%r,%r,%r,%r,%r',file_path,match_value,match_index,msg_value,msg_index,skip_count,limit)
                ret, skip_records = PyInterface.findLogEntry(file_path,match_value,match_index,len(match_index), msg_value,msg_index,len(msg_value),skip_count,limit)
                if skip_records:
                    cnt += skip_records
                    matched += skip_records
                for alog in ret:
                    alog = alog.rstrip('\n').split('\t')
                    cnt += 1
                    matched += 1
                    if (cnt <= skip):
                        continue
                    aresult = []
                    for c in column:
                        aresult.append(alog[logformat.index(c)])
                    result.append(aresult)
                    if (cnt - skip) == limit:
                        logging.debug('query log sessionID:%r and return ',sessionID)
                        delete_sessionID_ts(sessionID)
                        return json.dumps({'results': result, 'cursor': make_cursor(new_cursor)})
                new_cursor[blk] = matched

        #get next 10 summaries
        start += rows
        try:
            #summaries = g_sumEngine.findSummaries(companyID, filters, ts1, ts2, start, rows)
            summaries = g_sumEngine.findSummariesEx(companyID,facet_name,filters, ts1, ts2, start, rows, scheduled_time_filter)
        except Exception,e:
            logging.exception('solr db error:%r',e)
            delete_sessionID_ts(sessionID)
            return json.dumps({'err_code':'timeout'})

        time_stamp_now = read_sessionID_ts(sessionID)
        logging.debug('&&&&&&&&&&&&&&&&&&&&&&&&query log sessionID:%r and check time_stamp:%r',sessionID,time_stamp_now)
        #if time_stamp_now and time_stamp_now != time_stamp:
        #    logging.info('***********888query log sessionID:%r and return duprep',sessionID)
        #    return json.dumps({'err_code':'dupreq'})
        #if time_stamp_now - time_stamp >= 20:
        #    logging.info('query log time out')
        #    logging.debug('query log sessionID:%r and return timeout',sessionID)
        #    return json.dumps({'err_code':'timeout'})
        #if time_stamp_now == 0 and int(time.time()) - time_stamp >= 20:
        #    logging.debug('query log sessionID:%r and ts == 0 return timeout',sessionID)
        #    return json.dumps({'err_code':'timeout'})
    logging.info('query log sessionID:%r and return ts:%r ',sessionID, time.time())
    delete_sessionID_ts(sessionID)
    logging.debug('query data:%r',result)
    return json.dumps({'results': result, 'cursor': make_cursor(new_cursor)})

def queryLogsStream(companyID, column, filters, timeBegin, timeEnd, facet_name, isRaw=True, scheduled_time_filter=None):

    #make filter to correct format
    match_value = []
    match_index = []
    msg_value = []
    msg_index = 1
    for i in range(0, len(filters)):
        try:
            c_index = logformat.index(filters[i]['col'])
        except ValueError,e:
            logging.exception('Error: %r. Ignore the filter: %r', e, filters[i]['col'])
            continue

        if (log_datatype[c_index] in ['int', 'bigint']):
            if filters[i]['op'] == 'eq':
                filters[i]['op'] = '=='
            elif filters[i]['op'] == 'neq':
                filters[i]['op'] = '!='
            if isinstance(filters[i]['value'], list):
                for j in range(0,len(filters[i]['value'])):
                    filters[i]['value'][j] = int(filters[i]['value'][j])
            else:
               filters[i]['value'] = int(filters[i]['value'])

        if(filters[i]['col'] == 'CLF_MsgType'):
            msg_value = copy.deepcopy(filters[i]['value'])
            logging.info('msg_value:%r',msg_value)
            msg_index = c_index
            if isinstance(msg_value, list):
                for k, msg_item in enumerate(msg_value):
                    msg_value[k] = str(msg_value[k])
            else:
                msg_value = [str(msg_value)]
        else:
            match_index.append(c_index)
            match_value.append(str(filters[i]['value']))

    logging.debug(filters)

    #round ts to minute
    ts1 = int(timeBegin)/60*60
    ts2 = int(timeEnd)/60*60
    start = 0
    rows = 10
    summaries = g_sumEngine.findSummariesEx(companyID,facet_name, filters, ts1, ts2, start, rows, scheduled_time_filter)

    #get raw logs
    result = []
    resultstr = ''
    cnt = 0
    log_block_dir = os.path.join(CLFCfg.config.get('storage', 'root'),CLFCfg.config.get('storage', 'log_block'),companyID)
    while summaries:
        for s in summaries:
            for blk in s['zLnk']:
               # logging.info('findLogEntry:%r,%r,%r,%r,%r,%r,%r',file_path,match_value,match_index,msg_value,msg_index,skip_count,limit)
                ret = []
                skip = 0
                next_flag=False
                logging.debug('file dir = %r',log_block_dir)
                file_path = log_block_dir+'/'+blk
                while 1:
                    ret,skip_records = PyInterface.findLogEntry(file_path,match_value,match_index,len(match_index), msg_value,msg_index,len(msg_value),skip,CLFCfg.CLF_QUERY_CHUNK_SIZE)
                    for alog in ret:
                        alog = alog.rstrip('\n').split('\t')
                        aresult = []
                        for c in column:
                            aresult.append(alog[logformat.index(c)])
                        if isRaw:
                            resultstr += '\t'.join(aresult)
                            resultstr += '\n'
                        else:
                            result.append(aresult)
                        cnt += 1
                        # use CLF_QUERY_CHUNK_SIZE as record count to separate chunk
                        if cnt >= CLFCfg.CLF_QUERY_CHUNK_SIZE:
                            if isRaw:
                                yield resultstr
                                resultstr = ''
                            else:
                                yield json.dumps(result)
                                result = []
                            cnt = 0
                    if len(ret) < CLFCfg.CLF_QUERY_CHUNK_SIZE:
                        break
                    else:
                        skip += CLFCfg.CLF_QUERY_CHUNK_SIZE
                #raw_logs = zlib.decompress(g_logEninge.getLogBlock(companyID, blk))
                #f = cStringIO.StringIO(raw_logs)
                #for alog in f:
                #    alog = alog.rstrip('\n').split('\t')
                #    if (check_log_entry(alog, filters, timeBegin, timeEnd)):
                #        aresult = []
                #        for c in column:
                #            aresult.append(alog[logformat.index(c)])
                #        if isRaw:
                #            resultstr += '\t'.join(aresult)
                #            resultstr += '\n'
                #        else:
                #            result.append(aresult)
                #        cnt += 1
                #        # use CLF_QUERY_CHUNK_SIZE as record count to separate chunk
                #        if cnt >= CLFCfg.CLF_QUERY_CHUNK_SIZE:
                #            if isRaw:
                #                yield resultstr
                #                resultstr = ''
                #            else:
                #                yield json.dumps(result)
                #                result = []
                #            cnt = 0
        #get next 10 summaries
        start += rows
        summaries = g_sumEngine.findSummariesEx(companyID,facet_name, filters, ts1, ts2, start, rows, scheduled_time_filter)

    if result or resultstr:
        if isRaw:
            yield resultstr
        else:
            yield json.dumps(result)


if __name__ == '__main__':
    pass

