/*drop table tb_EntityType cascade;
drop table tb_PolicyType cascade;
drop table tb_PriorityList cascade;
drop table tb_Policy cascade;
drop table tb_Rule cascade;
drop table tb_Entity cascade;
drop table tb_report_by; 
drop table tb_version;*/

--Create table tb_Policy
CREATE TABLE tb_Policy
(
	"PolicyID"            SERIAL CONSTRAINT PK_PolicyID PRIMARY KEY,
	"PolicyName"          varchar(256),
	"PolicyType"          SMALLINT NOT NULL,
	"IsEnable"            INT NULL DEFAULT 0,
	"IsDefault"           INT NULL DEFAULT 0,
        "CreationTime"        TIMESTAMPTZ NOT NULL DEFAULT 'now',
        "LastModifiedTime"    TIMESTAMPTZ NOT NULL DEFAULT 'now',
        "Note"                varchar(256)
);

-- Create table tb_rule
CREATE TABLE tb_Rule
(
	"RuleID"      SERIAL   CONSTRAINT PK_RuleID PRIMARY KEY,
	"PolicyID"    INT NOT NULL,
	"ActiveTime"  SMALLINT NOT NULL,
	"RuleValue"   text NULL
);  

-- Craete table for entity
CREATE TABLE tb_Entity
(
    "EntityID"    SERIAL  CONSTRAINT PK_EntityID PRIMARY KEY,
    "EntityType"  SMALLINT NOT NULL,
    "EntityName"  varchar(256) NULL
);  

-- Create table for entity and policy associateion
CREATE TABLE tb_PriorityList
(
    "EntityID"    INT NOT NULL,
    "PolicyID"    INT NOT NULL,
    "Priority"    SMALLINT NOT NULL,
    CONSTRAINT PK_PolicyID_EntityID PRIMARY KEY ("PolicyID", "EntityID")
); 

-- Create table for entity ID mapping
-- The values in this table are just for ad-hoc query reference.
-- The daemon won't use any values in this table for operation/calculation.
CREATE TABLE tb_EntityType
(
    "EntityType"      SMALLINT NOT NULL CONSTRAINT PK_EntityType PRIMARY KEY,
    "EntityTypeName"  varchar(20) NOT NULL,
    "Note"            text
);  

-- Create table for policy type mapping.
-- The column TimeToLive value is in minutes.
-- If the TimeToLive value is less or equal to 0, then the policy module will never 
-- look up new policy until force refresh.
CREATE TABLE tb_PolicyType
(
    "PolicyType"      SMALLINT NOT NULL CONSTRAINT PK_PolicyType PRIMARY KEY,
    "TypeName"        varchar(30) NOT NULL,
    "TimeToLive"      SMALLINT NOT NULL,
    "UpdateVersion"   INT,
    "Note"            varchar(256)
);  

-- Main table of approved lists 
CREATE TABLE tb_List
(
    "ListID"        SERIAL CONSTRAINT PK_ListID PRIMARY KEY,
    "ListType"      SMALLINT NOT NULL,
    "ListTableName" varchar(64) NOT NULL,
    "LastModifiedTime"  TIMESTAMPTZ NOT NULL DEFAULT 'now',
    "UpdateVersion"     INT NOT NULL DEFAULT 0,
    "Active"        SMALLINT NOT NULL
);

-- List type defination
--      1	Allowed URL List
--      2	Allowed File List          
--      100	Custom URL Categories List
CREATE TABLE tb_ListType
(
    "ListType"        SMALLINT NOT NULL CONSTRAINT PK_ListType PRIMARY KEY,
    "TypeName"        varchar(30) NOT NULL,
    "UpdateVersion"   INT NOT NULL DEFAULT 0,
    "Note"            varchar(256)
);

-- Mapping between approved lists and policies
CREATE TABLE tb_PolicyList
(
    "PolicyListID"     SERIAL   CONSTRAINT PK_PolicyListID PRIMARY KEY,
    "PolicyID"         INT NOT NULL,
    "ListID"           INT NOT NULL,
    "LastModifiedTime" TIMESTAMPTZ NOT NULL DEFAULT 'now',
    "Active"           SMALLINT NOT NULL,
    CONSTRAINT FK_tb_policy_list_policyid FOREIGN KEY ("PolicyID") REFERENCES tb_Policy ON DELETE cascade,
    CONSTRAINT FK_tb_policy_list_listid FOREIGN KEY ("ListID") REFERENCES tb_list ON DELETE cascade
);

-- Approved File Entries. Entries with the same ListID belongs to the
-- same list
CREATE TABLE tb_FileEntry
(
    "FileEntryID" SERIAL CONSTRAINT PK_FileEntryID PRIMARY KEY,
    "ListID"      INT NOT NULL REFERENCES tb_List,
    "FileName"    varchar (1024) NOT NULL,
    "LastModifiedTime"  TIMESTAMPTZ NOT NULL DEFAULT 'now',
    "Active"      SMALLINT NOT NULL 
);

-- Approved URL entries. Entries with the same ListID belongs to the
-- same list. The URL field is set to 2083, which is the maximum URL
-- length in Internet Explorer
CREATE TABLE tb_URLEntry
(
    "URLEntryID"  SERIAL CONSTRAINT PK_URLEntryID PRIMARY KEY,
    "ListID"      INT NOT NULL REFERENCES tb_List,
    "URL"         varchar(2083) NOT NULL,
    "LastModifiedTime"  TIMESTAMPTZ NOT NULL DEFAULT 'now',
    "Active"      SMALLINT NOT NULL 
);

-- Custom URL category entries. Entries with the same ListID belongs to the
-- same list. The URL field is set to 2083, which is the maximum URL
-- length in Internet Explorer
CREATE TABLE tb_CustomCategoryURLEntry
(
    "CustomCategoryURLEntryID"  SERIAL CONSTRAINT PK_CustomCategoryURLEntryID PRIMARY KEY,
    "ListID"            INT NOT NULL REFERENCES tb_List,
    "CustomCategoryURL" varchar(2083) NOT NULL,
    "LastModifiedTime"  TIMESTAMPTZ NOT NULL DEFAULT 'now',
    "Active"            SMALLINT NOT NULL
);

/* Create table for search_engine */    
CREATE TABLE tb_report_by
(
    period       INTEGER,   /* 0 daily, 1 weekly, 2 monthly ( same as quota period ) */
    category     INTEGER,   /* 1 USER ( user, hostname or IP type ), 2 GROUP ( group or range type ) */
    entity_type  INTEGER,   /* Entity type.  1 USER, 2 GROUP 3 HOSTNAME, 4 IP ADDRESS, 5 IP ADDRESS RANGE. */
                            /* Same as policy entity type */
    entity_name  VARCHAR( 256 )
);
CREATE INDEX index_report_by ON tb_report_by ( period, category ) ;

CREATE TABLE tb_Version
(
    version    varchar(256) NOT NULL 
);  

--Log tables. Added from IWSVA 6.0
--Create table tb_log_favorite
CREATE TABLE tb_log_favorite
(
	"FavoriteID"          SERIAL CONSTRAINT PK_FavoriteID PRIMARY KEY,
	"FavoriteName"        varchar(256),
    "Description"         text,
    "LogFilters"          text
);

--Create table tb_log_device
CREATE TABLE tb_log_device
(
    "DeviceName"          text CONSTRAINT PK_DeviceName PRIMARY KEY,
    "Version"             text,
    "Password"            text,
    "LastUploadTime"      timestamp with time zone
);

/* create table for report template */
CREATE TABLE tb_report_template
(
 "tid"                   varchar(64) CONSTRAINT PK_ReportTid PRIMARY KEY,  /* uuid */
 "name"                  varchar(256) not null,
 "description"           text,
 "enable"                boolean not null,  
 "period"                character varying(16),
 "from"                  int not null,           /* timestamp */
 "to"                    int not null,           /* timestamp */
 "frequency"             int not null,           /* 0 - now, 1 - once, 2 - daily, 3 - weekly, 4 - monthly */
 "scheduled"             boolean not null,       /* if frequency is 0, then scheduled is false, otherwise true */
 "start_date"            int not null,           /* timestamp that begin to generate report */
 "runtime"               varchar(32),            /* month:day:hour */
 "max_exec_number"       int not null,           /* 0 - no ending */
 "type"                  varchar(32),            /* PDF or HTML */
 "list_number"           int not null,           /* number of report items shown in report template page */
 "mail_enable"           boolean NOT NULL DEFAULT false,
 "mail_from"             text,                   /* sender address */
 "mail_to"               text,                   /* receiver address */
 "subject"               text,                   /* mail subject */
 "message"               text,                   /* mail message */
 "mail_attach"           boolean,                /* wheather including the report file */
 "fail_notice"           boolean,                /* wheather send the fail notification if sending mail fail */
 "fail_mail_to"          text,                   /* receiver address for fail notification */
 "report_by"             int not null,           /* 0 - all users, 1 - specific users, 2 - specific groups */
 "report_by_list"        text,                   /* user list or group list */
 "reports"               text,                   /* report detail items */
 "last_gen_time"         int not null DEFAULT 0, /* the time of last report generated */
 "creation_time"         timestamp with time zone NOT NULL DEFAULT CURRENT_TIMESTAMP,
 "current_exec_time"     integer NOT NULL DEFAULT 0,
 "scheduled_time_filter" integer NOT NULL DEFAULT 0	/*0 - always, 1 - working time 2 - leisure time*/     
);
/* create table for report item */
CREATE TABLE tb_report_item
(
 "rid"              varchar(64) CONSTRAINT PK_ReportRid PRIMARY KEY,   /* uuid */
 "tid"              varchar(256) not null  REFERENCES tb_report_template ON DELETE cascade,
 "gen_time"         int not null,                /* generation time of report */
 "html_file"        text,                    
 "pdf_file"         text,
 "type"             varchar(64) not null,
 "state"            int not null DEFAULT 1,      /* the state of this report file: 1*/
 "current_exec_time"    integer not null DEFAULT 0        
);
/* create table for CCR */
CREATE TABLE tb_configuration_replication
(
  "role_type" integer DEFAULT 0,
  "source_ip" character varying(30),
  "root_password" character varying(256),
  "scheduled_frequency" integer,
  "enabled" integer,
  "synch_time" timestamp with time zone,
  "creation_time" timestamp with time zone DEFAULT now(),
  "synch_status" integer DEFAULT 0
);