#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import logging
import sys
reload(sys)
sys.setdefaultencoding('utf-8')

import reportTemplateDef
from chart_handler   import HTMLReportGenerator
from chart_handler   import CSVReportGenerator
from chart_handler   import PDFReportGenerator
from report_template import ReportTemplate
from report_template import ReportInfo
from report_engine   import ReportEngine
from report_config   import ReportConfig
from email_sender    import EmailSender
from report_util     import util
from reportTemplateDef import chartType
from reportTemplateDef import LOG_CATEGORY_MAP

currentDir = os.path.dirname(os.path.abspath(__file__))

class ReportGenerator(object):

    def __init__(self, tid, tasktime):
        self.tid           = tid
        self.tasktime      = tasktime
        self.template      = None
        self.param_status  = False
        self.report_status = False
        self.mail_status   = False

    def prepareParam(self):
        logging.info('prepare report param: tid=%s', self.tid)
        try:
            templatedata = ReportEngine.getReportTemplate(self.tid)
            if templatedata:
                self.template = ReportTemplate(templatedata)
                self.info     = ReportInfo(self.template.name, self.template.timeFrom, self.template.timeTo, self.tasktime, self.template.timeType)
                self.param_status = True
        except Exception, e:
            logging.exception("prepare report param exception: %s", e)
        return self.param_status

    def buildReport(self):
        logging.info('build report: tid=%s', self.tid)
        try:
            if not self.param_status:
                return False

            self.rid = ReportEngine.createReportItem(self.tid, reportTemplateDef.reportStat['running'], self.tasktime)
            if self.rid:
                html_report, pdf_report, cvs_report = self._buildTemplateReport(self.template.reportParamList, self.info, self.rid)
                if html_report and pdf_report and cvs_report:
                    self.html_report = ReportEngine.saveReportFile(html_report)
                    self.pdf_report  = ReportEngine.saveReportFile(pdf_report)
                    self.cvs_report  = ReportEngine.saveReportFile(cvs_report)
                    ReportEngine.updateReportItemFile(self.rid, self.html_report, self.pdf_report, self.tid)
                    ReportEngine.updateReportState(self.rid, reportTemplateDef.reportStat['succeed'], self.tid)
                    self.report_status = True
                    ReportEngine.updateReportCurrentExecTime(self.tid)
                    logging.info('succeed to build reports: %s, %s, %s)', html_report, pdf_report, cvs_report)
                else:
                    logging.info('fail to build reports: %s, %s, %s', html_report, pdf_report, cvs_report)
                    self.report_status = False
                    ReportEngine.updateReportState(self.rid, reportTemplateDef.reportStat['failed'], self.tid)
            else:
                html_report, pdf_report, cvs_report = None, None, None
        except Exception, e:
            logging.exception('build report exception: %s', e)
            self._cleanfile(html_report)
            self._cleanfile(pdf_report)
            self._cleanfile(cvs_report)
            if self.rid:
                ReportEngine.updateReportState(self.rid, reportTemplateDef.reportStat['failed'], self.tid)
        finally:
            self._cleanReportFile()

        return self.report_status

    def _cleanfile(self, filename):
        if filename and os.path.isfile(filename):
            os.remove(filename)

    def _buildTemplateReport(self, reportParamList, reportInfo, reportFileName):
        logging.info('begin to generate report file')
        try:
            reportData = {}
            reportData['custom_reports'] = {}

            filter, LDAPGroups, ClientIPRange = {}, [], []
            if self.template.reportBy != 0:
                if self.template.username:
                    filter['CLF_UserName'] = self.template.username
                if len(self.template.clientip) > 0:
                    ClientIPRange.append({'type': 0, 'IP': self.template.clientip})
                for iprange in self.template.clientiprange:
                    items = iprange.replace(' ', '').split('~')
                    fromIP, toIP = items[0], items[1]
                    ClientIPRange.append({'type': 1, 'fromIP': fromIP, 'toIP': toIP})
                for ipsubnet in self.template.clientipsubnet:
                    items = ipsubnet.replace(' ', '').split('/')
                    subnet, prefixlen = items[0], items[1]
                    ClientIPRange.append({'type': 2, 'subnet': subnet, 'prefixlen': prefixlen})
                LDAPGroups    = self.template.groupname

            for parameter in reportParamList:
                data = ReportEngine.getReportData(parameter.reportName,
                                                  reportInfo.timeFrom,
                                                  reportInfo.timeTo,
                                                  parameter.filters if parameter.custom else filter,
                                                  parameter.chartType,
                                                  parameter.topN,
                                                  LDAPGroups,
                                                  ClientIPRange,
                                                  self.template.timeType,
                                                  parameter.custom)

                if parameter.custom:
                    reportData['custom_reports'][parameter.favID] = data
                    logging.debug('get report data: (%s:%s)', parameter.favID, data)
                else:
                    reportData[parameter.reportName] = data
                    logging.debug('get report data: (%s:%s)', parameter.reportName, data)

                #For custom report, need to get time chart data if requested.
                if parameter.custom and parameter.chartType == chartType['line']:
                    data = ReportEngine.getReportData(LOG_CATEGORY_MAP[parameter.reportName]['timeChart'],
                                                  reportInfo.timeFrom,
                                                  reportInfo.timeTo,
                                                  parameter.filters,
                                                  parameter.chartType,
                                                  parameter.topN,
                                                  None,
                                                  None,
                                                  self.template.timeType,
                                                  False)

                    reportData['custom_reports']['/'.join([parameter.favID, 'timeChart'])] = data
                    logging.debug('get report data: (%s:%s)', '/'.join([parameter.favID, 'timeChart']), data)

                # For custom report, need to get total number for pie chart.
                if parameter.custom and parameter.chartType == chartType['pie']:
                    data = ReportEngine.getReportData(LOG_CATEGORY_MAP[parameter.reportName]['facet'].replace('facet_', 'facet_total_'),
                                                  reportInfo.timeFrom,
                                                  reportInfo.timeTo,
                                                  parameter.filters,
                                                  parameter.chartType,
                                                  100,
                                                  None,
                                                  None,
                                                  self.template.timeType,
                                                  False)

                    total = 0
                    if data:
                        for x in data:
                            total += int(x[1])
                    reportData['custom_reports']['/'.join([parameter.favID, 'total'])] = total
                    logging.debug('get report data: (%s:%s)', '/'.join([parameter.favID, 'total']), total)


            chartBuilder = HTMLReportGenerator(reportParamList, reportInfo, reportData)
            htmlReport, htmlReportForPdf = chartBuilder.buildReport(reportFileName)
            pdfReport    = PDFReportGenerator.buildReport(htmlReportForPdf, reportFileName)
            csvBuilder   = CSVReportGenerator(reportParamList, reportInfo, reportData)
            csvReport    = csvBuilder.buildReport(reportFileName)

            self.htmReportForPdf = htmlReportForPdf
        except Exception, e:
            htmlReport, htmlReportForPdf, csvReport = None, None, None
            logging.exception('generate report file exception: %s', e)
        return (htmlReport, pdfReport, csvReport)

    def sendReportMail(self):
        logging.info('begin to send notifiction mail')
        try:
            if not self.template.mailEnable:
                logging.info('No need sending notification.')
                return False

            if not self.report_status:
                logging.info('No report, no need sending notification.')
                return False

            attachFiles = []
            displayNames = []
            if self.template.type == 'HTML' and self.template.mailAttach:
                if self.htmReportForPdf:
                    attachFiles.append(self.htmReportForPdf)
                    attachFiles.append(ReportConfig.REPORT_TEMP_LOCATION + '/highcharts_report.js')
                    attachFiles.append(ReportConfig.REPORT_TEMP_LOCATION + '/jquery-1.7.1_report.js')
                    attachFiles.append(ReportConfig.REPORT_TEMP_LOCATION + '/logo.gif')
                    displayNames = ['IWSVA_Report.html','highcharts_report.js', 'jquery-1.7.1_report.js', 'logo.gif']
            elif self.template.type == 'PDF' and self.template.mailAttach:
                if self.pdf_report:
                    attachFiles.append(self.pdf_report)
                    displayNames = ['IWSVA_Report.pdf']
            else:
                if self.cvs_report and self.template.mailAttach:
                    attachFiles.append(self.cvs_report)
                    displayNames = ['IWSVA_Report.csv']

            self.mail_status = self._sendMail(self.template.name, self.template.mailFrom, self.template.mailTo, attachFiles, self.template.subject, self.template.message, displayNames)
            if not self.mail_status:
                logging.info('fail to send notification with attached report file')
                self.mail_status  = self._sendMail(self.template.name, self.template.mailFrom, self.template.failMailTo, [], self.template.subject, self.template.message, [])
        except Exception, e:
            logging.exception('send report mail exception: %s', e)
        finally:
            self._cleanfile(self.htmReportForPdf)

        return self.mail_status

    def _sendMail(self, report_name, from_addr, to_addr, attach_files, subject, content, display_names=[]):
        logging.debug('_send mail notification')
        try:
            sender = EmailSender(ReportConfig.EMAIL_SERVER_IP, ReportConfig.EMAIL_SERVER_PORT)
            result = sender.send_email(from_addr,
                                	   to_addr,
                                       subject.encode('utf-8'),
                                       content.encode('utf-8'),
                                       None,
                                       attach_files,
                                       file_display_name = display_names)
        except Exception, e:
            result = False
            logging.exception('_send mail exception: %s', e)
        return result

    def _cleanReportFile(self):
        logging.debug('clean Report files')
        try:
            reportItemList = ReportEngine.listReportItem(self.tid)
            # clean report file on disk
            if len(reportItemList) > self.template.listNumber:
                filetypes = ['.pdf', '.csv', '.html']
                for i in xrange(self.template.listNumber, len(reportItemList)):
                    reportItem = reportItemList[i]
                    rid = reportItem['rid']
                    for filetype in filetypes:
                        filepath = ReportConfig.REPORT_SAVE_LOCATION + rid + filetype
                        if os.path.exists(filepath):
                            os.remove(filepath)
            # clean database
            ReportEngine.cleanReportItem(self.tid, self.template.listNumber)
        except Exception, e:
            logging.exception('clean report files exception: %s', e)


if __name__ == '__main__':
    pass
