#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import logging.config
import sys
reload(sys)
sys.setdefaultencoding('utf-8')

import json
import zlib

import statDef
import reportDef
from stats_engine import StatsEngine
from clfcfg import CLFCfg
import report
import LDAP_query_handler
import clfutil

class ReportManager(object):

    def __init__(self):
        self.__stats_engine = StatsEngine()
        self.LDAPQueryHandler = LDAP_query_handler.CreateLDAPQueryHandler()
        CLFCfg.init()

    def build_report(self, rep_name, companyID, timeBegin, timeEnd, filter, LDAPGroupFilter=[], IPRanges=[], scheduledTime=None):
        '''
            Get the report data.
            'filter': {'col1': ['value'], 'col2': ['value']}
            'LDAPGroupFilter' : ['LDAPGroup1', 'LDAPGroup2', ...]
            'IPRanges': [{'type': 0, 'IP': ['IP1', 'IP2', ...]},
                        {'type': 1, 'fromIP': 'xxx': 'toIP': 'xxx'},
                        {'type': 2, 'subnet': 'xxx': 'prefixlen': 'xxx'}
                       ]
        '''
        try:
            rep_def = reportDef.reports[rep_name]
        except KeyError:
            raise 'No report definition found for ' + rep_name

        rep = report.ReportCreator.create_report(rep_def, 0, self)

        #will generate report for the groups in filter
        if LDAPGroupFilter or IPRanges:
            rep.set_groups(LDAPGroupFilter, IPRanges)

        stat_def = statDef.reportStats[rep_def['statName']]
        stats = self.__stats_engine.findStats(companyID, timeBegin, timeEnd, rep_def['statName'])
        for stat in stats:
            ts = stat[CLFCfg.CLF_TimeStamp]
            if stat[CLFCfg.CLF_StatsZip] == 1:
                stat = json.loads(zlib.decompress(stat[CLFCfg.CLF_Stats]))
            else:
                stat = json.loads(stat[CLFCfg.CLF_Stats])
            #stat = stat[rep_def['statName']]

            for s in stat:
                match = True
                #match all filters
                for k in filter.keys():
                    #ignore filters that are not in stats
                    if not k in stat_def['logFields']:
                        continue

                    if not isinstance(filter[k], list):
                        filter[k] = [filter[k]]
                    if not s[stat_def['logFields'].index(k)] in filter[k]:
                        match = False
                        break
                if match:  #matched all the other filters. Should match one of IP ranges or LDAP groups
                    done = False
                    if IPRanges:
                        clientIP = s[stat_def['logFields'].index('CLF_ClientIP')]
                        for ipr in IPRanges:
                            if clfutil.IPRange_Contains(ipr, clientIP):
                                rep.process_stat(s, ts)
                                done = True
                                break

                    if (not done) and (LDAPGroupFilter and self.LDAPQueryHandler):
                        for g in LDAPGroupFilter:
                            if self.LDAPQueryHandler.is_user_in_group(
                                    s[stat_def['logFields'].index('CLF_UserName')], g):
                                rep.process_stat(s, ts)
                                done = True
                                break

                    if not done:
                        if (not IPRanges) and (not (LDAPGroupFilter and self.LDAPQueryHandler)):
                            rep.process_stat(s, ts)
        return rep

    def build_reportEx(self, rep_name, companyID, timeBegin, timeEnd, filter, LDAPGroupFilter=[],IPRanges=[], scheduledTime=None):
        '''
            Get the report data.
            'filter': {'col1': ['value'], 'col2': ['value']}
            'LDAPGroupFilter' : ['LDAPGroup1', 'LDAPGroup2', ...]
            'IPRanges': [{'type': 0, 'IP': ['IP1', 'IP2', ...]},
                        {'type': 1, 'fromIP': 'xxx': 'toIP': 'xxx'},
                        {'type': 2, 'subnet': 'xxx': 'prefixlen': 'xxx'}
                       ]
        '''
        try:
            rep_def = reportDef.reports[rep_name]
        except KeyError:
            raise 'No report definition found for ' + rep_name

        rep = report.ReportCreator.create_report(rep_def, 0, self)
        stat_def = statDef.reportStats[rep_def['statName']]
        if LDAPGroupFilter or IPRanges:
            rep.set_groups(LDAPGroupFilter, IPRanges)

        for k in filter.keys():
            #remove filters that is not in stats
            if not k in stat_def['logFields']:
                filter.pop(k)
                continue

            if not isinstance(filter[k], list):
                filter[k] = [filter[k]]

        #def findStatsEx(self,companyID, timeBegin, timeEnd, stat_name,top_name,filter,is_ldap_filter, reportType):
        if (LDAPGroupFilter and self.LDAPQueryHandler) or IPRanges:
            user_filter = []

            if filter.has_key('CLF_UserName'):
                user_filter = filter.pop('CLF_UserName')

            Ldap_IP_filter = []
            CLF_ClientIP_index = -1
            CLF_UserName_index = -1
            masterColumnID_index = 0
            if LDAPGroupFilter and self.LDAPQueryHandler:
                Ldap_IP_filter.append('CLF_UserName')
                masterColumnID_index += 1
            if IPRanges:
                Ldap_IP_filter.append('CLF_ClientIP')
                masterColumnID_index += 1
            CLF_ClientIP_index = Ldap_IP_filter
            stats = self.__stats_engine.findStatsEx(companyID, timeBegin, timeEnd, rep_def['statName'],rep_def['masterColumnId'],filter, Ldap_IP_filter,rep_def['type'], scheduledTime)

            if LDAPGroupFilter and self.LDAPQueryHandler:
                CLF_UserName_index = Ldap_IP_filter.index('CLF_UserName')
            if IPRanges:
                CLF_ClientIP_index = Ldap_IP_filter.index('CLF_ClientIP')
            for stat_item in stats:  #  for example stats [('use','ip','name',10),('use','ip','name',20)....]
                                     # trending stats {(user,ip):{ts:xxx,ts1:xx},(user,ip):{ts:xx,ts1:xx}}
                                     # multi-trending { (user,ip,name):{ts:xxx,ts1:xx},(user,ip,name):{ts:xx,ts1:xx}}
                if user_filter:
                    if (rep_def['type'] == reportDef.reportType['trending']):
                        if stat_item[CLF_UserName_index] in user_filter:
                            rep.process_merge_stat_ldap(stats[stat_item], CLF_UserName_index, CLF_ClientIP_index)
                            continue
                    elif (rep_def['type'] == reportDef.reportType['multiTrending']):
                        if stat_item[CLF_UserName_index] in user_filter:
                            tmp_stats = {stat_item[masterColumnID_index]:stats[stat_item]}
                            rep.process_merge_stat_ldap(tmp_stats, CLF_UserName_index, CLF_ClientIP_index)
                            continue
                    else:
                        if stat_item[stat_def['logFields'].index('CLF_UserName')] in user_filter:
                            rep.process_merge_stat_ldap(stat_item, CLF_UserName_index, CLF_ClientIP_index)
                            continue

                done = False
                if IPRanges:
                    if (rep_def['type'] == reportDef.reportType['trending']):
                        clientIP = stat_item[CLF_ClientIP_index]
                        for ipr in IPRanges:
                            if clfutil.IPRange_Contains(ipr, clientIP):
                                rep.process_merge_stat_ldap(stats[stat_item], CLF_UserName_index, CLF_ClientIP_index)
                                done = True
                                break
                    elif (rep_def['type'] == reportDef.reportType['multiTrending']):
                        clientIP = stat_item[CLF_ClientIP_index]
                        for ipr in IPRanges:
                            if clfutil.IPRange_Contains(ipr, clientIP):
                                tmp_stats = {stat_item[masterColumnID_index]:stats[stat_item]}
                                rep.process_merge_stat_ldap(tmp_stats, CLF_UserName_index, CLF_ClientIP_index)
                                done = True
                                break
                    else:
                        clientIP = stat_item[stat_def['logFields'].index('CLF_ClientIP')]
                        for ipr in IPRanges:
                            if clfutil.IPRange_Contains(ipr, clientIP):
                                rep.process_merge_stat_ldap(stat_item, CLF_UserName_index, CLF_ClientIP_index)
                                done = True
                                break

                if not done:
                    for g in LDAPGroupFilter:
                        #if self.LDAPQueryHandler.is_user_in_group(stat_item[stat_def['logFields'].index('CLF_UserName')], g):
                        if (rep_def['type'] == reportDef.reportType['trending']):
                            if self.LDAPQueryHandler.is_user_in_group(stat_item[CLF_UserName_index], g):
                                rep.process_merge_stat_ldap(stats[stat_item], CLF_UserName_index, CLF_ClientIP_index)
                                break
                        elif (rep_def['type'] == reportDef.reportType['multiTrending']):
                            if self.LDAPQueryHandler.is_user_in_group(stat_item[CLF_UserName_index], g):
                                tmp_stats = {stat_item[masterColumnID_index]:stats[stat_item]}
                                rep.process_merge_stat_ldap(tmp_stats, CLF_UserName_index, CLF_ClientIP_index)
                                break
                        else:
                            if self.LDAPQueryHandler.is_user_in_group(stat_item[stat_def['logFields'].index('CLF_UserName')], g):
                                rep.process_merge_stat_ldap(stat_item, CLF_UserName_index, CLF_ClientIP_index)
                                break

        else:
            # findstast: cid, start, end, stats_name, top_name, filter, is_ldap_filter, report_type
            stats = self.__stats_engine.findStatsEx(companyID, timeBegin, timeEnd, rep_def['statName'],rep_def['masterColumnId'],filter, None,rep_def['type'], scheduledTime)
            rep.process_merge_stat_noldap(stats,timeBegin,rep_def['masterColumnId'])
        return rep


if __name__ == '__main__':
    pass

