#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import time
import datetime
import calendar
import reportTemplateDef
from report_engine import ReportEngine
from report_util   import util
class ReportParam():
    def __init__(self, reportName, topN = 0, filters = {}, chartType = 0, displayName = '', custom = False, favID = '0', viewChart = '', columns = ''):
        self.reportName  = reportName
        self.topN        = topN
        self.filters     = filters
        self.chartType   = chartType
        self.viewChart   = viewChart
        self.displayName = displayName
        self.custom = custom
        self.favID = favID
        self.columns = columns

class ReportInfo():
    def __init__(self, reportTitle, timeFrom, timeTo, taskTime, timeType=0):
        logging.debug('reportInfo.__init__')
        self.reportTitle = reportTitle
        self.timeFrom    = timeFrom
        self.timeTo      = timeTo
        self.taskTime    = taskTime
        self.timeType    = timeType
        logging.info('report infomation: title(%s), timeFrom(%s), timeTo(%s), timeType(%r)' % (reportTitle, timeFrom, timeTo, timeType))

class ReportTemplate(object):
    def __init__(self, template):
        logging.debug('ReportTemplate.__init__: tid(%s)', template['tid'])
        self.tid            = template['tid']
        self.name           = template['name']
        self.description    = template['description']
        self.enable         = template['enable']
        self.period         = template['period']
        self.timeFrom       = template['from']
        self.timeTo         = template['to']
        self.frequency      = template['frequency']
        self.startDate      = template['start_date']
        self.timeFrom, self.timeTo = self._getTimeRange(self.period, self.timeFrom, self.timeTo)

        runTimeItems = template['runtime'].split(':')
        self.runTimeDay     = int(runTimeItems[0])
        self.runTimeHour    = int(runTimeItems[1])
        self.runTimeMinute  = int(runTimeItems[2])
        self.maxExecNumber  = template['max_exec_number']
        self.type           = template['type']
        self.listNumber     = template['list_number']
        self.mailEnable     = template['mail_enable']
        self.mailFrom       = template['mail_from']
        self.mailTo         = template['mail_to']
        self.subject        = template['subject']
        self.message        = template['message']
        self.mailAttach     = template['mail_attach']
        self.failNotice     = template['fail_notice']
        self.failMailTo     = template['fail_mail_to']

        self.reportBy       = template['report_by']
        self.reportByList   = template['report_by_list']
        self.timeType       = template['scheduled_time']


        if self.reportBy != 0:
            self.username, self.clientip = [], []
            self.groupname, self.clientipsubnet, self.clientiprange = [], [], []
        self._get_specific_list()

        self.lastReportTime = template['last_gen_time']
        self.currentExecTime = template['current_exec_time']

        self.reportParamList = []
        reportType = ["internet_security", "internet_access", "bandwidth", "policy_enforcement", "data_security", "custom_reports"]
        for typeItem in reportType:
            if typeItem in template['reports']:
                items = template['reports'][typeItem]
                for item in items:
                    self._addReportParam(item, typeItem == 'custom_reports')

    def _getTimeRange(self, period, timeFrom, timeTo):
        logging.debug('get time range: period(%s)', period)
        periodType = (period[-1]).upper()
        endDate = datetime.datetime.date(datetime.datetime.now())

        deltatime = datetime.timedelta(days=1)
        if periodType == 'D':
            days      = int(period[0])
            beginDate = endDate - deltatime * days
            timeFrom  = time.mktime(beginDate.timetuple())
            timeTo    = time.mktime(endDate.timetuple()) - 1
        elif periodType == 'W':
            weeks     = int(period[0])
            beginDate = endDate - deltatime * 7 * weeks
            timeFrom  = time.mktime(beginDate.timetuple())
            timeTo    = time.mktime(endDate.timetuple()) - 1
        elif periodType == 'M':
            months     = int(period[0])
            beginDay   = endDate.day
            beginMonth = endDate.month
            beginYear  = endDate.year
            if beginMonth <= months:
                beginYear -= 1
                beginMonth = beginMonth + 12 - months
            else:
                beginMonth -= months
            maxMonthDay = calendar.monthrange(beginYear, beginMonth)[1]
            beginDay = min(beginDay, maxMonthDay)
            beginDate = datetime.datetime(beginYear, beginMonth, beginDay)
            timeFrom  = time.mktime(beginDate.timetuple())
            timeTo    = time.mktime(endDate.timetuple()) - 1

        logging.debug('calculate time range result: from(%s), to(%s)', timeFrom, timeTo)
        return int(timeFrom), int(timeTo)

    def _addReportParam(self, item, custom=False):
        logging.debug('add report parameters: %s', item)

        filter = {}
        favID = 0
        viewChart = ''
        columns = ''
        if custom:
            #get log favorite definition
            favID = str(item)
            fav = ReportEngine.getLogFavorite(favID)
            if not fav:
                logging.info('_addReportParam cannot find favorite [%r]', favID)
                return

            reportName = fav[3]['logCategory']    #report id in reportDef.py
            topN       = fav[3]['topN']
            enable     = True
            chartType  = fav[3]['chartType']
            displayName = fav[1]
            filter = fav[3]['filters']
            viewChart = fav[3]['viewChart']
            columns = fav[3]['cols']['c']
        else:
            reportName = item[0]
            topN       = item[1]
            enable     = item[2]
            chartType  = item[3]
            if enable:
                displayName = util.getReportDisplayName(reportName)

        if custom or enable:
            param = ReportParam(reportName, topN, filter, chartType, displayName, custom, favID, viewChart, columns)
            self.reportParamList.append(param)
            logging.debug('parameters: %s,%s,%s,%s,%s,custom report:%s, view chart:%s', reportName,topN, filter,chartType,reportName, custom, viewChart)

    def _get_specific_list(self):
        if self.reportBy != 0:
            if self.reportByList.has_key('user'):
                self.username = self.reportByList['user']
            if self.reportByList.has_key('ip'):
                self.clientip = self.reportByList['ip']
            if self.reportByList.has_key('group'):
                self.groupname = self.reportByList['group']
            if self.reportByList.has_key('ip mask'):
                self.clientipsubnet = self.reportByList['ip mask']
            if self.reportByList.has_key('ip range'):
                self.clientiprange = self.reportByList['ip range']

    def isTimeInRange(self, dateInput, dateNow):
        logging.debug('ReportTemplate.isTimeInRange: lastDate(%s), nowDate(%s)', dateInput, dateNow)

        ret = False
        if self.frequency == reportTemplateDef.reportFrequency['daily'] or self.frequency == reportTemplateDef.reportFrequency['once']:
            if dateInput.day == dateNow.day and dateInput.month == dateNow.month and dateInput.year == dateNow.year:
               ret = True
        elif self.frequency == reportTemplateDef.reportFrequency['weekly']:
            if dateInput.day - dateInput.isoweekday() % 7 == dateNow.day - dateNow.isoweekday() % 7 and dateInput.month == dateNow.month and dateInput.year == dateNow.year:
               ret = True
        elif self.frequency == reportTemplateDef.reportFrequency['monthly']:
            if dateInput.month == dateNow.month and dateInput.year == dateNow.year:
               ret = True
        return ret

    def checkExecState(self):
        logging.debug('ReportTemplate.checkExecState')

        # check the max execute times
        logging.debug("currently exec time: %s", self.currentExecTime)
        if self.maxExecNumber > 0 and self.currentExecTime >= self.maxExecNumber:
            return False
        return True

    def isTimeToCreateReport(self):
        logging.debug('ReportTemplate.isTimeToCreateReport')

        ret = 0
        nowUTCtime = calendar.timegm(time.gmtime())

        # check the start time
        logging.debug("startDatetime(%s), nowUTCtime(%s)", self.startDate, nowUTCtime)
        if nowUTCtime < self.startDate:
            return 0

        # check the max execute times
        logging.debug("currently exec time: %s, max exce number:%s", self.currentExecTime, self.maxExecNumber)
        if self.maxExecNumber > 0 and self.currentExecTime >= self.maxExecNumber:
            return 0

        # check last generation time
        lastReportDate = datetime.datetime.fromtimestamp(self.lastReportTime)
        nowDate        = datetime.datetime.fromtimestamp(nowUTCtime)

        # check the day number of month
        maxMonthDay = calendar.monthrange(nowDate.year, nowDate.month)[1]
        day = min(self.runTimeDay, maxMonthDay)
        if day != self.runTimeDay:
            logging.info('change day from %d to %d', self.runTimeDay, day)
            self.runTimeDay = day

        if self.lastReportTime:
            if self.isTimeInRange(lastReportDate, nowDate):
                ret = 0
            else:
                (isTimeTo, nextDate) = self._isTimeToCreateReport(nowDate)
                if isTimeTo:
                    ret = time.mktime(nextDate.timetuple())
        else:
            (isTimeTo, nextDate) = self._isTimeToCreateReport(nowDate)
            if isTimeTo:
                ret = time.mktime(nextDate.timetuple())

        return ret

    def _isTimeToCreateReport(self, nowDate):
        logging.debug('ReportTemplate._isTimeToCreateReport')
        logging.debug('now date: %s %s %s', nowDate.day, nowDate.hour, nowDate.minute)
        logging.debug('run time: %s %s %s', self.runTimeDay, self.runTimeHour, self.runTimeMinute)

        result = False
        if self.frequency == reportTemplateDef.reportFrequency['daily'] or self.frequency == reportTemplateDef.reportFrequency['once']:
            if self.runTimeHour == 0:
                if nowDate.hour == 0:
                    deltaSecond = abs(nowDate.minute - self.runTimeMinute ) * 60 + nowDate.second
                    logging.debug("delta second(daily or once): %s", deltaSecond)
                    if deltaSecond <= 2700:
                        result = True
            else:
                deltaSecond = abs((nowDate.hour * 3600 + nowDate.minute * 60 + nowDate.second) - (self.runTimeHour * 3600 + self.runTimeMinute * 60))
                logging.debug('delta second(daily or once): %s', deltaSecond)
                if deltaSecond <= 1800:
                    result = True

        elif self.frequency == reportTemplateDef.reportFrequency['weekly']:  # run time for weekly- day:hour:minute, and sun, mon, tue, wed, thu, fri, sat(0, 1, 2, 3, 4, 5, 6)
            if self.runTimeDay == 0 and self.runTimeHour == 0:
                if nowDate.isoweekday() % 7  == 0 and nowDate.hour == 0:
                    deltaSecond = nowDate.minute * 60 + nowDate.second
                    logging.debug("delta second(weekly): %s", deltaSecond)
                    if deltaSecond <= 2700:
                        result = True
            else:
                nowWeekSecond = (nowDate.isoweekday() % 7) * 24 * 3600 + nowDate.hour * 3600 + nowDate.minute * 60 + nowDate.second
                runTimeSecond = self.runTimeDay * 24 * 3600 + self.runTimeHour * 3600 + self.runTimeMinute * 60
                deltaSecond   = abs(nowWeekSecond - runTimeSecond)
                logging.debug("delta second(weekly): %s", deltaSecond)
                if deltaSecond <= 1800:
                    result = True
        elif self.frequency == reportTemplateDef.reportFrequency['monthly']:
            if self.runTimeDay ==1 and self.runTimeHour == 0:
                if nowDate.day == 1 and nowDate.hour == 0:
                    deltaSecond = nowDate.minute * 60 + nowDate.second
                    logging.debug("delta second(monthly): %s", deltaSecond)
                    if deltaSecond <= 2700:
                        result = True
            else:
                nowMonthSecond = (nowDate.day - 1) * 24 * 3600 + nowDate.hour * 3600 + nowDate.minute * 60 + nowDate.second
                runTimeSecond  = (self.runTimeDay - 1) * 24 * 3600 + self.runTimeHour * 3600 + self.runTimeMinute * 60
                deltaSecond    = abs(nowMonthSecond - runTimeSecond)
                logging.debug("delta second(monthly): %s", deltaSecond)
                if deltaSecond <= 1800:
                    result = True

        ret = (False, None)
        if result:
            if self.frequency == reportTemplateDef.reportFrequency['daily'] or self.frequency == reportTemplateDef.reportFrequency['once']:
                ret = (True, datetime.datetime(nowDate.year, nowDate.month, nowDate.day, self.runTimeHour, self.runTimeMinute))
            else:
                ret = (True, datetime.datetime(nowDate.year, nowDate.month, nowDate.day, self.runTimeHour, self.runTimeMinute))
        return ret

if __name__ == '__main__':
    pass
