#!/bin/sh

SCRIPT_NAME=`/bin/basename $0`

MODPROBE="/sbin/modprobe"
INSMOD="/sbin/insmod"
LSMOD="/sbin/lsmod"
GREP="/bin/grep"
AWK="/bin/awk"
CAT="/bin/cat"
MKNOD="/bin/mknod"
LN="/bin/ln"
CHMOD="/bin/chmod"
RM="/bin/rm"

OPT_KMOD_DIR="/opt/lanbypass/lib/modules"
BP_SD_KMOD_NAME="bpctl_mod"
BP_SD_KMOD="${OPT_KMOD_DIR}/${BP_SD_KMOD_NAME}.ko"
BP_DEV_NODE="bpctl"

BP_SD_DRV="e1000"
BP_DRV="e1000bp"

is_vm()
{
    #use a hidden key to control whether enable lanbypass support on VM.
    grep "^lanbypass_vm" /etc/iscan/intscan.ini | grep "yes" > /dev/null && /usr/iwss/vmdetector vmware-open-vm-tools
}
if is_vm ;then
    BPCTL="/bin/bpvmctl"
else
    BPCTL="/bin/bpctl"
fi

# default value of watchdog
WD_AUTORESET=1000
WD_TIMEOUT=6400

BP_SD_init() 
{
	${LSMOD} | ${GREP} ${BP_SD_DRV} > /dev/null 2>&1
	if [ "$?" != "0" ] ; then
		${MODPROBE} ${BP_SD_DRV} > /dev/null 2>&1
	fi

	${LSMOD} | ${GREP} ${BP_SD_KMOD_NAME} > /dev/null 2>&1
	if [ "$?" != "0" ] ; then
		${INSMOD} ${BP_SD_KMOD} > /dev/null 2>&1
	fi

	${LSMOD} | ${GREP} ${BP_SD_KMOD_NAME} > /dev/null 2>&1
	ret="$?"

	# mknod when bpctl_mod is inserted
	
	if [ "${ret}" = "0" ] ; then	
		# remove old device node
		${RM} -f /dev/${BP_DEV_NODE}?

		major=`${CAT} /proc/devices | \
			${AWK} "\\$2==\"${BP_DEV_NODE}\" {print \\$1}"`
	
		# make new node
		${MKNOD} /dev/${BP_DEV_NODE}0 c ${major} 0
		${LN} -sf /dev/${BP_DEV_NODE}0  /dev/${BP_DEV_NODE}
		${CHMOD} 644 /dev/${BP_DEV_NODE}0
	fi

	# return 0 if insmod successfully
	return "${ret}"
}

BP_init()
{
	${LSMOD} | ${GREP} ${BP_DRV} > /dev/null 2>&1
	if [ "$?" != "0" ] ; then
		${MODPROBE} ${BP_DRV} > /dev/null 2>&1
	fi

	# return 0 if insmod successfully
	${LSMOD} | ${GREP} ${BP_DRV} > /dev/null 2>&1
	return "$?"
}

init()
{
	ret=1

	BP_SD_init
	if [ "$?" = "0" ] ; then
		ret=0
	fi

	BP_init
	if [ "$?" = "0" ] ; then
		ret=0
	fi

	return ${ret}
}

is_bypass()
{
	${BPCTL} $1 is_bypass | ${GREP} \
		"The interface is a control interface." > /dev/null 2>&1

	# return 0 if specified interface is a control interface
	return $?
}

get_slave()
{
	if is_vm ;then
		get_slave_vm
	else
		${BPCTL} $1 get_bypass_slave | \
		  ${GREP} "^[[:alnum:]]* is a slave port.$" |
		  ${AWK} -F " " '{print $1}'
	
	fi
}

get_slave_vm()
{
	local nics=`ls /sys/class/net | grep "^eth[0-9]*$"`
	for eth in $nics; do
        ${BPCTL} $eth is_bypass 2>&1 | ${GREP} -v 'recv' | ${GREP} "The interface is a slave interface." >/dev/null 2>&1
        if [ "$?" = "0" ]; then
            echo $eth
            break
        fi
    done
}

bypass_now()
{
	eth=$1
	action=$2

	if [ "$#" -lt 2 ] ; then
		echo "Usage: ${SCRIPT_NAME} bypass_now <eth> <action>"
		return 1
	fi
	
	if [ "${action}" = "on" ] ; then
		${BPCTL} ${eth} set_std_nic off > /dev/null 2>&1 
		${BPCTL} ${eth} set_bypass on > /dev/null 2>&1
	elif [ "${action}" = "off" ] ; then
		${BPCTL} ${eth} set_bypass off > /dev/null 2>&1
	fi

	return 0
}

enable_bypass()
{
	eth=$1
	action=$2
	timeout=$3
	autoreset=$4

	if [ "$#" -lt 2 ] ; then
		echo -n "Usage: ${SCRIPT_NAME} enable_bypass <eth> <action> "
		echo "[<timeout> <autoreset>]"
		return 1
	fi

	if [ "${action}" = "on" ] ; then
		if [ "${timeout}" = "-1" ] ; then
			WD_TIMEOUT="0"
		elif [ "${timeout}" != "" ] ; then
			WD_TIMEOUT="${timeout}"
		fi
		
		if [ "${autoreset}" = "-1" ] ; then
			WD_AUTORESET="0"
		elif [ "${autoreset}" != "" ] ; then
			WD_AUTORESET="${autoreset}"
		fi

		${BPCTL} ${eth} set_std_nic off > /dev/null 2>&1
		${BPCTL} ${eth} set_bypass_pwoff on > /dev/null 2>&1
		${BPCTL} ${eth} set_bypass_pwup on > /dev/null 2>&1
		if is_vm ; then
            ${BPCTL} ${eth} set_wd_autoreset 0 > /dev/null 2>&1
            start_reset_wd ${eth}
        else
			${BPCTL} ${eth} set_wd_autoreset ${WD_AUTORESET} > /dev/null 2>&1
		fi
		${BPCTL} ${eth} set_bypass_wd ${WD_TIMEOUT} > /dev/null 2>&1
	
	elif [ "${action}" = "off" ] ; then
		${BPCTL} ${eth} set_wd_autoreset 0 > /dev/null 2>&1
		${BPCTL} ${eth} set_bypass_wd 0 > /dev/null 2>&1
		${BPCTL} ${eth} set_bypass_pwup off > /dev/null 2>&1
		${BPCTL} ${eth} set_bypass_pwoff off > /dev/null 2>&1
		${BPCTL} ${eth} set_std_nic on > /dev/null 2>&1
        if is_vm; then
            kill `ps aux | grep reset_wd | grep -v grep | awk '{ print $2 }'`
        fi
	fi

	return 0
}

reset_wd()
{
    #used in VM only.
	eth=$1
	while true; do 
        ${BPCTL} ${eth} reset_bypass_wd > /dev/null 2>&1
        sleep 1
    done
    return 0
}

start_reset_wd()
{
    #used in VM only.
    nohup /etc/lanbypass.d/silicom-generic.sh reset_wd $1 > /dev/null 2>&1 &
}

main()
{
	action="$1"
	shift

	case ${action} in 
	init)
		init
		;;
	is_bypass)
		is_bypass $@
		;;
	get_slave)
		get_slave $@
		;;
	bypass_now)
		bypass_now $@
		;;
	enable_bypass)
		enable_bypass $@
		;;
	reset_wd)
		reset_wd $@
		;;
	esac

	return $?
}

main $@
exit $?
