#!/usr/bin/env python
# -*- coding: utf-8 -*-

from clfcfg import CLFCfg
from DB import MongoDB
from DB import Sqlite
import clfutil

class StatsEngine(object):
    '''Save and get stats data in DB.'''
    def __init__(self):
        CLFCfg.init()
        self.stats_db= CLFCfg.CLF_STATS_DB_TYPE
        if self.stats_db == CLFCfg.CLF_DB_LOCAL_DISK:
            self.db = LocalStorage()
            self.db.connect(CLFCfg.CLF_STATS_LOCAL_DIK_DIR)
        if self.stats_db == CLFCfg.CLF_DB_MONGODB:
            self.db = MongoDB()
            self.db.connect(CLFCfg.CLF_DB_NAME)
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            self.db = Sqlite()
            self.db.connect(CLFCfg.CLF_DB_NAME)

    #stats functions
    def saveStats(self, companyID, time, stat_name, interval, content, zip):
        self.db.update(CLFCfg.CLF_DB_STATS,
                                [{'col': CLFCfg.CLF_CompanyID, 'op':'eq', 'value': companyID},
                                {'col': CLFCfg.CLF_TimeStamp, 'op': '==', 'value': time},
                                {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': stat_name},
                                {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': interval},
                                {'col': CLFCfg.CLF_StatsZip, 'op': '==', 'value': zip}],
                                {CLFCfg.CLF_Stats: content})

    def findStats(self, companyID, timeBegin, timeEnd, stat_name, interval=3600):
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            return []
        return self.db.find(CLFCfg.CLF_DB_STATS, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': stat_name},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '>=', 'value': timeBegin},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': timeEnd},
                        {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': interval}])

    def findStats2(self, filters, results):
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            return self.db.find2(CLFCfg.CLF_DB_STATS, filters, results)
        else:
            return self.db.find(CLFCfg.CLF_DB_STATS, filters, results)

    def findStatsTime(self,companyID, timeBegin, timeEnd, stat_name,filter,mode):
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            return self.db.find_stats_time(CLFCfg.CLF_DB_STATS,companyID, timeBegin, timeEnd, stat_name,filter,mode)
        else:
            return None

#  findStatsEx will call sqilte API and not call it using other DB solution
    def findStatsEx(self,companyID, timeBegin, timeEnd, stat_name,top_name,filter,ldap_filter, reportType=None, scheduledTime=None):
        return self.db.find(CLFCfg.CLF_DB_STATS,companyID, timeBegin, timeEnd, stat_name,top_name,filter,ldap_filter,reportType, scheduledTime)


    def cleanupStats(self, companyID, expiredTime, filters = []):
        self.db.remove(CLFCfg.CLF_DB_STATS, [
                            {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                            {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': expiredTime}
                        ] + filters)

    #dashboard stats functions
    def saveDashboardStats(self, companyID, time, stat_name, interval, content, zip):
        self.db.update(CLFCfg.CLF_DB_DASHBOARD_STATS,
                                [{'col': CLFCfg.CLF_CompanyID, 'op':'eq', 'value': companyID},
                                {'col': CLFCfg.CLF_TimeStamp, 'op': '==', 'value': time},
                                {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': stat_name},
                                {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': interval},
                                {'col': CLFCfg.CLF_StatsZip, 'op': '==', 'value': zip}],
                                {CLFCfg.CLF_Stats: content})

    def findDashboardStats(self, companyID, timeBegin, timeEnd, stat_name, interval):
        if self.stats_db == CLFCfg.CLF_DB_STATS_SQLITE:
            return self.db.find(CLFCfg.CLF_DB_DASHBOARD_STATS,companyID,timeBegin, timeEnd,stat_name,[],{'dur':interval},None,None)
        else:
            return self.db.find(CLFCfg.CLF_DB_DASHBOARD_STATS, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_StatsName, 'op': 'eq', 'value': stat_name},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '>=', 'value': timeBegin},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': timeEnd},
                        {'col': CLFCfg.CLF_StatsInterval, 'op': '==', 'value': interval}])


    def cleanupDashboardStats(self, companyID, expiredTime):
        self.db.remove(CLFCfg.CLF_DB_DASHBOARD_STATS, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': expiredTime}])

    def getSize(self, companyID='0'):
        if CLFCfg.CLF_STATS_DB_TYPE == CLFCfg.CLF_DB_MONGODB:
            return clfutil.get_path_size(CLFCfg.config.get('db', 'mongodb_path'))
        else:   #sqlite
            return clfutil.get_path_size(CLFCfg.CLF_STATS_STORE_PATH)

if __name__ == '__main__':
    pass
