#!/usr/bin/env python
# -*- coding: utf-8 -*-

from clfcfg import CLFCfg
from DB import MongoDB
from DB import SolrDB
from log_search_engine import LogSearchEngine
from common_id import summary_indx
import clfutil

class SummaryEngine(object):
    '''Save and get summary data in DB.'''
    def __init__(self):
        CLFCfg.init()
        db_type = CLFCfg.CLF_SUMMARY_DB_TYPE
        if db_type == CLFCfg.CLF_DB_SOLR:
            self.db = SolrDB()
            self.db.connect(CLFCfg.CLF_LOG_DB_NAME)
        else:
            self.db = MongoDB()
            self.db.connect(CLFCfg.CLF_DB_NAME)

        #self.log_search_eng = LogSearchEngine()

    def saveSummary(self, summary):
        '''{'cid': 'trend', 'ts': '13400089', 'columnID1':[ 'ID1', 'ID2','ID3'], 'block':['blockID1', 'blockID2'] }'''
        self.db.insert(CLFCfg.CLF_DB_SUMMARY,summary)


    def getSummary(self, companyID, time):
        result = []
        data = self.db.find(CLFCfg.CLF_DB_SUMMARY, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '==', 'value': time}])
        for d in data:
            result.append(d)
        return result

    def findSummaries(self, companyID, filters, timeBegin, timeEnd, start=0, rows=10):
        ft = [{'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
              {'col': CLFCfg.CLF_TimeStamp, 'op': '>=', 'value': timeBegin},
              {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': timeEnd}]
        for f in filters:
            if f['col'] in summary_indx.keys():
                ft.append(f)
        return self.db.find(CLFCfg.CLF_DB_SUMMARY, ft, start, rows)

    def findSummariesEx(self, companyID,facetName,filters, timeBegin, timeEnd, start=0, rows=10, scheduled_time_filter=None):
        return  self.db.findEx(CLFCfg.CLF_DB_SUMMARY, companyID,facetName,filters, timeBegin, timeEnd,start, rows, 'ts desc', scheduled_time_filter)

    def removeSummaries(self, companyID, expiredTime):

        self.db.remove(CLFCfg.CLF_DB_SUMMARY, [
                        {'col': CLFCfg.CLF_CompanyID, 'op': 'eq', 'value': companyID},
                        {'col': CLFCfg.CLF_TimeStamp, 'op': '<', 'value': expiredTime}])

    def getSize(self, companyID='0'):
        if CLFCfg.CLF_SUMMARY_DB_TYPE == CLFCfg.CLF_DB_SOLR:
            return clfutil.get_path_size(CLFCfg.config.get('db', 'solrdb_path'))
        else:   #mongo
            return clfutil.get_path_size(CLFCfg.config.get('db', 'mongodb_path'))

if __name__ == '__main__':
    pass
