#!/bin/sh

# global var:
#  APP_PATCH_ROOT=/etc/iscan
#  PATCHINFOINI=stargate_patch.ini
#  PATCHMGMINI=${INSTALL_DIR}/patch/config/patch_management.ini
#  PATCH_NUMBER=`F_ini_get $PATCHINFOINI PatchInfo Number`

# will be called before database is shutdown.
F_apply_database_changes()
{
    HOST_NAME=localhost
    DATABASE_PORT=5432
    DATABASE_NAME=iwss
    USER_NAME=sa
    PG_BIN=/usr/iwss/PostgreSQL/bin
    SQL_FILE=/tmp/extract_patch/files/tqo_IncrementConsumption.sql

    #extra time quota
	_postmasterNO=`ps -ef | grep postgres | grep -v grep | wc -l`
	if [ $_postmasterNO -ne 0 ] ; then
		if [ $(($_current_build_number)) -lt 1240 ] ; then
			echo "Update IncrementTimeQuotaConsumption"
			ret=` $PG_BIN/psql -h $HOST_NAME -p $DATABASE_PORT -U $USER_NAME -d $DATABASE_NAME -f $SQL_FILE `
		fi
	else
		F_log "Database is stopped, please check postgres process."
		F_log "Cannot apply patch successfully. Exit."
		exit $ERR_APP
	fi

	echo "install app control map"
	MAP_FILE=/tmp/extract_patch/files/app_control_map
    /usr/iwss/PostgreSQL/bin/psql -U sa iwss -c "delete from tb_app_protocol_map; COPY tb_app_protocol_map from '$MAP_FILE' with delimiter '|';"
    /etc/iscan/S99ISappd install

	# for common log database
	/bin/su - iscan -c "/usr/iwss/PostgreSQL/bin/psql -U sa iwss -c \"ALTER TABLE tb_report_template ADD COLUMN scheduled_time_filter integer NOT NULL DEFAULT 0\""
}

# will be called before database is shutdown.
F_rollback_database_changes()
{
	# roll back common log db
	/bin/su - iscan -c "/usr/iwss/PostgreSQL/bin/psql -U sa iwss -c \"ALTER TABLE tb_report_template DROP COLUMN scheduled_time_filter\""
}

F_binary_extra_copy()
{
    #apply Solr schema file
    cp -f solr_schema/schema.xml /etc/iscan/commonlog_solr/solr/collection1/conf/
    chown iscan:iscan /etc/iscan/commonlog_solr/solr/collection1/conf/schema.xml
    chmod 644 /etc/iscan/commonlog_solr/solr/collection1/conf/schema.xml

	cp -f /usr/iwss/ismpeek /usr/iwss/bin/ismpeek
}

F_update_extra_ini() {
	echo "Updating the ini file."

    #disable wmi
    #F_ini_set "/usr/iwss/commonldap/WMISetting.ini" "common" "trans_auth_force_disable"  "yes"
    IWSS_DATA=/var/iwss
    INTSCAN_CONF=$IWSS_DATA/intscan.ini
    URLF_CONF=$IWSS_DATA/urlfcIFX.ini
	HTTP_CONF=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni

	#_UseReqGUID=`F_ini_get $URLF_CONF TMUFE UseReqGUID`
	#if [ $? -ne 0 ] ; then
	#	F_ini_add_key "$URLF_CONF" "TMUFE" "UseReqGUID" "yes"
	#	F_ini_add_key "$URLF_CONF" "TMUFE" "ReqGUID" ""
	#	echo "Updating the GUID config."
	#fi

	#Add this key for CAA issue - TT282825
	#F_ini_add_key "$INTSCAN_CONF" "app-control" "app_control_notification_for_https" "no"
	F_ini_set "$INTSCAN_CONF" "health-monitor" "normal_scan_timeout" "30"
	F_ini_set "$INTSCAN_CONF" "health-monitor" "non_blockio_timeout" "30"
	F_ini_set "$INTSCAN_CONF" "health-monitor" "disk_io_timeout" "30"

    HTTPPNI=$IWSS_DATA/IWSSPIProtocolHttpProxy.pni
    _wccp_mask_value=`F_ini_get $HTTPPNI http wccp_assignment_mask_value`
    if [[ $_wccp_mask_value == 1 ]] ; then
        F_ini_set $HTTPPNI "http" "wccp_assignment_mask_value" "5953"
    fi

	F_ini_add_key_only "$INTSCAN_CONF" "URL-blocking" "access_log_even_server_ip_empty" "no"

	F_ini_add_key_only "$INTSCAN_CONF" "user-identification" "enable_standard_cookie_mode" "no"
	F_ini_add_key "$INTSCAN_CONF" "user-identification" "disable_app_www_authentication" "yes"

	F_ini_add_key_only "$INTSCAN_CONF" "health-monitor" "core_dump" "no"
	F_ini_add_key_only "$INTSCAN_CONF" "bridge" "lanbypass_vm" "no"

	F_ini_add_key_only "$INTSCAN_CONF" "https-scanning" "client_hello_no_host_tunnel" "yes"
	F_ini_add_key_only "$INTSCAN_CONF" "https-scanning" "https_enhance_enable" "no"
	F_ini_add_key_only "$INTSCAN_CONF" "https-scanning" "tunnel_domain" "yes"
	F_ini_add_key_only "$INTSCAN_CONF" "https-scanning" "auto_tunnel_by_failure" "yes"

	F_ini_set "$HTTP_CONF" "http" "tunnel_non_http_traffic" "yes"
	F_ini_add_key_only "$HTTP_CONF" "http" "browser_user_agent" "MSIE,Mozilla,Chrome,Opera,Safari"

	# for squid
	grep "^max_filedesc 65535" /etc/squid/squid.conf > /dev/null || echo "max_filedesc 65535" >> /etc/squid/squid.conf
	grep "^squid soft nofile 65535" /etc/security/limits.conf  > /dev/null || echo "squid soft nofile 65535" >> /etc/security/limits.conf
	grep "^squid hard nofile 65535" /etc/security/limits.conf  > /dev/null || echo "squid hard nofile 65535" >> /etc/security/limits.conf
}

F_apply_cert()
{
	NEW_CER_LIST="
		MD5_2uTyW0ZXZD63F3iARUBwhg__
		MD5_3hp+gELlhrZEjxJDlBfwaA__
		MD5_3RFejlWMTQ1h-lyqXQgusA__
		MD5_6-vvW-roLzaUXePbL2RkVA__
		MD5_b0C01bXCpe24uvHiJ4OecQ__
		MD5_BcpnGCJ+SMC3NxLVVTkM0w__
		MD5_bnr5Tt2sYw1Fcy3vy6R2jQ__
		MD5_Bo6ipXRvQODyDKQHIpKlxg__
		MD5_Bu703KNVFZrkK12Rs4V4iQ__
		MD5_CIWKP3HSHaaNhriR2lA5LA__
		MD5_CjyqIT2jq8LsqqioaecCQg__
		MD5_E1HSS6xashSpEPpAuIeVcA__
		MD5_fjtnMdgp7tK-Rl6JdgHWQQ__
		MD5_iTkdR3VJyKqgH7yUJ1ZQFg__
		MD5_IXhEbJIt6ga42q1WSW1KTA__
		MD5_J7o2xPDpj334bdwwgJXSBA__
		MD5_jWMCAq8Wg8JKrlOtMyRdvw__
		MD5_k6fRARB8HQEjkTMopb36ew__
		MD5_PjgE5g9JIhtIsVCUe390pg__
		MD5_pnMjVYPMOVTCCGUDVmRpJw__
		MD5_QnvAOHDYDYkHaTKc-FPQ9Q__
		MD5_rnmX2ngLoYP7mZR9ehlesA__
		MD5_uSyoiNaxSxucY44fUuE4Nw__
		MD5_uvOMLgg2fr+Yib6nGYZwKQ__
		MD5_vjS3k9-9JTiSbnHZV7NCTg__
		MD5_VrR6BPpY6vtHSQ1A-+238g__
		MD5_wO7vE4WqEUxUHkN4TOLgLg__
		MD5_WXioQ-Xv-czjzrQJ0qY1Mw__
		MD5_zGng23iycvpsvSBG9Jl-8g__
				"

	CERPROP=/etc/iscan/jscan/certificates/certstatus.props

	sed -i -e :a -e '/^\n*$/{$d;N;};/\n$/ba' $CERPROP
	for cer in $NEW_CER_LIST; do
		grep $cer $CERPROP > /dev/null 2>&1

		if [ $? -ne 0 ]; then
			echo "${cer}.act = active" >> $CERPROP
		fi
	done

	rm -f /etc/iscan/https/certificates/pem/*.*
	rm -f /etc/iscan/https/certificates/active/*.*
	rm -f /etc/iscan/https/certificates/inactive/*.*

	su iscan /usr/iwss/certsynctool/certsync_tohttpsdir.sh > /dev/null 2>&1
	su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/active /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/active
	su iscan /usr/iwss/CA_certs_transhash.sh /etc/iscan/https/certificates/sync/inactive /etc/iscan/https/certificates/pem /etc/iscan/https/certificates/inactive
}

# will be called after binary copying
F_apply_extra_changes()
{
	echo F_apply_extra_changes

	F_binary_extra_copy
	F_update_extra_ini
	F_apply_cert

    echo "15 2 * * * /usr/iwss/bin/auto_import_ca.sh > /dev/null 2>&1" >> /var/spool/cron/root
}

# will be called after binary copying
F_rollback_extra_changes()
{
	echo F_rollback_extra_changes

	cp -f /usr/iwss/ismpeek /usr/iwss/bin/ismpeek

    sed -i "/auto_import_ca/d" /var/spool/cron/root
}
